<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 34fe5be02d62 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-11-17 15:38 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
define('WE_DOCUMENT_ROOT', (string) realpath(__DIR__ . '/../'));
require_once('we/include/we.inc.php');
//FIXME: hooks for changing username

/**
 * Display password reset form, asking for username and e-mail.
 * Note: CSRF token is added automatically.
 */
function defaultReset(): string{
	$_SESSION['resetTok'] = md5(uniqid(__FILE__, true));
	return we_html_element::div(['style' => 'float: left;height: 50%;width: 1px;']) . we_html_element::div(['style' => 'clear:left;position:relative;top:-25%;'], we_html_element::form([
						'action' => WEBEDITION_DIR . 'resetpwd.php', 'method' => 'post'], '
	<table id="mainTable">
		<tr><td colspan="2">' . we_html_element::h(2, g_l('global', '[changePass]')) . '</td></tr>
		<tr><td>' . g_l('global', '[username]') . '</td><td>' . we_html_element::input(['type' => 'text', 'name' => 's[username]', 'required' => 'required', 'placeholder' => g_l('global', '[username]')]) . '</td></tr>
		<tr><td>' . g_l('modules_users', '[email]') . '</td><td>' . we_html_element::input(['type' => "email", 'name' => 's[Email]', 'required' => "required", 'placeholder' => g_l('modules_users', '[email]')]) . '</td></tr>
		<tr><td colspan="2"></td><td>' . we_html_button::create_button(we_html_button::OK, "javascript:submit();") . '</td></tr>
	</table>' .
							we_html_element::hiddens([
								'type' => 'mail',
								'resetTok' => $_SESSION['resetTok']
							])
	));
}

/**
 * Display a form to set a new password for the given username.
 * Note: CSRF token is added automatically.
 */
function resetPwd(): string{
	$uid = we_base_request::int('user');
	$token = we_base_request::string('token', '');
	$_SESSION['resetTok'] = md5(uniqid(__FILE__, true));
	return we_html_element::div(['style' => 'float: left;height: 50%;width: 1px;']) . we_html_element::div(['style' => 'clear:left;position:relative;top:-25%;'], we_html_element::form([
						'action' => WEBEDITION_DIR . 'resetpwd.php', 'method' => 'post'], '
	<table id="mainTable">
		<tr><td colspan="2">' . we_html_element::h(2, g_l('global', '[changePass]')) . '</td></tr>
		<tr><td colspan="2" style="padding:1em 0;">' . SECURITY_USER_PASS_DESC . '</td></tr>
		<tr><td>' . g_l('global', '[newPass]') . '</td><td>' . we_html_element::input(['type' => 'password', 'name' => 's[Password]', 'oninput' => "comparePwd('s[Password]','s[Password2]')",
								'placeholder' => g_l('global', '[newPass]')]) . '</td></tr>
		<tr><td>' . g_l('global', '[newPass2]') . '</td><td>' . we_html_element::input(['type' => 'password', 'name' => 's[Password2]', 'oninput' => "comparePwd('s[Password]','s[Password2]')",
								'placeholder' => g_l('global', '[newPass2]')]) . '</td></tr>
		<tr><td colspan="2"></td><td>' . we_html_button::create_button(we_html_button::SAVE, "javascript:submit();") . '</td></tr>
	</table>' .
							we_html_element::hiddens([
								'type' => 'mailreset',
								'user' => $uid,
								'token' => $token,
								'resetTok' => $_SESSION['resetTok']
							])
	));
}

/**
 * Display error message.
 * @param  String $txt The error message to be displayed.
 */
function showError(string $txt): string{
	return we_html_element::div(['class' => 'error'], we_html_element::div([], $txt));
}

$body = '';
switch(we_base_request::string('type', '')){
	case 'mailreset':
		$resetTok = we_base_request::string('resetTok');
		$uid = we_base_request::int('user');
		if(!$resetTok || empty($_SESSION['resetTok']) || $resetTok != $_SESSION['resetTok']){
			$body .= showError(g_l('global', '[CSRF][tokenInvalid]'));
			break;
		}
		$data = getHash('SELECT ID,password FROM ' . PWDRESET_TABLE . ' WHERE UserTable="tblUser" AND ID=' . $uid . ' AND token="' . DB_WE::$inst->escape(we_base_request::string('token')) . '"');
		$newPwd = mb_substr((string) we_base_request::_(we_base_request::RAW_CHECKED, 's', '', 'Password'), 0, we_users_user::MAX_PWD_LEN) ?: '';
		$body .= we_weTag_util::tag('customerResetPassword', ['type' => 'resetFromMail', 'passwordRule' => SECURITY_USER_PASS_REGEX], '', true);

		if(we_weTag_util::tag('ifNotCustomerResetPassword')){
			$body .= showError(g_l('global', '[pwd][changeFailed]') . '<br/>');
			if(we_weTag_util::tag('ifNotCustomerResetPassword', ['type' => 'passwordMismatch'])){
				$body .= showError(g_l('global', '[pass_not_confirmed]'));
			} elseif(strlen($newPwd) < 6 || we_weTag_util::tag('ifNotCustomerResetPassword', ['type' => 'passwordRule'])){
				$body .= showError(g_l('global', '[pass_to_short]'));
			} else if(we_weTag_util::tag('ifNotCustomerResetPassword', ['type' => 'token'])){
				$body .= showError(g_l('global', '[pwd][invalidToken]'));
				unset($_REQUEST['user']);
				$body .= defaultReset();
				break;
			}
			$body .= resetPwd();
		} else {
			$hook = new we_hook_handler('user_Login', [
				'user' => getHash('SELECT * FROM ' . USER_TABLE . ' WHERE ID=' . $data['ID']),
				'password' => $newPwd,
				'type' => 'resetPassword'
			]);

			$hookRet = $hook->executeHook();

			$body .= we_html_element::div(['style' => 'float: left;height: 50%;width: 1px;']) .
					we_html_element::div(['style' => 'clear:left;position:relative;top:-25%;'], we_html_element::form(['action' => WEBEDITION_DIR . 'resetpwd.php', 'method' => 'post'], '
					<table id="mainTable">
						<tr><td colspan="2">' . we_html_element::h(2, ($hookRet ? g_l('global', '[pass_changed]') : $hook->getErrorString())) . '</td></tr>
						<tr><td></td><td></td><td>' . we_html_button::create_button('back_to_login', WEBEDITION_DIR . 'index.php') . '</td></tr>
					</table>
				'));
		}
		break;

	default:
		if(($uid = we_base_request::int('user')) && ($token = we_base_request::string('token', ''))){
			$body .= resetPwd();
		} else {
			$body .= defaultReset();
		}
		break;

	case 'mail':
		$resetTok = we_base_request::string('resetTok');
		if(!$resetTok || empty($_SESSION['resetTok']) || $resetTok != $_SESSION['resetTok']){
			$body .= showError(g_l('global', '[CSRF][tokenInvalid]'));
			break;
		}
		$body .= we_weTag_util::tag('customerResetPassword', ['type' => 'email', 'required' => 'username,Email', 'customerEmailField' => 'Email', 'loadFields' => 'First,Second,username'], '', true);

		if(we_weTag_util::tag('ifNotCustomerResetPassword')){
			$body .= showError(g_l('global', '[pwd][changeFailed]') . '<br/>');
			if(we_weTag_util::tag('ifNotCustomerResetPassword', ['type' => 'userNotExists'])){
				$body .= showError(g_l('global', '[pwd][noSuchUser]'));
			}
			$body .= defaultReset();
		} else {

			if(we_mail($_SESSION['webuser']['Email'], g_l('global', '[pwd][mailSubject]'), $_SESSION['webuser']['First'] . ' ' . $_SESSION['webuser']['Second'] . ' (' . $_SESSION['webuser']['username'] . '),
' . sprintf(g_l('global', '[pwd][resetMail]'), we_http_util::getServerUrl()) . "\n" .
							we_weTag_util::tag('customerResetPasswordLink', ['plain' => true], '', true)
					)){
				$body .= showError(sprintf(g_l('global', '[pwd][mailSent]'), $_SESSION['webuser']['Email']));
			} else {
				$body .= showError(sprintf(g_l('modules_newsletter', '[mail_not_sent]'), $_SESSION['webuser']['Email']));
			}

			unset($_SESSION['webuser']);
		}
		break;
}

echo we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'loginScreen.css') .
		we_html_element::jsScript(JS_DIR . 'weFrontendEdit_header.js') .
		we_html_element::jsScript(JS_DIR . 'comparePwd.js', '', ['id' => 'loadVarComparePwd', 'data-passwd' => setDynamicVar([
				'pwdCheck' => SECURITY_USER_PASS_REGEX
	])]), we_html_element::body(['id' => 'loginScreen'], $body)
);
