<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 883c0197a34e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-08-03 19:59 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* * New Format for classes: we_Domain_specific
 * 1. ALL classes start with we_
 * 2. ALL files in one domain
 * 3. all files of one domain reside in the same directory
 * 4. if further division is needed, create a subdomain which does not contain _ !
 * 5. domains are looked up in this class
 * 6. NOTE that files MUST end with .class.php
 */

abstract class we_base_autoloader{
	//fallback classes if local classes do not exist - mostly pear
	private const FALLBACK = [
		'DB_WE' => 'we/classes/database/we_database_mysqli.class.php',
		'Image_IPTC' => 'we/libraries/pear/Image_IPTC.class.php',
		'Less_Parser' => 'we/libraries/Less/Parser.php',
		'PHPGangsta_GoogleAuthenticator' => 'we/libraries/googleAuthenticator/googleAuthenticator.class.php',
		'lessc' => 'we/libraries/Less/lessc.inc.php',
		//add upgrade classes
		//'' => 'we/include/we_classes/update/we_update_upgrade.class.php',
	];

	/** @phan-var array<int,array<string,array{0:string,1:string}>> first int notes how many namespace parts should be stripped of */
	private static $nameSpaces = [
		'1' => [
			'Sabre' => [WE_LIB_PATH . 'sabre', '.php'],
			'Psr' => [WE_LIB_PATH . 'Psr', '.php'],
		],
		'0' => [
			'' => [PHPCLASS_PATH, '.class.php'], //fallback
		],
	];
	private static $registeredUsrClasses = false;
	private static $registeredFinal = false;

	public static function prefixLoader(string $class_name): bool{
		$tmp = explode('_', $class_name, 3);
		@[$where, $domain] = count($tmp) > 1 ? $tmp : [$tmp[0], ''];
		switch($where){
			case 'we':
				if(file_exists(($file = WE_CLASS_PATH . $domain . '/' . $class_name . '.class.php'))){
					include($file);
					return true;
				}
				switch($domain){
					case 'tag':
						if(file_exists(($file = WE_CUSTOM_PATH . '/' . we_weTag_tag::DIR . '/' . $class_name . '.class.php'))){//special handling of custom tags
							include($file);
							return true;
						}
						return false;
					case 'hook':
						if(file_exists(($file = WE_CUSTOM_PATH . '/' . we_hook_handler::DIR . '/' . $class_name . '.class.php'))){//special handling of custom hooks
							include($file);
							return true;
						}
				}
				return false;
			case 'welu':
				if(file_exists(($file = WEBEDITION_PATH . 'liveUpdate/classes/' . $domain . '/' . $class_name . '.class.php'))){
					include($file);
					return true;
				}
				return false;
			case 'Horde':
				include(WE_LIB_PATH . str_replace('_', '/', $class_name) . '.php');
				return true;
			case 'cs'://user extensions
				$domain = explode('_', $class_name);
				array_shift($domain);
				$filename = array_pop($domain);
				if(file_exists(($file = PHPCLASS_PATH . '/' . implode('/', $domain) . '/' . $class_name . '.class.php')) || //@phan-suppress-current-line PhanUnusedVariable
					file_exists(($file = PHPCLASS_PATH . '/' . implode('/', $domain) . '/' . $filename . '.class.php'))){
					include($file);
					return true;
				}
				return false;
		}
		return false;
	}

	public static function nameSpaceLoader(string $class_name): bool{
		if(!self::$registeredUsrClasses){
			self::$registeredUsrClasses = true;
			include_once(PHPCLASS_PATH . '/autoload.php');
			krsort(self::$nameSpaces, SORT_NUMERIC);
		}
		$fullDomain = explode('\\', $class_name);
		foreach(self::$nameSpaces as $no => $vals){
			$domain = $fullDomain;
			$prefix = implode('\\', array_splice($domain, 0, (int) $no));
			if(array_key_exists($prefix, $vals)){
				$key = $vals[$prefix];
				if(file_exists(($file = $key[0] . '/' . implode('/', $domain) . $key[1]))){
					include($file);
					return true;
				}/*else{
					t_e($file);
				}*/
			}
		}
		return false;
	}

	/**
	 * default webEdition autoloader
	 * @param string $class_name
	 */
	public static function autoload(string $class_name): void{
		if(str_contains($class_name, '\\') ?
				self::nameSpaceLoader($class_name) :
				self::prefixLoader($class_name)
		){
			return;
		}

		// add this loader at the end, if class was not yet found
		if(!self::$registeredFinal){
			self::$registeredFinal = true;
			spl_autoload_register('we_base_autoloader::finalLoad', true);
		}
		//will try next auto-loader, if present - else finalload is called
	}

	public static function registerNameSpace(string $nameSpace, string $trgDir, string $ext = '.class.php', string $prefixPath = PHPCLASS_PATH): void{
		if(!$nameSpace){
			return;
		}
		$ns = explode('\\', $nameSpace);
		self::$nameSpaces[count($ns)][$nameSpace] = [$prefixPath . '/' . $trgDir, $ext];
	}

	/**
	 * Added after last loader to trigger not found classes
	 * @param string $class_name
	 */
	public static function finalLoad(string $class_name): void{
		if(array_key_exists($class_name, self::FALLBACK)){
			include(WEBEDITION_PATH . self::FALLBACK[$class_name]);
			return;
		}
	}

}
