<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e6c2be15eff4 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-12-12 01:00 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*
  data of variations have the following format in document

  we_doc->elements[WE_VARIANTS_ELEMENT_NAME] =
  [0] =>
  'VARIATIONNAME1' =>
  'fieldName1' =>
  'type' = 'txt',
  'dat' = 'Text'
  ),
  'fieldName2' =>
  'type' = 'img',
  'dat' = 152
  )
  ),
  [1] =>
  'VARIATIONNAME2' =>
  'fieldName1' =>
  'type' = 'txt',
  'dat' = 'CU'
  ),
  'fieldName2' =>
  'type' = 'img',
  'dat' = 155
  )
  )
  )
  )
  =====>>

  in editmode available in document
  we_doc->elements[WE_VARIANTS_PREFIX . '0'] ='type' = 'txt', 'dat' = 'VARIATIONNAME1');
  we_doc->elements[WE_VARIANTS_PREFIX . '0' . '_' . fieldName1] ='type' = 'txt', 'dat' = 'Text');
  we_doc->elements[WE_VARIANTS_PREFIX . '0' . '_' . fieldName2] ='type' = 'img', 'dat' = 152);

  we_doc->elements[WE_VARIANTS_PREFIX . '1'] ='type' = 'txt', 'dat' = 'VARIATIONNAME2');
  we_doc->elements[WE_VARIANTS_PREFIX . '1' . '_' . fieldName1] ='type' = 'txt', 'dat' = 'CU');
  ...
 */

abstract class we_base_variants{
	const PREFIX = 'we__intern_variant___';
	const ELEMENT_NAME = 'weInternVariantElement';
	const ELEMENT_NAME_ARRAY = 'weInternVariantElement';//weInternVariantElementA
	const REQUEST = 'we_variant';
	const VARIANT_TAGS = ['input', 'link', 'textarea', 'img', 'select'];
	const ALLVARIANT_FIELD = 'allVariants';//FIXME: what is this?

	/**
	 * Searchs all elements of document/object
	 * fetches all variation-data in one single field
	 * and deletes all other fields
	 * when not save, the field is resettet for the editor
	 *
	 * @param we_document_base $model
	 */
	public static function correctModelFields(we_document_base $model): void{//FIXME: is this still relevant???
		$elements = $model->elements;

		// all variant fields must be stored in one single field of the content table
		// store variationfields in one array
		$variationElements = [];
		'@phan-var array<string,array> $variationElements';

		foreach($elements as $element => $elemArr){
			if(str_contains((string) $element, self::PREFIX)){

				//since delete might have deleted this instance, check if this id is still set
				[$pos] = explode('_', (string) (substr($element, strlen(self::PREFIX) ?: 0, 2) ?: 0));
				if(isset($model->elements[self::ELEMENT_NAME_ARRAY]['dat'][$pos])){
					//only add elements which are not deleted
					$variationElements[$element] = $elemArr;
				}
			}
		}
		// :ATTENTION: if nr of variants is > 10 a ksort of the elements is not
		// enough to build blocks of data of a single variant.
		ksort($variationElements);

		$variationElement = [];
		$nameOfPosition = [];

		// :ATTENTION: if nr of variants is > 10 a ksort of the elements is not
		// enough to build blocks of data of a single variant.
		foreach($variationElements as $element => $data){
			'@phan-var string $element';
			$elemNr = self::getNrFromElemName($element);
			if(!isset($nameOfPosition['nameof_' . $elemNr])){
				$nameOfPosition['nameof_' . $elemNr] = $data['dat'];
				$variationElement[$elemNr][$nameOfPosition['nameof_' . $elemNr]] = [];
			} else {
				$fieldName = self::getFieldNameFromElemName($element);
				$variationElement[$elemNr][$nameOfPosition['nameof_' . $elemNr]][$fieldName] = $data;
			}
		}

		// now create element for the model
		// just overwrite new values ...
		$model->setElementA(self::ELEMENT_NAME_ARRAY, $variationElement, 'variant');
	}

	/**
	 * this function is reverse function to correctModelFields
	 * initialises variant data in the model and stores them in special fields
	 * @param we_document_base $model
	 */
	public static function setVariantDataForModel(we_document_base $model): void{
		$variations = $model->getElementA(self::ELEMENT_NAME_ARRAY, [], true); //Fix #9349
		if(!$variations){
			return;
		}
		foreach($variations as $i => $variation){
			if(!is_array($variation)){
				continue;
			}

			foreach($variation as $name => $varArr){
				$model->elements[self::PREFIX . $i] = [
					'type' => 'txt',
					'dat' => $name
				];

				foreach($varArr as $n => $datArr){
					$model->elements[self::PREFIX . $i . '_' . $n] = $datArr;
				}
			}
		}
	}

	private static function getNrFromElemName(string $elemName): int{
		return (int) (preg_replace('/_(.*)/', '', substr($elemName, strlen(self::PREFIX)) ?: ''));
	}

	private static function getFieldNameFromElemName(string $elemName): string{
		$fieldNameTmp = substr($elemName, strlen(self::PREFIX)) ?: '';
		$fieldName = preg_replace('/(\d+_*)/', '', $fieldNameTmp, 1);

		return ($fieldNameTmp == $fieldName ? '' : $fieldName);
	}

	public static function getNumberOfVariants(we_document_base $model): int{
		return count($model->getElementA(self::ELEMENT_NAME_ARRAY));
	}

	private static function insertVariant(we_document_base $model, int $position): void{
		$amount = we_base_variants::getNumberOfVariants($model);

		// init model->elements if neccessary

		$el = $model->getElementA(self::ELEMENT_NAME_ARRAY) ?: [];

		// add new element at end of array, move it when neccesary
		$el[] = self::createNewVariantElement(/* $model */);
		$model->setElementA(self::ELEMENT_NAME_ARRAY, $el, 'variant');

		// now move element, it is actually at last position
		if($amount > $position){ // move all elements
			$newElemPos = $amount;
			while($position < $newElemPos){
				self::changeVariantPosition($newElemPos, --$newElemPos, $model);
			}
		}
	}

	private static function createNewVariantElement(/* we_base_mainmodel $model */): array{

		// :TODO: improve me
		return [];
	}

	/**
	 *
	 * @param we_document_base $model
	 * @param int $pos
	 * @return array<string,array>
	 */
	public static function getAllVariationFields(we_document_base $model, int $pos = -1): array{
		$elements = $model->elements;
		$variationElements = [];

		foreach($elements as $element => $elemArr){
			if(str_contains((string) $element, self::PREFIX)){
				$variationElements[$element] = $elemArr;
			}
		}
		ksort($variationElements);

		if($pos === -1){
			return $variationElements;
		}
		foreach(array_keys($variationElements) as $name){
			if(self::getNrFromElemName($name) != $pos){
				unset($variationElements[$name]);
			}
		}
		return $variationElements;
	}

	private static function moveVariant(we_document_base $model, int $pos, string $direction): void{
		// check if a move is possible
		self::changeVariantPosition($pos, ($pos + ($direction === 'up' ? -1 : 1)), $model);
	}

	/**
	 * @param integer $pos1
	 * @param integer $pos2
	 * @param we_document_base $model
	 */
	private static function changeVariantPosition(int $pos1, int $pos2, we_document_base $model): void{
		// first move all fields in the $modell
		$elements = $model->getElementA(self::ELEMENT_NAME_ARRAY);

		$tmp = $elements[$pos1];
		$elements[$pos1] = $elements[$pos2];
		$elements[$pos2] = $tmp;
		$model->setElementA(self::ELEMENT_NAME_ARRAY, $elements, 'variant');
		// move elements for editmode
		$variationElements_1 = we_base_variants::getAllVariationFields($model, $pos1);
		$variationElements_2 = we_base_variants::getAllVariationFields($model, $pos2);

		// backup pos 1
		$tmp = [];
		foreach($variationElements_1 as $name => $arr){
			$tmp[$name] = $arr;
			$model->delElement($name);
		}

		// overwrite pos 1 with pos 2
		foreach(array_keys($variationElements_2) as $name){
			$model->elements[self::getNameForPosition($name, $pos1)] = $model->elements[$name];
			$model->delElement($name);
		}

		// restore pos 1 to pos2
		foreach(array_keys($tmp) as $name){
			$model->elements[self::getNameForPosition($name, $pos2)] = $tmp[$name];
		}
		// delete backup
		unset($tmp);
	}

	private static function getNameForPosition(string $name, int $pos): string{
		return self::PREFIX . $pos .
			(self::getFieldNameFromElemName($name) ? '_' . self::getFieldNameFromElemName($name) : '');
	}

	private static function removeVariant(we_document_base $model, int $delPos): void{
		$total = we_base_variants::getNumberOfVariants($model);

		$lastPos = $total - 1;
		// move at last position, then remove it
		while($delPos < $lastPos){
			self::moveVariant($model, $delPos++, 'down');
		}
		// first remove all fields from doc
		$variationFields = we_base_variants::getAllVariationFields($model, $delPos);
		foreach(array_keys($variationFields) as $name){
			$model->delElement($name);
		}
		if(is_array($model->elements[self::ELEMENT_NAME_ARRAY]['dat'][$delPos])){
			unset($model->elements[self::ELEMENT_NAME_ARRAY]['dat'][$delPos]);
		}
	}

	public static function getVariantsEditorMultiBoxArrayObjectFile(we_base_jsCmd $jsCmd, we_contents_objectFile $model): we_html_multiIconBoxContent{
		$variantFields = $model->getVariantFields();
		$count = we_base_variants::getNumberOfVariants($model);
		$parts = new we_html_multiIconBoxContent();

		if($count){
			$tableInfo_sorted = we_contents_objectFile::getSortedTableInfo($model->TableID, true, DB_WE::$inst);
			$fields = $regs = [];
			foreach($tableInfo_sorted as $cur){
				if(in_array($cur['name'], ['OF_ID', 'OF_DocumentTable'], true)){
					continue;
				}
				if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
					$fields[$regs[2]] = $cur;
				}
			}
			for($i = 0; $i < $count; $i++){
				$name = self::PREFIX . $i;
				$field = we_html_element::div(['class' => 'weObjectContent'], $model->getFieldHTML($jsCmd, self::PREFIX . $i, 'input', [], [], true));
				$content = we_html_element::div(['id' => $name, 'class' => 'objectFileElement'],
																		we_html_element::div(['id' => 'f' . $name, 'class' => ''], $field)
				);

				foreach($variantFields as $realName => $attributes){
					[$type, $realname] = explode('_', $realName, 2);
					$name = self::PREFIX . $i . '_' . $realname;
					$field = we_html_element::div(['class' => 'weObjectContent'], $model->getFieldHTML($jsCmd, $name, $type, $attributes, $fields[$realname], true));
					$content .= we_html_element::div(['id' => $type . '_' . $name, 'class' => 'objectFileElement'],
																			we_html_element::div(['id' => 'f' . $name, 'class' => ''], $field)
					);
					$jsCmd->addCmd('orderContainerAdd', $type . '_' . $name, true);
				}

				$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_shop', '[variant]') . ' ' . ($i + 1) .
						we_html_element::div(['class' => 'addTrashUpDown'], we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:we_cmd('insert_variant','" . $i . "');") . we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('remove_variant','" . $i . "');") .
							we_html_element::div(['class' => 'UpDown'], ($i == 0 ? we_html_button::create_button(we_html_button::DIRUP, '', '', '', true) : we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('move_variant_up','" . $i . "');")) .
								($i == ($count - 1) ? we_html_button::create_button(we_html_button::DIRDOWN, '', '', '', true) : we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('move_variant_down','" . $i . "');"))
						)), $content))->space(we_html_multiIconBox::SPACE_MED2));
			}
		} else {
			$i = 0;
		}

		$parts->add((new we_html_multiIconBoxContentLine(we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:we_cmd('insert_variant','" . $i . "');"), ''))->space(we_html_multiIconBox::SPACE_MED2));
		return $parts;
	}

	public static function getVariantsEditorMultiBoxArray(we_document_base $model): we_html_multiIconBoxContent{
		$variationFields = $model->getVariantFields();
		$count = we_base_variants::getNumberOfVariants($model);
		$parts = new we_html_multiIconBoxContent();

		if($count){
			for($i = 0; $i < $count; $i++){
				$content = '<table class="lowContrast" style="width:100%;">
<tr><td class="bold">Name</td></tr>
<tr><td>' . $model->formInput('input', self::PREFIX . $i) . '</td></tr>';

				foreach($variationFields as $name => $fieldInformation){
					$fieldInformation['attributes']['name'] = self::PREFIX . $i . '_' . $fieldInformation['attributes']['name'];
					$content .= '<tr><td><b>' . $name . '</b></td></tr>
	<tr><td>' . we_weTag_util::tag($fieldInformation['type'], $fieldInformation['attributes'], ($fieldInformation['content'] ?? '')) . '</td></tr>
	<tr>';
				}
				$content .= '</table>';

				$variantName = $model->getElementS(self::PREFIX . $i);
				$parts->add((new we_html_multiIconBoxContentLine(($variantName ?: g_l('modules_shop', '[variant]') . ' ' . ($i + 1)) .
						we_html_element::div(['class' => 'addTrashUpDown'], we_html_element::div([], we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:we_cmd('insert_variant','" . $i . "');") .
								we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('remove_variant','" . $i . "');")) .
							we_html_element::div(['class' => 'UpDown'], ($i === 0 ? we_html_button::create_button(we_html_button::DIRUP, '', '', '', true) : we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('move_variant_up','" . $i . "');")) .
								($i === ($count - 1) ? we_html_button::create_button(we_html_button::DIRDOWN, '', '', '', true) : we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('move_variant_down','" . $i . "');"))) .
							we_html_element::div(['class' => 'UpDown'], we_html_button::create_button(we_html_button::VIEW, "javascript:we_cmd('preview_variant','" . we_base_WE::$transaction . "','" . $variantName . "');")
							)
						), $content))->space(we_html_multiIconBox::SPACE_MED));
			}
		} else {
			$i = 0;
		}

		$parts->add((new we_html_multiIconBoxContentLine(we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:we_cmd('insert_variant','" . $i . "');"), ''))->space(we_html_multiIconBox::SPACE_MED2));
		return $parts;
	}

	public static function useVariant(we_document_base $model, string $name): void{
		$variantDatArray = $model->getElementA(self::ELEMENT_NAME_ARRAY);

		$model->Variant = $name;

		foreach($variantDatArray as $variant){
			if(!is_array($variant)){
				continue;
			}
			if(isset($variant[$name])){//can we break if one is found??
				$variantData = $variant[$name];
				foreach($variantData as $elementName => $elementData){
					$model->elements[$elementName] = $elementData;
				}
			}
		}
	}

	public static function stripVariantPrefix(string $name = ''): string{
		if(!str_starts_with($name, we_base_variants::PREFIX)){
			return $name;
		}
		$parts = explode('_', str_replace(we_base_variants::PREFIX, '', $name), 2);

		return $parts[1] ?? 'Name';
	}

	/**
	 * This function sets variant data for serialised document in the shopping basket
	 * different function, due to performance reasons and the shop itself
	 *
	 * @param array<string,int|string> $record
	 * @param string $name
	 */
	public static function useVariantForShop(array &$record, string $name): void{
		if(!isset($record[self::ELEMENT_NAME])){
			return;
		}
		$variantDatArray = we_unserialize((string) $record[self::ELEMENT_NAME]);

		foreach($variantDatArray as $variant){
			foreach($variant as $variantName => $variantData){
				if($variantName == $name){
					foreach($variantData as $elementName => $elementData){
						$record[$elementName] = ($elementData['type'] === 'img' ? $elementData[we_contents_base::ETYPE_NUMBER] : $elementData[we_contents_base::ETYPE_TEXT]);
					}
				}
			}
		}
	}

	/**
	 * This function sets variant data for serialised object in the shopping basket
	 * different function, due to performance reasons and the shop itself
	 *
	 * @param array<string,int|string> $record
	 * @param string $name
	 * @param we_contents_objectFile $model
	 * @return array<string,mixed>
	 */
	public static function useVariantForShopObject(array $record, string $name, we_contents_objectFile $model): array{
		if(!$model->issetElement(self::ELEMENT_NAME_ARRAY)){
			return $record;
		}
		$variantDatArray = $model->getElementA(self::ELEMENT_NAME_ARRAY);

		foreach($variantDatArray as $variant){
			foreach($variant as $variantName => $variantData){
				if($variantName == $name){
					foreach($variantData as $elementName => $elementData){
						// fields have the prefix we_
						$record[$elementName] = ($elementData['type'] === 'img' ? ($elementData[we_contents_base::ETYPE_NUMBER] ?? '') : ($elementData[we_contents_base::ETYPE_TEXT] ?? ''));
					}
				}
			}
		}
		return $record;
	}

	/**
	 *
	 * @param we_document_base $model
	 * @param string $defaultname
	 * @return array<string,mixed>
	 */
	public static function getVariantData(we_document_base $model, string $defaultname): array{
		if(!$model->issetElement(self::ELEMENT_NAME_ARRAY)){
			return [];
		}
		$elements = $model->getElementA(self::ELEMENT_NAME_ARRAY, [], true);
		// add default data to listview
		//this elemets contains only the variant fields, not the non-variant fields of the object

		$newPos = count($elements);
		$elementF = [];

		if($newPos > 0){
			$elemdata = $elements[0];
			if(is_array($elemdata) && $defaultname != ''){
				$noFirst = (!str_contains($defaultname, 'FIRST'));
				foreach($elemdata as $varArr){
					foreach($varArr as $key => $fieldArr){
						if(isset($model->elements[$key])){
							if($noFirst){
								$elements[$newPos][$defaultname][$key] = $model->elements[$key];
							} else {
								$elementF[$defaultname][$key] = $model->elements[$key];
							}
						}
					}
				}
				if(!$noFirst){
					array_unshift($elements, $elementF);
				}
			}
		}
		// attempt to add the other fields
		$modelElements = $model->elements; //get a copy of the non variant fields
		unset($modelElements[self::ELEMENT_NAME_ARRAY]); // get rid of some keys
		foreach(array_keys($modelElements) as $key){
			if(str_starts_with($key, self::PREFIX)){
				unset($modelElements[$key]);
			}
		}
		if($newPos > 0 && $elements){ //Fix #6883 - not sure if this has an impact
			foreach($elements as &$varArr){//now add the elements
				foreach($varArr as &$fieldArr){
					$fieldArr = array_merge($modelElements, $fieldArr);
				}
			}
		}
		//
		return $elements;
	}

	/**
	 *
	 * @param we_base_jsCmd $jsCmd
	 * @param string $command
	 * @param we_document_base|we_contents_objectFile $we_doc
	 * @return string
	 */
	public static function edit(we_base_jsCmd $jsCmd, string $command, we_document_base $we_doc): string{
		switch($command){
			case 'insert_variant':
				self::insertVariant($we_doc, we_base_request::int('we_cmd', 0, 1));
				break;
			case 'move_variant_up':
				self::moveVariant($we_doc, we_base_request::int('we_cmd', 0, 1), 'up');
				break;
			case 'move_variant_down':
				self::moveVariant($we_doc, we_base_request::int('we_cmd', 0, 1), 'down');
				break;
			case 'remove_variant':
				self::removeVariant($we_doc, we_base_request::int('we_cmd', 0, 1));
				break;
			case 'preview_variant':
				self::correctModelFields($we_doc);
				self::useVariant($we_doc, we_base_request::string('we_cmd', '', 2));
				echo (new we_dialog_previewVariant($we_doc->getDocument()))->getHTML();
				exit;
		}
		$GLOBALS['we_editmode'] = we_base_WE::$editmode = true;

		return (new we_html_multiIconBox(($we_doc instanceof we_contents_objectFile ? self::getVariantsEditorMultiBoxArrayObjectFile($jsCmd, $we_doc) : self::getVariantsEditorMultiBoxArray($we_doc))))->marginLeft()->get() .
			($we_doc instanceof we_document_webEdition && we_wysiwyg_editor::isWysiwygInstances() ? we_wysiwyg_editor::getHTMLConfigurationsTag() : '');
	}

	public static function getTemplateCode(we_contents_template $we_doc): string{
		$fields = $we_doc->getVariantFieldNames();

		$content = [];
		foreach($fields as $field){
			$element = $we_doc->getElementS('variant_' . $field);
			$content[] = [we_html_forms::checkboxWithHidden($element ? true : false, 'we_' . $we_doc->Name . '_variant[variant_' . $field . ']', (string) $field, false, 'middlefont', '_EditorFrame.setEditorIsHot(true);')
			];
		}

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('weClass', '[variant_info]'), we_html_tools::TYPE_INFO, 620, false)))->noline())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::dialogBorder(600, $content, [g_l('weClass', '[variant_fields]')])))->noline());

		return we_contents_base::hiddenTrans() .
			(new we_html_multiIconBox($parts, 'template_variant'))->marginLeft()->get();
	}

}
