<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b3427f66048b $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-03-31 12:31 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_cache_backendDB implements we_cache_backend{

	private const PACK_TYPE_PLAIN = 'plain';
	private const PACK_TYPE_PACKED = 'packed';
	private const PACK_TYPE_SERIALIZED = 'serialized';

	public function __construct(){
		DB_WE::$inst->query('DELETE FROM ' . DB_CACHE_TABLE . ' WHERE valid<NOW()');
	}

	public function clean(string $pattern = ''): bool{
		DB_WE::$inst->query('DELETE FROM ' . DB_CACHE_TABLE . ($pattern ? ' WHERE name LIKE "' . $pattern . '%"' : ''));
		return DB_WE::$inst->affected_rows() > 0;
	}

	public function get(string $entry): mixed{
		$data = getHash('SELECT `data`,`type` FROM ' . DB_CACHE_TABLE . ' WHERE id=x\'' . md5($entry) . '\' AND valid>NOW()');
		if(!$data){
			throw new we_exception_cache();
		}
		switch($data['type']){
			case self::PACK_TYPE_PLAIN:
				return $data['data'];
			case self::PACK_TYPE_PACKED:
				return gzuncompress($data['data']);
			case self::PACK_TYPE_SERIALIZED:
				return unserialize(gzuncompress($data['data']) ?: '') ?: '';
			default:
				throw new we_exception_cache();
		}
	}

	public function getStatus(): string{
		$tmp = getHash('SHOW TABLE STATUS like "' . DB_CACHE_TABLE . '"');
		$ret = new we_html_table(['class' => 'default statusTable'], 3, 2,
				[
			new we_html_tr([new we_html_td('Rows'), new we_html_td($tmp['Rows'])]),
			new we_html_tr([new we_html_td('Avg row length'), new we_html_td($tmp['Avg_row_length'])]),
			new we_html_tr([new we_html_td('Data length'), new we_html_td($tmp['Data_length'])])
		]);
		return $ret->getHtml();
	}

	public function increment(string $entry, int $default = 1, int $expiry = 3600): int{
		return 0;
	}

	public function quit(): void{

	}

	public function set(string $entry, $data, int $expiry): bool{
		if(is_string($data)){
			if(strlen($data) > 255){
				$type = self::PACK_TYPE_PACKED;
				$data = gzcompress($data, 4);
			} else {
				$type = self::PACK_TYPE_PLAIN;
			}
		} else {
			$type = self::PACK_TYPE_SERIALIZED;
			$data = gzcompress(serialize($data), 4);
		}
		assert(is_string($data));
		return DB_WE::$inst->query('INSERT INTO ' . DB_CACHE_TABLE . ' SET ' . we_database_base::arraySetter([
							'id' => sql_function('x\'' . md5($entry) . '\''),
							'name' => $entry,
							'type' => $type,
							'valid' => sql_function('now()+INTERVAL ' . $expiry . ' SECOND'),
							'data' => sql_function('x\'' . bin2hex($data) . '\''),
		]));
	}

	public function setServers(array $servers): void{

	}

	public function touch(string $entry, int $expiry = 3600): bool{
		DB_WE::$inst->query('UPDATE ' . DB_CACHE_TABLE . ' SET valid=now()+INTERVAL ' . $expiry . ' SECOND WHERE id=x\'' . md5($entry) . '\' ');
		return (DB_WE::$inst->affected_rows() > 0);
	}

}
