<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 7699ac94e97c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-12 01:24 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_category_view extends we_modules_view{
	/** @var we_category_category */
	var $Model;

	function __construct(){
		parent::__construct(we_modules_info::CATEGORY);
		$this->Model = new we_category_category();
	}

	function getJSTop(): string{
		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/category/category_top.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	function getJSProperty(array $jsVars = []): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/category/category_prop.js');
	}

	private function renameChildrenPath(int $id): void{
		$db = $this->db;
		$parentPath = id2path($id, CATEGORY_TABLE);
		$db->query('UPDATE ' . CATEGORY_TABLE . ' SET Path=CONCAT("' . $db->escape($parentPath) . '","/",Text) WHERE ParentID=' . $id);
		$updates = $db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . $id, true);
		foreach($updates as $id){
			$this->renameChildrenPath((int) $id);
		}
	}

	private function CatInUse(int $id): bool{
		$childs = $this->getChilds($id);
		$childs[] = $id;
		if(fB('SELECT 1 FROM ' . FILE_TABLE . ' f JOIN ' . CATEGORY_TABLE . ' c ON (FIND_IN_SET(c.ID,f.Category) OR FIND_IN_SET(c.ID,f.temp_category) )  WHERE c.ID IN (' . implode(',', $childs) . ') LIMIT 1') ||
			(we_modules_info::isActive(we_modules_info::OBJECT) && fB('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' `of` JOIN ' . CATEGORY_TABLE . ' c ON (FIND_IN_SET(c.ID,`of`.Category)) WHERE c.ID IN (' . implode(',', $childs) . ') LIMIT 1'))){
			return true;
		}
		return false;
	}

	/**
	 *
	 * @param int $id
	 * @return int[]
	 */
	private function getChilds(int $id): array{
		$ret = [];
		$ids = [$id];
		while(($ids = $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID IN (' . implode(',', $ids) . ')', true))){
			'@phan-var int[] $ids';
			$ret = array_merge($ret, $ids);
		}
		return $ret;
	}

	private function delDir(int $id): void{
		if(!$id){
			return;
		}
		$entries = $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . $id, true);
		foreach($entries as $entry){
			$this->delDir((int) $entry);
		}
		$entries = array_map('intval', $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . $id, true));
		$entries[] = $id;
		$this->delEntries($entries);
	}

	/**
	 *
	 * @param int[] $ids
	 * @return void
	 */
	private function delEntries(array $ids): void{
		$this->db->query('DELETE FROM ' . CATEGORY_TABLE . ' WHERE ID IN (' . implode(',', $ids) . ')');
	}

	function processCommands(we_base_jsCmd $jsCmd): void{
		switch(we_base_request::string('cmd', '')){
			case 'we_category_save':
				if(!we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT])){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$this->Model->isnew = empty($this->Model->ID); // FIXME: if init was ok we wouldn't need this!

				if(empty($this->Model->Text)){
					$jsCmd->addMsg(g_l('weEditor', '[category][filename_empty]'), we_base_util::WE_MESSAGE_ERROR);
				} else if(str_contains($this->Model->Text, ',')){
					$jsCmd->addMsg(g_l('weEditor', '[category][name_komma]'), we_base_util::WE_MESSAGE_ERROR);
				} elseif(fB('SELECT 1 FROM ' . CATEGORY_TABLE . ' WHERE ID!=' . intval($this->Model->ID) . ' AND ParentID=' . $this->Model->ParentID . ' AND Text="' . $this->db->escape($this->Model->Text) . '" LIMIT 1')){
					$jsCmd->addMsg(sprintf(g_l('weEditor', '[category][response_path_exists]'), $this->Model->Path), we_base_util::WE_MESSAGE_ERROR);
				} elseif(preg_match('|[\\\'"<>/]|', $this->Model->Text)){
					$jsCmd->addMsg(sprintf(g_l('weEditor', '[category][we_filename_notValid]'), $this->Model->Path), we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->Model->isSameAsParent()){
					$jsCmd->addMsg(g_l('weEditor', '[folder_save_nok_parent_same]'), we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->Model->save()){
					$jsCmd->addMsg(sprintf(g_l('thumbnails', '[saved_successfully]'), $this->Model->Text), we_base_util::WE_MESSAGE_NOTICE);
					if($this->Model->wasUpdate){
						$this->renameChildrenPath($this->Model->ID);
						$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text]);
					} else {
						//update parent
						if($this->Model->ParentID){
							$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ParentID, 'typ' => we_tree_base::TYP_FOLDER, 'contenttype' => we_base_ContentTypes::CATEGORIES]);
						}
						$jsCmd->addCmd('makeTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text, 'open' => false, 'contenttype' => we_base_ContentTypes::CATEGORY,
							'table' => CATEGORY_TABLE, 'published' => 1]);
					}
				} else {
					echo "ERROR";
				}
				break;
			case 'we_category_delete':
				if(!$this->Model->ID || !we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT])){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				$catsToDel = [$this->Model->ID];
				$finalDelete = [];
				$catlistNotDeleted = '';
				foreach($catsToDel as $id){
					if($this->CatInUse($id)){
						$catlistNotDeleted .= id2path($id, CATEGORY_TABLE) . "\n";
					} else {
						$finalDelete[] = $id;
					}
				}
				if($finalDelete){
					foreach($finalDelete as $id){
						$this->delDir($id);
						$jsCmd->addCmd('deleteTreeEntry', $id);

						if($id === $this->Model->ID){ // currently opened cat is deleted: replace editor by home
							$jsCmd->addCmd('unsetHot');
							$jsCmd->addCmd('loadHome');
							$this->Model = new we_category_category();
							$_SESSION['weS']['module_sessions']['category'] = null;
						}
					}
				}
				if($catlistNotDeleted){
					$jsCmd->addMsg(g_l('fileselector', '[cat_in_use]') . "\n\n" . $catlistNotDeleted, we_base_util::WE_MESSAGE_ERROR);
				}
				break;
			default:
				$id = (we_base_request::int('we_cmd', 0, 1) ?: 0);
				if($id){
					$this->Model->load($id);
					break;
				}
				$this->Model = new we_category_category();
		}
		$_SESSION['weS']['module_sessions']['category'] = $this->Model;
	}

	function processVariables(): void{
		if(!empty($_SESSION['weS']['module_sessions']['category'])){
			$this->Model = $_SESSION['weS']['module_sessions']['category'];

			$this->Model->Text = we_base_request::string('Text');
			$this->Model->ParentID = we_base_request::int('FolderID');
			$this->Model->Path = ($this->Model->ParentID ? id2path((int) $this->Model->ParentID, CATEGORY_TABLE) : '') . '/' . $this->Model->Text;

			$this->Model->Title = we_base_request::string('catTitle');
			$this->Model->Description = we_base_request::_(we_base_request::RAW_CHECKED, 'catDescription'); //keep HTML Code
			//$this->Model->isnew = false; // FIXME: make isnew persistent or
		} else {
			$_SESSION['weS']['module_sessions']['category'] = null;
		}
	}

	public function formName(): string{
		$path = (($pos = strrpos($this->Model->Path, '/' . $this->Model->Text)) !== false ? substr($this->Model->Path, 0, $pos) : $this->Model->Path) ?: '/';
		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('Doc')
			->setTable(CATEGORY_TABLE)
			->setContentType([])
			->setInput('FolderIDPath', $path)
			->setMaxResults(20)
			->setRequired(true)
			->setResult('FolderID', (string) ($this->Model->ParentID))
			->setSelector(we_gui_suggest::DirSelector)
			->setWidth(250)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_file', {id:document.we_form.elements.FolderID.value, table:WE().consts.tables.CATEGORY_TABLE, targetID:'FolderID', targetName:'FolderIDPath'})"));

		return we_html_tools::formElementTable(g_l('weClass', '[category]'), we_html_tools::textInput('Text', $this->Model->Text, 0, ['id' => 'category'], 'text', '360px')) .
			we_html_element::br() .
			we_html_tools::formElementTable(g_l('weClass', '[dir]'), $weSuggest->getHTML());
	}

	public function formSettings(): string{
		return we_html_tools::formElementTable(g_l('global', '[title]'), we_html_tools::textInput('catTitle', $this->Model->Title, 0, [], 'text', '360px')) .
			we_html_element::br() .
			we_html_tools::formElementTable(g_l('global', '[description]'), we_html_forms::weTextarea('catDescription', $this->Model->Description, [
					'bgcolor' => 'white',
					'inlineedit' => 'true',
					'wysiwyg' => 'true',
					'width' => '570',
					'height' => '250',
					'commands' => 'prop,font,xhtmlxtras,color,justify,list,link,table,insert,fullscreen,visibleborders,editsource'
					], true, true, true));
	}

	public function getHomeScreen(we_base_jsCmd $jscmd): string{
		return parent::getActualHomeScreen(we_modules_info::CATEGORY, $jscmd, we_html_button::create_button('fat:add,fa-lg fa-plus', "javascript:we_cmd('we_category_new')"), we_html_element::form([
					'name' => we_base_constants::WE_FORM], $this->getCommonHiddens(['cmd' => 'home', 'pnt' => 'edbody']) . we_html_element::hidden('home', 0)));
	}

}
