<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 791cbddde658 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-08-25 14:02 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
ini_set('mysqli.reconnect', '1');
!defined('MYSQL_BOTH') && define('MYSQL_BOTH', MYSQLI_BOTH);
!defined('MYSQL_ASSOC') && define('MYSQL_ASSOC', MYSQLI_ASSOC);
!defined('MYSQL_NUM') && define('MYSQL_NUM', MYSQLI_NUM);

class DB_WE extends we_database_base{

	/** @var mysqli_result|null */
	protected $Query_ID = null;

	public const SQLMODE = 'STRICT_TRANS_TABLES,ERROR_FOR_DIVISION_BY_ZERO,NO_ENGINE_SUBSTITUTION,NO_ZERO_DATE,NO_ZERO_IN_DATE';

	protected function _free(): void{
		if(is_object($this->Query_ID)){
			try{
				@$this->Query_ID->free();
			}catch(Exception $e){
				t_e($this->Query_ID);
			}
		}
		$this->Query_ID = null;
	}

	/**
	 *
	 * @param string $Query_String
	 * @param bool $unbuffered
	 * @return mysqli_result|int|null
	 */
	protected function _query(string $Query_String, bool $unbuffered = false){
		$this->_free();
		try{
			$tmp = $this->Link_ID->query($Query_String, ($unbuffered ? MYSQLI_USE_RESULT : MYSQLI_STORE_RESULT));
		}catch(\Exception $e){
			return null; //in php 8.1 mysqli will raise exceptions instead of false
		}
		if($tmp === false){
			return null;
		}
		if($tmp === true){
			return -1; //!=0 what is tested.
		}
		return $tmp;
	}

	protected function _setCharset(string $charset): void{
		$this->Link_ID->set_charset($charset);
	}

	protected function _seek(int $pos = 0): bool{
		return (is_object($this->Query_ID)) && $this->Query_ID->data_seek($pos);
	}

	protected function setError(): void{
		$this->Warnings = [];
		if($this->Link_ID){
			$this->Errno = $this->Link_ID->errno;
			$this->Error = $this->Link_ID->error;
			if($this->Link_ID->warning_count){
				$warn = $this->Link_ID->get_warnings();
				for($i = 0; $i < $this->Link_ID->warning_count; $i++){
					$this->Warnings[] = $warn->message;
				}
			}
		} else {
			$this->Errno = 2006;
			$this->Error = '';
		}
	}

	protected function info(): string{
		return ($this->Link_ID ? $this->Link_ID->info ?? '' : '');
	}

	protected function fetch_array(int $resultType): ?array{
		return (is_object($this->Query_ID)) ?
				$this->Query_ID->fetch_array($resultType) :
				null;
	}

	protected function _affected_rows(): int{
		return $this->Link_ID->affected_rows;
	}

	public function close(): void{
		if($this->Link_ID){
			$this->Link_ID->close();
			$this->Link_ID = null;
			$this->linkType = '';
			$this->Query_ID = null;
		}
	}

	protected function connect(string $Database = DB_DATABASE, string $Host = DB_HOST, string $User = DB_USER, string $Password = DB_PASSWORD): bool{
		$this->Database = $Database;
		if($this->isConnected()){
			return true;
		}
		$hostPort = explode(':', $Host);
		$Host = $hostPort[0];
		$Port = intval($hostPort[1] ?? ini_get('mysqli.default_port')) ?: 3306;

		$this->Query_ID = null;
		if($this->Link_ID === null){
			self::$counts['link']++;
			$this->Link_ID = mysqli_init();
			$this->Link_ID->options(MYSQLI_OPT_CONNECT_TIMEOUT, 60);
			if(defined('MYSQLI_OPT_INT_AND_FLOAT_NATIVE')){
				$this->Link_ID->options(MYSQLI_OPT_INT_AND_FLOAT_NATIVE, 1);
			}
		}
		//skip mails and log entries, if connection is reset and can be reestablished
		$log = we_base_errorHandler::getHandling(we_base_errorHandler::TARGET_LOG);
		$mail = we_base_errorHandler::getHandling(we_base_errorHandler::TARGET_MAIL);
		we_base_errorHandler::setHandling(we_base_errorHandler::TARGET_LOG, false);
		we_base_errorHandler::setHandling(we_base_errorHandler::TARGET_MAIL, false);
		$error = null;
		for($retry = 3; $retry > 0; $retry--){
			$error = null;
			try{
				if($this->Link_ID->real_connect($Host, $User, $Password, $Database, $Port, '', MYSQLI_CLIENT_COMPRESS)){
					break;
				}
			}catch(mysqli_sql_exception $e){
				$error = $e;
			}
			usleep(200);
		}
		we_base_errorHandler::setHandling(we_base_errorHandler::TARGET_LOG, $log);
		we_base_errorHandler::setHandling(we_base_errorHandler::TARGET_MAIL, $mail);
		if($retry <= 0 ||
				$this->Link_ID->connect_error ||
				$error){
			$this->Link_ID = null;
			$this->linkType = '';
			$GLOBALS['we']['ERROR'] = 'mysqli_(p)connect(' . $Host . ', ' . $User . ') ' .
					($this->Link_ID ? $this->Link_ID->connect_error : '') .
					($error ? $error->getMessage() : '') .
					' failed.';
			//trigger_error('WE-Connect error ' . $this->Link_ID->connect_error, E_USER_WARNING);
			//$this->halt("mysqli_(p)connect($Host, $User) failed.");
			return false;
		}

		$this->_setup();
		return true;
	}

	/**
	 * call this function to make sure the connection is setup correctly
	 */
	protected function _setup(): void{
		$this->_query('SET SESSION sql_mode="' . self::SQLMODE . '"');
		if(defined('DB_SET_CHARSET') && DB_SET_CHARSET){
			$this->_setCharset(DB_SET_CHARSET);
		} else {
			$this->_setCharset(self::DB_CHARSET);
		}
	}

	public function field_flags(int $no): int{
		return (is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->flags : 0);
	}

	public function field_default(int $no): string{
		return (is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->def : '');
	}

	public function field_len(int $no): int{
		if(is_object($this->Query_ID)){
			$len = $this->Query_ID->fetch_field_direct($no)->length;
			//fix faulty lenght on text-types with connection in utf-8
			$type = $this->Query_ID->fetch_field_direct($no)->type;
			if(in_array($type, [MYSQLI_TYPE_VAR_STRING, MYSQLI_TYPE_VAR_STRING, MYSQLI_TYPE_STRING])){
				$len /= self::DB_CHARSET_LEN;
			}
			return (int) $len;
		}
		return 0;
	}

	public function field_name(int $no): string{
		return (is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->name : '');
	}

	public function field_table(int $no): string{
		return (is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->table : '');
	}

	public function field_type(int $no): string{
		switch(is_object($this->Query_ID) ? $this->Query_ID->fetch_field_direct($no)->type : 0){
			case MYSQLI_TYPE_CHAR:
				return self::TYPE_TINYINT;
			case MYSQLI_TYPE_SHORT:
				return self::TYPE_SMALLINT;
			case MYSQLI_TYPE_LONG:
				return self::TYPE_INT;
			case MYSQLI_TYPE_INT24:
				return self::TYPE_MEDIUMINT;
			case MYSQLI_TYPE_LONGLONG:
				return self::TYPE_BIGINT;
			case MYSQLI_TYPE_FLOAT:
				return self::TYPE_FLOAT;
			case MYSQLI_TYPE_DOUBLE:
				return self::TYPE_DOUBLE;
			case MYSQLI_TYPE_NEWDECIMAL:
				return self::TYPE_DECIMAL;
			case MYSQLI_TYPE_NULL:
				return self::TYPE_NULL;
			case MYSQLI_TYPE_TIMESTAMP:
				return self::TYPE_TIMESTAMP;
			case MYSQLI_TYPE_DATE:
				return self::TYPE_DATE;
			case MYSQLI_TYPE_TIME:
				return self::TYPE_TIME;
			case MYSQLI_TYPE_DATETIME:
				return self::TYPE_DATETIME;
			case MYSQLI_TYPE_YEAR:
				return self::TYPE_YEAR;
			case MYSQLI_TYPE_NEWDATE:
				return self::TYPE_DATE;
			case MYSQLI_TYPE_BIT:
				return self::TYPE_BIT;
			case MYSQLI_TYPE_ENUM:
				return self::TYPE_ENUM;
			case MYSQLI_TYPE_SET:
				return self::TYPE_SET;
			case MYSQLI_TYPE_TINY_BLOB:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_TINYBLOB : self::TYPE_TINYTEXT);
			case MYSQLI_TYPE_MEDIUM_BLOB:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_MEDIUMBLOB : self::TYPE_MEDIUMTEXT);
			case MYSQLI_TYPE_BLOB:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_BLOB : self::TYPE_TEXT);
			case MYSQLI_TYPE_LONG_BLOB:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_LONGBLOB : self::TYPE_LONGTEXT);
			case MYSQLI_TYPE_GEOMETRY:
				return self::TYPE_GEOMETRY;
			//252 is currently mapped to all text and blob types (MySQL 5.0.51a)
			case MYSQLI_TYPE_VAR_STRING:
				return ($this->field_flags($no) & MYSQLI_BINARY_FLAG ? self::TYPE_VARBIN : self::TYPE_VARCHAR);
			case MYSQLI_TYPE_STRING:
				$flags = $this->field_flags($no);
				return ($flags & MYSQLI_BINARY_FLAG ?
						self::TYPE_BIN :
						($flags & MYSQLI_SET_FLAG ?
						self::TYPE_SET :
						($flags & MYSQLI_ENUM_FLAG ?
						self::TYPE_ENUM : self::TYPE_CHAR)
						)
						);
			default:
				return '';
		}
	}

	public function field_type_short(int $no): string{
		switch($this->field_type($no)){
			case self::TYPE_TINYINT:
			case self::TYPE_SMALLINT:
			case self::TYPE_INT:
			case self::TYPE_MEDIUMINT:
			case self::TYPE_BIGINT:
			case self::TYPE_FLOAT:
			case self::TYPE_DOUBLE:
			case self::TYPE_DECIMAL:
			case self::TYPE_YEAR:
			case self::TYPE_BIT:
				return self::TYPE_SHORT_NUMBER;
			case self::TYPE_NULL:
				return self::TYPE_NULL;
			case self::TYPE_TIME:
				return self::TYPE_SHORT_TIME;
			case self::TYPE_TIMESTAMP:
			case self::TYPE_DATETIME:
				return self::TYPE_SHORT_DATETIME;
			case self::TYPE_DATE:
				return self::TYPE_SHORT_DATE;
			case self::TYPE_TINYBLOB:
			case self::TYPE_MEDIUMBLOB:
			case self::TYPE_BLOB:
			case self::TYPE_LONGBLOB:
			case self::TYPE_VARBIN:
			case self::TYPE_BIN:
				return self::TYPE_SHORT_BINARY;
			case self::TYPE_ENUM:
			case self::TYPE_SET:
			case self::TYPE_VARCHAR:
			case self::TYPE_CHAR:
			case self::TYPE_TINYTEXT:
			case self::TYPE_MEDIUMTEXT:
			case self::TYPE_TEXT:
			case self::TYPE_LONGTEXT:
				return self::TYPE_SHORT_STRING;
			case self::TYPE_GEOMETRY:
				return self::TYPE_GEOMETRY;
			default:
				return '';
		}
	}

	protected function _getInsertId(): int{
		return $this->Link_ID->insert_id;
	}

	public function num_fields(): int{
		return (is_object($this->Query_ID) ? $this->Query_ID->field_count : 0);
	}

	public function num_rows(): int{
		return is_object($this->Query_ID) ? $this->Query_ID->num_rows : 0;
	}

	public function _getCurrentCharset(): string{
		$link = $this->Link_ID;
		if(!$link){
			throw new we_exception_database('not connected');
		}
		$charset = mysqli_get_charset($link);
		return $charset->charset;
	}

	/**
	 * @throws we_exception_database
	 * @return array{type:string,protocol:string,client:string,host:string,server:string,database:string,encoding:string}
	 */
	public function _getInfo(): array{
		$link = $this->Link_ID;
		if(!$link){
			throw new we_exception_database('not connected');
		}
		$charset = mysqli_get_charset($link);
		return [
			'type' => 'mysqli_connect',
			'protocol' => $this->Link_ID->protocol_version,
			'client' => $this->Link_ID->client_info,
			'host' => $this->Link_ID->host_info,
			'server' => $this->Link_ID->server_info,
			'database' => $this->Database,
			'encoding' => $charset->charset,
			'read-hosts' => implode(we_html_element::br(), defined('DB_SLAVES') ? DB_SLAVES : []) ?: '-',
		];
	}

	protected function ping(): bool{
		return $this->Link_ID->ping();
	}

	/**
	 * @return array<int,string>
	 */
	public function fieldNames(): array{
		if(!($this->Query_ID) || !is_object($this->Query_ID)){
			throw new we_exception_database('no pending query');
		}
		$fields = $this->Query_ID->fetch_fields();
		$res = [];
		foreach($fields as $f){
			$res[] = $f->name;
		}
		return $res;
	}

	public function flags2txt(int $flags_num): string{
		static $flags = [];

		if(!$flags){
			$constants = get_defined_constants(true);
			$m = [];
			foreach($constants['mysqli'] as $c => $n){
				'@phan-var int $n';
				if(preg_match('/MYSQLI_(.*)_FLAG$/', $c, $m) && !array_key_exists($n, $flags)){
					$flags[$n] = $m[1];
				}
			}
		}

		$result = [];
		foreach($flags as $n => $t){
			if($flags_num & $n){
				$result[] = $t;
			}
		}
		return implode(',', $result);
	}

}
