<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3b8044c13b9c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-16 00:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_dialog_copyFolder{

	private static function formCreateCategoryChooser(we_base_jsCmd $jsCmd): string{
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document',{id:-1,table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'addCat'})");
		$del_but = we_html_button::create_button(we_html_button::TRASH, "javascript:" . we_base_constants::PLACEHOLDER);
		$jsCmd->addCmd('setCategories_edit', $del_but);

		$table = (new we_html_table([
				'id' => 'CategoriesBlock',
				'style' => 'display: block;',
				'class' => 'default withSpace',
				], 3, 2))
			->setCol(0, 0, g_l('copyFolder', '[categories]'), ['width' => '100'])
			->setCol(1, 1, we_html_forms::checkbox('1', false, 'OverwriteCategories', g_l('copyFolder', '[overwrite_categories]'), false, "defaultfont", "toggleButton();"))
			->setCol(1, 0, we_html_element::div(['id' => 'categories',
					'class' => 'blockWrapper',
					'style' => 'width: 488px; height: 60px; border: #AAAAAA solid 1px;'
				]), ['colspan' => '2'])
			->setCol(2, 0, we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:removeAllCats()") . $addbut, ['colspan' => '2', 'style' => 'text-align:right;']);

		return $table->getHtml();
	}

	private static function formCreateTemplateDirChooser(): string{
		$path = '/';
		$myid = 0;

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('Template')
			->setContentType([we_base_ContentTypes::FOLDER_TEMPLATE])
			->setInput('foo', $path, [], true)
			->setLabel(g_l('copyFolder', '[destdir]'))
			->setMaxResults(10)
			->setRequired(true)
			->setResult('CreateTemplateInFolderID', (string) $myid)
			->setSelector(we_gui_suggest::DirSelector)
			->setTable(TEMPLATES_TABLE)
			->setWidth(370)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements.CreateTemplateInFolderID.value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'CreateTemplateInFolderID',targetName:'foo',jsCmd:'setCreateTemplate'})", '', '', true, false));

		return $weSuggest->getHTML();
	}

	public static function getDialog(): void{


		if(we_base_request::bool('we_cmd', false, 3)){
			$cmd1 = we_base_request::int('we_cmd', 0, 1);
			$cmd4 = we_base_request::_(we_base_request::TABLE, 'we_cmd', '', 4);
			$jsCmd = new we_base_jsCmd();

			$yes_button = we_html_button::create_button(we_html_button::OK, we_html_button::WE_FORM . ":we_form");
			$cancel_button = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");

			$pb = new we_gui_progressBar(0, 270);

			$pb->addText("&nbsp;", we_gui_progressBar::TOP, "pbar1");

			$buttons = '<table class="default" style="width:100%"><tr><td id="pbTd" style="text-align:left;">' . $pb->getHTML('', 'display:none;') . '</td><td style="text-align:right">' .
				we_html_button::position_yes_no_cancel($yes_button, '', $cancel_button) .
				'</td></tr></table>';

			$hidden = we_html_element::hiddens([
					'we_cmd[0]' => we_base_request::string('we_cmd', '', 0),
					'we_cmd[1]' => $cmd1,
					'we_cmd[2]' => we_base_request::int('we_cmd', 0, 2),
					($cmd4 ? 'we_cmd[4]' : '') => $cmd4]);

			switch($cmd4){
				case OBJECT_FILES_TABLE:
					$content = g_l('copyFolder', '[object_copy]') . '<br/>' .
						we_html_forms::checkbox('1', false, 'DoNotCopyFolders', g_l('copyFolder', '[object_copy_no_folders]')) .
						'&nbsp;<br/>' . g_l('copyFolder', '[sameName_headline]') . '<br/>' .
						we_html_tools::alertAttentionBox(g_l('copyFolder', '[sameName_expl]'), we_html_tools::TYPE_INFO, 380) .
						we_html_element::div(['style' => 'margin-top:10px;'],
													 we_html_forms::radiobutton(we_exim_ExIm::COLLISION_OVERWRITE, false, 'OverwriteObjects', g_l('copyFolder', '[sameName_overwrite]')) .
							we_html_forms::radiobutton(we_exim_ExIm::COLLISION_RENAME, false, 'OverwriteObjects', g_l('copyFolder', '[sameName_rename]')) .
							we_html_forms::radiobutton(we_exim_ExIm::COLLISION_SKIP, true, 'OverwriteObjects', g_l('copyFolder', '[sameName_nothing]'))
						) .
						$hidden;
					break;
				default:
					$content = '<table class="default" style="width:500px;"><tr>' .
						(we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATE_NEW]) ?
						'<td>' . we_html_forms::checkbox('1', false, 'CreateTemplate', g_l('copyFolder', '[create_new_templates]'), false, '', "toggleButton(); incTemp(this.checked)") .
						we_html_element::div(['id' => "imTemp", 'style' => "display:block"], we_html_forms::checkbox('1', false, 'CreateMasterTemplate', g_l('copyFolder', '[create_new_masterTemplates]'), false, '', '', true) .
							we_html_forms::checkbox('1', false, 'CreateIncludedTemplate', g_l('copyFolder', '[create_new_includedTemplates]'), false, '', '', true)
						) . '</td>' : '') .
						'<td style="vertical-align:top">' .
						(we_base_permission::hasPerm([we_base_perm::WORK_DOCTYPE_EDIT]) ?
						we_html_forms::checkbox('1', false, 'CreateDoctypes', g_l('copyFolder', '[create_new_doctypes]')) : '') .
						we_html_forms::checkbox('1', true, 'InheritLangFromToID', g_l('copyFolder', '[inherit_language]'), false, "defaultfont", "toggleInheritLanguage(this.checked);") .
						we_html_forms::checkbox('1', true, 'KeepLanglinks', g_l('copyFolder', '[keep_langlinks]')) .
						'</td></tr>' .
						(we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATE_NEW]) ?
						'<tr><td colspan="2">' . self::formCreateTemplateDirChooser() . '</td></tr>' .
						(we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT]) && we_modules_info::isActive(we_modules_info::CATEGORY) ?
						'<tr><td colspan="2">' . self::formCreateCategoryChooser($jsCmd) . '</td></tr>' : '') : '') .
						'</table>' . $hidden;
			}

			echo we_html_tools::getHtmlTop('', we_gui_progressBar::getJSCode() .
				we_html_element::jsScript(JS_DIR . 'copyFolder.js') .
				$jsCmd->getCmds(),
															we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], '<form name="' . we_base_constants::WE_FORM . '" target="pbUpdateFrame" method="get">' .
					we_html_tools::dialogLayout($content, g_l('copyFolder', '[headline]') . ": " . we_base_util::shortenPath(id2path($cmd1, $cmd4), 46), $buttons) .
					'</form>' .
					'<iframe src="about:blank" name="pbUpdateFrame" style="width:0;height:0;" id="pbUpdateFrame"></iframe>'
			));
			return;
		}

		$bodyAttribs = ['style' => "background-color:#FFFFFF;margin:10px;"];
		$frag = (we_base_request::bool('finish') ?
			new we_fragment_copyFolderFinish('we_copyFolderFinish', 5, $bodyAttribs) :
			new we_fragment_copyFolder('we_copyFolder', 5, $bodyAttribs)
			);
		$frag->run();
	}

}
