<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0fb4cd0ea2f4 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-12-15 12:13 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_document_audio extends we_document_binary{

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::AUDIO;

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		if(isWE()){
			$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_PREVIEW;
		}
	}

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				return new we_editor_document_preview($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
	}

	public function getHtml(bool $dyn = false): string{
		$data = $this->getElementS('data');
		if($this->ID || ($data && !is_dir($data) && is_readable($data))){

			return '<audio controls preload="none" style="margin-left:2em;">
							<source src="' . ( $dyn ?
					WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showBinaryDoc&we_cmd[1]=' . $this->ContentType . '&we_cmd[2]=' . we_base_WE::$transaction . '&rand=' . we_base_file::getUniqueId() :
					$this->Path) . '" type="audio/' . str_replace('.', '', $this->Extension) . '">
						</audio>';
		}
		return '';
	}

	static function checkAndPrepare(string $formname, we_contents_root $doc): void{
		$reqName = 'we_ui_' . $formname;
		if(empty($_FILES[$reqName]) || !is_array($_FILES[$reqName]) || empty($_FILES[$reqName]['name']) || !is_array($_FILES[$reqName]['name'])){
			return;
		}

		foreach($_FILES[$reqName]['name'] as $audioName => $filename){
			'@phan-var string $audioName';
			'@phan-var string $filename';
			if(!($audioDataId = we_base_request::string(we_fileupload::USERINPUT_TYPE_AUDIO . $audioName)) || empty($_SESSION[$audioDataId])){
				continue;
			}

			if(we_base_request::_(we_base_request::BOOL, we_fileupload::USERINPUT_TYPE_DEL . $audioName)){
				$_SESSION[$audioDataId]['doDelete'] = true;
				continue;
			}
			$_SESSION[$audioDataId]['doDelete'] = false;
			if($filename &&
					getContentTypeFromFile($filename) === we_base_ContentTypes::AUDIO){
				$audioid = $doc->getElementI($audioName);
				$_SESSION[$audioDataId]['serverPath'] = $serverPath = TEMP_PATH . we_base_file::getUniqueId();
				we_base_file::insertIntoCleanUp($serverPath);
				move_uploaded_file((string) $_FILES[$reqName]['tmp_name'][$audioName], $serverPath);
				$unique = we_base_file::getUniqueId();
				$tmp_Filename = $audioName . '_' . $unique . '_' . preg_replace('[^A-Za-z0-9._-]', '', (string) $_FILES[$reqName]['name'][$audioName]);

				if($audioid){
					$_SESSION[$audioDataId]['id'] = $audioid;
				}
				[$tmp_Filename, $extension] = self::cleanUploadFilename($tmp_Filename);
				$_SESSION[$audioDataId] = array_merge($_SESSION[$audioDataId], [
					'fileName' => $tmp_Filename,
					'extension' => $extension,
					'text' => $tmp_Filename . $extension,
					'unique' => $unique,
					'type' => $_FILES[$reqName]['type'][$audioName],
					'size' => $_FILES[$reqName]['size'][$audioName],
				]);
			}
		}
	}

}
