<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0b80d828f85e $
 * $Author: Lukas Imhof $
 * $Date: 2021-11-06 23:04 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_html_multiIconBox{
	private static $pos = 0;

	//INFO
	public const SPACE_SMALL = 'small';
	public const SPACE_ICON = 'icon';
	public const SPACE_MED = 'med';
	public const SPACE_MED2 = 'med2';
	public const SPACE_BIG = 'big';
	public const PROP_ATTRIB = '<i class="fa fa-cog"></i>';
	public const PROP_AUTONAME = '<i class="fa fa-font"></i>';
	public const PROP_BANNER = '<i class="fa fa-flag"></i>';
	public const PROP_CATEGORIES = '<i class="fa fa-tags"></i>';
	public const PROP_CHARSET = '<i class="fa fa-paragraph"></i>';
	public const PROP_CLASS = '<i class="fa fa-code"></i>';
	public const PROP_CONTENT = '<i class="fa fa-inbox"></i>';
	public const PROP_COPY = '<i class="fa fa-copy"></i>';
	public const PROP_CUSTOMER = '<i class="fa fa-users"></i>';
	public const PROP_DISPLAY = '<i class="far fa-eye"></i>';
	public const PROP_DOC = '<i class="fa fa-file-text"></i>';
	public const PROP_FILTER = '<i class="fa fa-filter"></i>';
	public const PROP_HELP = '<i class="fa fa-info"></i>';
	public const PROP_IMAGE = '<i class="fa fa-image"></i>';
	public const PROP_KEYBOARD = '<i class="fa fa-keyboard-o"></i>';
	public const PROP_LANG = '<i class="fa fa-language"></i>';
	public const PROP_LINK = '<i class="fa fa-link"></i>';
	public const PROP_MAIL = '<i class="fa fa-envelope"></i>';
	public const PROP_MASTER = '<i class="fa fa-sitemap"></i>';
	public const PROP_META = '<i class="fa fa-plus-circle"></i>';
	public const PROP_NAVI = '<i class="fa fa-compass"></i>';
	public const PROP_NEWSLETTER = '<i class="fa fa-newspaper-o"></i>';
	public const PROP_OTHER = '<i class="fa fa-magic"></i>';
	public const PROP_PATH = '<i class="fa fa-folder"></i>';
	public const PROP_REVREL = '<i class="fa fa-exchange"></i>';
	public const PROP_RESULT = '<i class="fa fa-road"></i>';
	public const PROP_SEO = '<i class="fa fa-search-plus"></i>';
	public const PROP_SEARCH = '<i class="fa fa-search"></i>';
	public const PROP_SIDEBAR = '<i class="fa fa-columns"></i>';
	public const PROP_SORT = '<i class="fa fa-sort"></i>';
	public const PROP_STATISTIC = '<i class="fa fa-bar-chart"></i>';
	public const PROP_TIME = '<i class="fa fa-clock-o"></i>';
	public const PROP_UPLOAD = '<i class="fa fa-upload"></i>';
	public const PROP_USER = '<i class="fa fa-user"></i>';
	public const PROP_WINDOW = '<i class="fa fa-window-maximize"></i>';
	public const PROP_WORKSPACE = '<i class="fa fa-gavel"></i>';
	//CUSTOMER
	public const INFO_CALENDAR = '<i class="fa fa-calendar"></i>';
	public const INFO_WORKFLOW = '<i class="fa fa-gears"></i>';
	public const INFO_REFERENCES = '<i class="fa fa-shield"></i>';
	//CUSTOMER
	public const CUSTOMER_FILTER = '<i class="fa fa-filter"></i>';

	/** @var string */
	private $name = '';

	/** @var we_html_multiIconBoxContent */
	private $content;

	/** @var string */
	private $headline = '';

	/** @var int */
	private $foldAtNr = -1;

	/** @var string */
	private $foldRight = '';

	/** @var string */
	private $foldDown = '';

	/** @var bool */
	private $isFoldNamedMode = false;

	/** @var bool */
	private $displayAtStartup = false;

	/** @var string */
	private $buttons = '';

	/** @var string */
	private $delegate = '';

	/** @var bool */
	private $marginLeft = false;

	public function __construct(we_html_multiIconBoxContent $content, string $name = ''){
		$this->content = $content;
		$this->name = $name;
	}

	public function fold(int $at, bool $displayAtStartup, string $textRight, string $textDown = '', string $delegate = '', bool $isNamedMode = false): self{
		$this->foldAtNr = $at;
		$this->displayAtStartup = $displayAtStartup;
		$this->foldRight = $textRight;
		$this->foldDown = ($textDown ?: we_html_element::b($textRight));
		$this->delegate = $delegate;
		$this->isFoldNamedMode = $isNamedMode;
		return $this;
	}

	public function marginLeft($left = true): self{
		$this->marginLeft = $left;
		return $this;
	}

	public function headline(string $text): self{
		$this->headline = $text;
		return $this;
	}

	public function buttons(string $buttons): self{
		$this->buttons = $buttons;
		return $this;
	}

	public function get(): string{
		return self::getHTML($this->name, $this->content, $this->marginLeft, $this->buttons, $this->foldAtNr, $this->foldRight, $this->foldDown, $this->displayAtStartup, $this->headline, $this->delegate, $this->isFoldNamedMode);
	}

	/**
	 * @desc 	Get HTML-Code of the multibox
	 * @param	$name				string
	 * @param	$content			array content is ['icon'=>'','headline'=>'','space'=>'','rightHeadline'=>false,'html'=>'','noline'=>false,'fold'=>false,'preline'=>false,'hide'=>false,'class'=>'']
	 * @param	$buttons			string
	 * @param	$foldAtNr			int
	 * @param	$foldRight			unknown
	 * @param	$foldDown			unknown
	 * @param	$displayAtStartup	bool
	 * @param	$headline			string
	 * @return	string
	 */
	private static function getHTML(string $name, we_html_multiIconBoxContent $content, bool $marginLeft = false, string $buttons = '', int $foldAtNr = -1, string $foldRight = '', string $foldDown = '', bool $displayAtStartup = false, string $headline = '', string $delegate = '', bool $isFoldNamedMode = false): string{
		if($content->isEmpty()){
			return '';
		}

		$uniqname = $name ?: md5(uniqid(__FILE__, true));
		$fold = $foldSpace = '';

		$out = $headline ?
			self::getBoxStartHeadline($name, $headline, $marginLeft) :
			self::getBoxStart($name);

		foreach($content->getContent() as $i => $c){
			assert($c instanceof we_html_multiIconBoxContentLine);
			if($c->isHide()){
				continue;
			}
			$rightHeadline = $c->isRightHead();
			$icon = $c->getIcon();
			$headline = $c->getHeadline();
			$leftWidth = ($c->getSpace() ?: ($icon ? self::SPACE_ICON : '') );
			$leftContent = ($icon ?: ($leftWidth && !$rightHeadline && $headline ? we_html_element::div(['class' => 'weMultiIconBoxHeadline'], $headline) : ''));
			$rightHead = (($icon || !$leftContent || $rightHeadline) && $headline ? we_html_element::div(['class' => 'weMultiIconBoxHeadline'], $headline) : '');

			if($i + 1 === $foldAtNr){// only if the folded items contain stuff.
				$fold = ' ';
				$foldSpace = $leftWidth;
			}
			$leftContent = (!empty($leftContent) || $leftWidth ?
				we_html_element::div(['class' => 'multiiconleft leftSpace-' . $leftWidth], $leftContent) :
				'');

			$tmp = we_html_element::div(['class' => 'weMultiIconBoxContent' . (!$c->isPreline() ? '' : ' weMultiIconBoxPreLine') . ($i < (count($content) - 1) && (!$c->isNoline()) ? ' weMultiIconBoxLine' : '' ) . ($marginLeft ? ' withMargin' : '') . ' ' . $c->getClass(),
					'id' => 'div_' . $uniqname . '_' . $i],
																																																																			 $leftContent .
					we_html_element::div(['class' => 'multiIconRight'], (!$c->getHTML() ? '' :
						(!$c->isFold() ? $rightHead . we_html_element::div([], $c->getHTML()) :
						self::getFold(we_html_element::div([], $c->getHTML()), $uniqname . '_' . $i, $c->isFoldNamedMode(), false, $rightHead ? $headline : '', $rightHead ? $headline : '')
						)
						)
					)
			);

			if($fold){
				$fold .= $tmp;
			} else {
				$out .= $tmp;
			}
		}

		if($fold){
			$out .= self::getFold($fold, $uniqname, $isFoldNamedMode, $displayAtStartup, $foldDown, $foldRight, $delegate, $marginLeft, $foldSpace);
		}

		$out .= '</div>';

		return ($buttons ?
			//ignore height, replace by bottom:
			we_html_element::div(['class' => 'weMultiIconBoxWithFooter'], $out) .
			we_html_element::div(['class' => 'editfooter'], $buttons) :
			$out);
	}

	private static function getFold(string $content, string $uniqname, bool $isNamedMode, bool $displayAtStartup, string $foldDown = '', string $foldRight = '', string $moreJS = '', bool $marginLeft = false, string $foldSpace = ''): string{
		$displayAtStartup = $isNamedMode ? we_base_request::bool('fold_named', $displayAtStartup, $uniqname) :
			we_base_request::bool('fold', $displayAtStartup, self::$pos);

		$control = we_html_element::div(['class' => 'btn_direction_weMultibox_table' . ($marginLeft ? ' withMargin' : '')], self::getButton($uniqname, "WE().layout.toggleBox(document, '" . $uniqname . "');" . ($moreJS ?: '' ), ($displayAtStartup ? 'down' : 'right'), g_l('global', '[openCloseBox]')) .
				we_html_element::div(['class' => 'weMultiIconBoxHeadline toggleBox', 'id' => 'textRight_' . $uniqname, 'style' => 'display:' . ($displayAtStartup ? 'none' : '') . ';'], we_html_element::label([
						'for' => 'btn_direction' . $uniqname], $foldRight)) .
				we_html_element::div(['class' => 'weMultiIconBoxHeadline toggleBox', 'id' => 'textDown_' . $uniqname, 'style' => 'display:' . ($displayAtStartup ? '' : 'none') . ';'], we_html_element::label([
						'for' => 'btn_direction' . $uniqname], $foldDown))
			) .
			we_html_element::hidden('fold[' . self::$pos . ']', intval($displayAtStartup), 'hidden_' . $uniqname) .
			we_html_element::hidden('fold_named[' . $uniqname . ']', intval($displayAtStartup), 'hidden_' . $uniqname . '_named');

		self::$pos++;
		return ($foldSpace ? we_html_element::div(['class' => 'multiiconleft leftSpace-' . $foldSpace], '') . we_html_element::div(['class' => 'multiIconRight'], $control) : $control) .
			we_html_element::div(['class' => 'default iconBoxTable', 'id' => 'table_' . $uniqname, 'style' => ($displayAtStartup ? '' : 'display:none')], $content);
	}

	public static function addBoxContainer(string $name, string $data): string{
		return self::getBoxStart($name) . $data . '</div>';
	}

	private static function getBoxStartHeadline(string $name, string $headline, bool $marginLeft = false): string{
		return self::getBoxStart($name) .
			we_html_element::div(['class' => 'weDialogHeadline' . ($marginLeft ? ' withMargin' : '')], $headline);
	}

	private static function getBoxStart(string $name = ''): string{
		return '<div class="default multiIcon defaultfont"' . ($name ? ' id="' . $name . '"' : '') . '>';
	}

	private static function getButton(string $name, string $cmd, string $state = "right", string $title = ''): string{
		return we_html_button::create_button(($state == 'right' ? we_html_button::DIRRIGHT : we_html_button::DIRDOWN), "javascript:" . $cmd . ";", '', '', false, false, '', false, $title, '', 'btn_direction' . $name);
	}

	public static function setStartPos(int $startPos): void{
		self::$pos = $startPos;
	}

}
