<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6f31ae772e29 $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2021-12-19 19:28 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_CSV{
	/** @var string */
	protected $FieldDelim = ',';

	/** @var string */
	protected $Enclosure = '"';

	/** @var string */
	protected $FromCharset = 'UTF-8';

	/** @var string */
	protected $ToCharset = 'UTF-8';

	/** @var string */
	protected $CSVFile = '';

	/** @var string */
	protected $CSVData = '';

	/** @var string[] */
	protected $CSVError = [];

	/** @var array<int,string> */
	protected $FieldNames = [];

	/** @var bool */
	var $hasHeader = true;

	/** @var array<int,array> */
	var $Fields = [];

	/** @var string[] */
	var $FieldTypes = [];

	/** @var int */
	var $fetchCursor = 0;

	function setDelim(string $delimiter): self{
		switch($delimiter){
			case '\t':
				$this->FieldDelim = "\t";
				break;
			case '':
				$this->FieldDelim = ' ';
				break;
			default:
				$this->FieldDelim = $delimiter;
		}
		return $this;
	}

	function setHeader(bool $hasheader): void{
		$this->hasHeader = $hasheader;
	}

	function setEnclosure(string $enclosure): self{
		$this->Enclosure = $enclosure;
		return $this;
	}

	function parseCSV(): bool{
		if(!$this->CSVData){
			$this->CSVError[] = 'CSV data empty.';
			return false;
		}

		$akt_line = $akt_field = $quote = $field_input = 0;
		$akt_field_value = $last_char = '';
		$head_complete = !$this->hasHeader;

		$end_cc = strlen($this->CSVData);

		for($cc = 0; $cc < $end_cc; $cc++){
			$akt_char = $this->CSVData[$cc];

			if(($akt_char == $this->Enclosure) && ($last_char != "\\")){
				$quote = !$quote;
				$akt_char = '';
			}

			if(!$quote){
				if($akt_char == $this->FieldDelim){
					$field_input = 0;
					$akt_char = '';
					$akt_field++;
					$akt_field_value = '';
				} else if(($akt_char === '\\') && $field_input){
					$field_input++;
					$quote++;
				} else if($akt_char == $this->Enclosure){
					$quote--;

					if($field_input){
						$field_input--;
					} else {
						$field_input++;
					}
				} else if($akt_char === "\n"){
					if($head_complete && (($akt_field + 1) > $this->CSVNumFields())){
						$this->CSVError[] = 'Fehler in <b>Zeile ' . ($akt_line + 2) . '</b>';
					}
					$akt_line++;
					$akt_field = 0;
					if(!$head_complete){
						$akt_line = 0;
					}
					$head_complete = true;
					$akt_char = '';
					$akt_field_value = '';
				}
			}

			$last_char = $akt_char;
			if($akt_char === '\\'){
				$akt_char = '';
			}
			$akt_field_value .= $akt_char;

			if($head_complete){
				$this->Fields[$akt_line][$akt_field] = trim($akt_field_value);
			} else {
				$this->FieldNames[$akt_field] = trim($akt_field_value);
			}
		}

		if(!$akt_field){
			unset($this->Fields[$akt_line]);
		}

		$this->fetchCursor = 0;
		return true;
	}

	function setFile(string $file): bool{
		$this->CSVFile = $file;
		if(file_exists($this->CSVFile) && ($this->CSVFile != 'none') && !empty($this->CSVFile)){
			$this->setData(implode('', file($this->CSVFile)));
			return true;
		}
		$this->CSVError[] = 'The file ' . $file . ' does not exist or is empty.';
		return false;
	}

	/**
	 *
	 * @param string $string
	 * @return static
	 */
	function setData(string $string): self{
		$this->CSVData = $string;
		$this->Fields = $this->FieldTypes = $this->FieldNames = [];
		return $this;
	}

	public function CSVFetchRow(): ?array{
		if($this->fetchCursor < $this->CSVNumRows()){
			$r = $this->Fields[$this->fetchCursor];
			$this->fetchCursor++;
			return $r;
		}
		$this->CSVError[] = "No more data sets.";
		return null;
	}

	function CSVFetchFieldNames(): array{
		if(!$this->FieldNames){
			$cnt = count($this->Fields[0]);
			for($i = 1; $i <= $cnt; $i++){
				$this->FieldNames[] = g_l('modules_customer', '[record_field]') . $i;
			}
		}
		return $this->FieldNames;
	}

	function CSVFieldName(int $field_id): string{
		return $this->FieldNames[$field_id];
	}

	function CSVNumRows(): int{
		return count($this->Fields);
	}

	function CSVNumFields(): int{
		return count($this->FieldNames);
	}

	function setCursor(int $pos): void{
		$this->fetchCursor = $pos;
	}

	function getCursor(): int{
		return $this->fetchCursor;
	}

	function resetCursor(): void{
		$this->setCursor(0);
	}

	function getFieldID(string $search_field): ?int{
		if(!is_array($this->FieldNames)){
			return null;
		}
		foreach($this->FieldNames as $field_id => $field_name){

			if(trim($search_field) == trim($field_name)){
				return $field_id;
			}
		}
		return null;
	}

	function isOK(): bool{
		t_e($this->CSVError);
		return empty($this->CSVError);
	}

}
