<?php
declare(strict_types=1);

/**
 * Zend Framework
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

/**
 * Class for sending eMails via the PHP internal mail() function
 *
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
class we_mail_TransportSendmail extends we_mail_TransportAbstract{
	/**
	 * Subject
	 * @var string
	 * @access public
	 */
	public $subject = null;


	/**
	 * EOL character string
	 * @var string
	 * @access public
	 */
	public $EOL = PHP_EOL;

	/**
	 * error information
	 * @var string
	 */
	protected $errstr = null;

	/**
	 * Constructor.
	 *
	 * @return void
	 */
	public function __construct(){
	}

	/**
	 * Send mail using PHP native mail()
	 *
	 * @access public
	 * @return void
	 * @throws we_exception_mail if mail has not been sent
	 */
	public function sendMail(): void{
		if(!mail($this->recipients, $this->mail->getSubject(), $this->body, $this->header)){
			t_e('mail error; php-mail was unable to send', $this->recipients, $this->header);
			throw new we_exception_mail('Unable to send mail.');
		}
	}

	/**
	 * Format and fix headers
	 *
	 * mail() uses its $to and $subject arguments to set the To: and Subject:
	 * headers, respectively. This method strips those out as a sanity check to
	 * prevent duplicate header entries.
	 *
	 * @access  protected
	 * @param   array<string,string[]> $headers
	 * @return  void
	 * @throws  we_exception_mail
	 */
	protected function prepareHeaders(array $headers): void{
		if(!$this->mail){
			throw new we_exception_mail('_prepareHeaders requires a registered Mail object');
		}

		// mail() uses its $to parameter to set the To: header, and the $subject
		// parameter to set the Subject: header. We need to strip them out.
		if(str_starts_with(PHP_OS, 'WIN')){
			// If the current recipients list is empty, throw an error
			if(empty($this->recipients)){
				throw new we_exception_mail('Missing To addresses');
			}
		} else {
			// All others, simply grab the recipients and unset the To: header
			if(!isset($headers['To'])){
				throw new we_exception_mail('Missing To header');
			}

			unset($headers['To']['append']);
			$this->recipients = implode(',', $headers['To']);
		}

		// Remove recipient header
		unset($headers['To']);

		// Remove subject header, if present
		if(isset($headers['Subject'])){
			unset($headers['Subject']);
		}

		// Prepare headers
		parent::prepareHeaders($headers);

		// Fix issue with empty blank line ontop when using Sendmail Trnasport
		$this->header = rtrim($this->header);
	}

}
