<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c6eb4ae8220e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-10-31 12:21 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_navigation_dynList{

	/**
	 *
	 * @param int $doctypeid
	 * @param int $dirID
	 * @param int[] $categories
	 * @param string $catlogic
	 * @param array<int,array{field:string,order:string}> $order
	 * @param int $count
	 * @param string $field
	 * @return array<int,array{id:int,text:string,field:string}>
	 */
	public static function getDocuments(int $doctypeid, int $dirID, array $categories, string $catlogic, array $order, int $count, string $field): array{
		$db = DB_WE::$inst;
		$cats = array_unique(array_filter($categories));
		$sort = empty($order) ? [] : $order[0];

		$where = array_filter([
			'f.IsFolder=0',
			'f.IsPublished=1',
			($cats ? 'CONCAT(",",f.Category,",") REGEXP ",(' . implode('|', $cats) . '),"' : ''),
			($dirID ? 'f.ParentID=' . $dirID : ''),
			($doctypeid ? ' f.DocType=' . $doctypeid : ''),
			($sort ? 'cs.nHash=x\'' . md5($sort['field']) . '\'' : '')
		]);

		$ret = $db->getAllq('
SELECT
	f.ID AS id,
	f.Text AS text,
	COALESCE(c.Dat,c.BDID) AS field
FROM ' .
			FILE_TABLE . ' f JOIN ' .
			CONTENT_TABLE . ' c ON (f.ID=c.DID AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND c.nHash=x\'' . md5($field ?: 'Title') . '\') ' .
			($sort ? ' JOIN ' .
			CONTENT_TABLE . ' cs ON (cs.DID=f.ID AND cs.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '")' : '' ) . '
WHERE ' .
			implode(' AND ', $where) . '
ORDER BY ' .
			($sort ? 'COALESCE(cs.Dat,cs.BDID) ' . $sort['order'] : 'f.Filename') . '
LIMIT ' . $count);
		'@phan-var array<int,array{id:int,text:string,field:string}> $ret';
		return $ret;
	}

	/**
	 *
	 * @param int $classid
	 * @param int $dirid
	 * @param int[] $categories
	 * @param string $catlogic
	 * @param array<int,array{field:string,order:string}> $sorts
	 * @param int $count
	 * @param string $field
	 * @return array<int,array{id:int,text:string,field:string}>
	 */
	public static function getObjects(int $classid, int $dirid, array $categories, string $catlogic, array $sorts, int $count, string $field): array{
		$select = ($field ? ',obx.' . $field : '');
		$order = [];
		foreach($sorts as $sort){
			$order[] = $sort['field'] . ' ' . $sort['order'];
		}
		$fieldset = self::getObjData($select, $classid, id2path($dirid, OBJECT_FILES_TABLE), $categories, /* $catlogic, */ $order, $count);
		if(!$fieldset){
			return [];
		}
		$ids = [];
		'@phan-var array<int,array{ID:int,Text:string}> $fieldset';
		foreach($fieldset as $data){
			$ids[] = [
				'id' => $data['ID'],
				'text' => $data['Text'],
				'field' => empty($field) ? '' : we_navigation_navigation::encodeSpecChars((string) $data[$field])
			];
		}

		return $ids;
	}

	/**
	 *
	 * @param string $select
	 * @param int $classid
	 * @param string $dirpath
	 * @param int[] $categories
	 * @param string[] $order
	 * @param int $count
	 * @return array<int,array{ID:int,Text:string}>
	 */
	private static function getObjData(string $select, int $classid, string $dirpath, array $categories, array $order, int $count): array{
		$db = DB_WE::$inst;
		$cats = array_unique(array_filter($categories));
		$where = array_filter([
			'`of`.ID!=0',
			'`of`.IsFolder=0',
			'`of`.IsPublished=1', // Bug #4797
			($cats ? 'CONCAT(",",`of`.Category,",") REGEXP ",(' . implode('|', $cats) . '),"' : ''),
			($dirpath != '/' ? '`of`.Path LIKE "' . $db->escape($dirpath) . '%"' : ''),
		]);

		$ret = $db->getAllq('
SELECT `of`.ID,`of`.Text' .
			$select . '
FROM ' .
			OBJECT_X_TABLE . $classid . ' obx JOIN ' .
			OBJECT_FILES_TABLE . ' `of` ON (obx.OF_ID=`of`.ID AND obx.OF_DocumentTable="tblObjectFiles")
WHERE ' .
			implode(' AND ', $where) . '
ORDER BY ' .
			($order ? implode(',', $order) : '`of`.Text' ) . '
LIMIT ' . $count);
		'@phan-var array<int,array{ID:int,Text:string}> $ret';
		return $ret;
	}

	/**
	 *
	 * @param int $dirid
	 * @param int $count
	 * @return array<int,array{id:int,text:string,field:string}>
	 */
	public static function getCatgories(int $dirid, int $count): array{
		$ids = DB_WE::$inst->getAllq('SELECT ID as id,Text as text,Title as field FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . $dirid . ' LIMIT ' . $count);
		'@phan-var array<int,array{id:int,text:string,field:string}> $ids';

		foreach($ids as &$rec){
			$rec['field'] = we_navigation_navigation::encodeSpecChars($rec['field']);
		}

		return $ids;
	}

	public static function getWorkspacesForObject(int $id): array{
		$hash = getHash('SELECT `of`.Workspaces AS ofWs,o.Workspaces AS oWS FROM ' . OBJECT_FILES_TABLE . ' `of` JOIN ' . OBJECT_TABLE . ' o ON `of`.TableID=o.ID WHERE `of`.ID=' . $id);
		if(!$hash){
			return [];
		}
		['ofWs' => $obWsp, 'oWS' => $clWsp] = $hash;
		$ret = id2pathA(we_base_request::filterIntList($obWsp), FILE_TABLE);
		$all = we_contents_objectFile::getPossibleWorkspaces(we_base_request::filterIntList($clWsp));

		return array_intersect_key($ret, $all);
	}

	/**
	 *
	 * @param int $id
	 * @return array<int,string>
	 */
	public static function getWorkspacesForClass(int $id): array{
		return id2pathA(we_base_request::filterIntList(fS('SELECT Workspaces FROM ' . OBJECT_TABLE . ' WHERE ID=' . $id)), FILE_TABLE);
	}

	public static function getFirstDynDocument(int $id): int{
		return fI('SELECT ID,' . (NAVIGATION_DIRECTORYINDEX_NAMES ? '`Text` IN ("' . implode('","', explode(',', NAVIGATION_DIRECTORYINDEX_NAMES)) . '")' : '1') . '  AS o FROM ' . FILE_TABLE . ' WHERE ParentID=' . $id . ' AND IsFolder=0 AND IsDynamic=1 AND IsPublished=1 ORDER BY o DESC LIMIT 1');
	}

	public static function getWorkspaceFlag(int $id): bool{
		$clsid = fI('SELECT `of`.TableID FROM ' . OBJECT_FILES_TABLE . ' `of` WHERE `of`.ID=' . $id);
		$cls = new we_contents_object($clsid);

		return $cls->WorkspaceFlag;
	}

}
