<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 9d9bbc819f45 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-06-30 18:49 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_categoryGet extends we_rpc_cmd{

	function execute(): we_rpc_response{
		$error = [];
		// check for necessory params
		if(!($obj = we_base_request::string('obj'))){
			$error[] = 'Missing field obj';
		}
		if(!we_base_request::string('cats')){
			$error[] = 'Missing field cats';
		}
		if(we_base_request::string('part') === 'table' && (!we_base_request::bool('target'))){
			$error[] = 'Missing target for table';
		}

		if($error){
			$this->resp->setData('error', $error);
			return $this->resp;
		}
		//$part = we_base_request::string('part',"rows");
		$target = we_base_request::string('target', $obj . 'CatTable');
		$catField = we_base_request::string('catfield', '');
		$categories = strtr($this->getCategory($obj, we_base_request::intList('cats'), $catField), ["\r" => '', "\n" => '']);
		$this->resp->setData('elements', [$target => ['innerHTML' => $categories]]);

		return $this->resp;
	}

	/**
	 *
	 * @param string $obj
	 * @param int[] $categories
	 * @param string $catField
	 * @return string
	 */
	private function getCategory(string $obj, array $categories, string $catField = ''): string{
		$cats = new we_chooser_multiDirExtended($categories, 'delete_' . $obj . 'Cat', '', [], '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE);
		$cats->setRowPrefix($obj);
		$cats->setCatField($catField);
		$cats->setOnchangeSetHot(false);
		return $cats->getTableRows();
	}

}
