<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 8e6e89d66713 $
 * $Author: Nils Reibold <reibold@digirezi.de> $
 * $Date: 2021-01-17 18:11 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_copyCustomerFilter extends we_rpc_cmd{

	public const USE_CT = [
		we_base_ContentTypes::FOLDER_DOCUMENT,
		we_base_ContentTypes::FOLDER_OBJECTFILE,
		we_base_ContentTypes::WEDOCUMENT,
		we_base_ContentTypes::OBJECT_FILE,
		we_base_ContentTypes::APPLICATION,
		we_base_ContentTypes::PDF,
		we_base_ContentTypes::AUDIO,
		we_base_ContentTypes::VIDEO,
	];

	function execute(): we_rpc_response{
		$error = "";
		// check for necessary params
		if(! ($id = we_base_request::int('we_cmd', 0, 'id'))){
			$error .= "Call without ID\n";
		}
		if(! ($table = we_base_request::string('we_cmd', '', 'table'))){
			$error .= "Call without Table";
		}

		if($error){
			$this->resp->setData('error', $error);
			return $this->resp;
		}

		// now get all childs of this folder which are open by the user
		$open = $this->getOpenChilds($id, $table);
		if($open){
			$data_filter = [
			'allChilds' => $open,
			'table' => $table,
			'id' => $id,
			'question' => g_l('modules_customerFilter', '[apply_filter_cofirm_close]')
		    ];
			$this->resp->setData('data_filter', $data_filter);
			return $this->resp;
		}

		//$part = we_base_request::string('part',"rows");
		$res = $this->copyFilters($id, $table);

		if($res){
			$this->resp->setData('data',g_l('modules_customerFilter', '[apply_filter_done]'));
		} else {
			$this->resp->setData('error',"Unable to change Customer Filters");
		}
		return $this->resp;
	}

	private function getOpenChilds(int $tableID, string $table): array{

		$db = DB_WE::$inst;
		// FIXME : What about other users?
		$allChildsJS = $db->getAllFirstq('SELECT
								f.ID,
								f.ContentType
							FROM
								' . $db->escape($table) . ' f JOIN
								' . LOCK_TABLE . ' l ON (l.tbl="' . stripTblPrefix($table) . '" AND l.ID=f.ID AND l.UserID=' . $_SESSION['user']['ID'] . ')
							WHERE
								f.ContentType IN("' . implode('","', self::USE_CT) . '" ) AND
								f.Path LIKE CONCAT((SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $tableID . '),"/%")', false);

		return $allChildsJS;

	}

	private function copyFilters(int $tableID, string $table = FILE_TABLE): bool{

		$db = DB_WE::$inst;

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){

			if(!$tableID || !fB('SELECT 1 FROM ' . CUSTOMER_FILTER_TABLE . ' WHERE modelTable="' . stripTblPrefix($table) . '" AND modelId=' . $tableID)){//delete all filters on documents
				$db->query('DELETE filt FROM ' . CUSTOMER_FILTER_TABLE . ' filt JOIN ' . $db->escape($table) . ' f ON (f.ID=filt.modelId AND modelTable="' . stripTblPrefix($table) . '") WHERE f.Path LIKE CONCAT((SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $tableID . '),"/%")');

			} else {
				$db->query('
REPLACE INTO ' . CUSTOMER_FILTER_TABLE . ' (modelTable,modelId,mode,specificCustomers,filter,whiteList,blackList,accessControlOnTemplate,errorDocNoLogin,errorDocNoAccess)(
SELECT
	filt.modelTable,f.ID,filt.mode,filt.specificCustomers,filt.filter,filt.whiteList,filt.blackList,filt.accessControlOnTemplate,filt.errorDocNoLogin,filt.errorDocNoAccess
FROM
	' . $db->escape($table) . ' f,
	' . CUSTOMER_FILTER_TABLE . ' filt
WHERE
	filt.modelTable="' . stripTblPrefix($table) . '" AND
	filt.modelId=' . $tableID . ' AND
	f.ContentType IN("' . implode('","', self::USE_CT) . '" ) AND
	f.Path LIKE CONCAT((SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $tableID . '),"/%")
)');
			}
			return true;
		}
		return false;

			/*FIXME: Does this 3rd cmd still have relevance?
// print the window
		// if we_cmd 3 is set, take filters of that folder as parent!!
		//$idForFilter = we_base_request::int('we_cmd', $id, 3);

		$pb = new we_gui_progressBar(0, 300);
		$pb->addText('&nbsp;', we_gui_progressBar::TOP, 'copyWeDocumentCustomerFilterText');

		echo we_html_tools::getHtmlTop(g_l('modules_customerFilter', '[apply_filter]'), we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/copyWeDocumentFilter.js', '', [
					'id' => 'loadVarFilter',
					'data-filter' => setDynamicVar([
						'allChilds' => $allChildsJS,
						'table' => $table,
						'question' => g_l('modules_customerFilter', '[apply_filter_cofirm_close]'),
						'redirect' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=' . we_base_request::_(we_base_request::RAW, 'we_cmd', '', 0) . '&we_cmd[1]=' . $tableID . '&we_cmd[2]=' . $table ./* ($idForFilter ? '&we_cmd[3]=' . $idForFilter : '' ) . '&startCopy=1'
			])]) .
				we_gui_progressBar::getJSCode(), we_html_element::body([
					'class' => 'weDialogBody', 'onload' => "checkForOpenChilds()"
						], we_html_tools::dialogLayout($pb->getHTML(), g_l('modules_customerFilter', '[apply_filter]'), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")) .
						'<div style="display: none;">
	<iframe style="position: absolute; top: 150px; height: 1px; width: 1px;" name="iframeCopyWeDocumentCustomerFilter" id="iframeCopyWeDocumentCustomerFilter" src="about:blank"></iframe>
</div>')); */
	}
}
