<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 961618297c50 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-08-17 13:15 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_validation extends we_rpc_cmd{
	/** @var we_validation_service */
	private $selectedService = null;

	/** @var string */
	private $selectedName = '';

	/** @var string */
	private $cmd = '';

	function execute(): we_rpc_response{
		switch(($this->cmd = $this->CmdShell->getCmd())){
			case 'saveService':
				$this->validationSaveService();
				break;
			case 'deleteService':
				$this->validationDeleteService();
				break;
			case 'selectService';
				$this->validationSelectService();
				break;
			case 'newService':
				$this->validationNewService();
				break;
			case 'firstService':
			// nothing to do
		}
		$this->processDialogData();

		return $this->resp;
	}

	private function validationSaveService(): void{
		$this->selectedService = new we_validation_service(we_base_request::int('id'), 'custom', we_base_request::string('category'), we_base_request::string('name'), we_base_request::string('host'), we_base_request::_(we_base_request::FILE, 'path'), we_base_request::string('s_method'), we_base_request::string('varname'), we_base_request::string('checkvia'), we_base_request::string('ctype'), we_base_request::string('additionalVars'), we_base_request::string('fileEndings'), we_base_request::bool('active'));
		if(we_validation_base::saveService($this->selectedService)){
			$this->resp->setData('success', 1);
			$this->resp->setData('message', ['text' => g_l('validation', '[edit_service][saved_success]'), 'type' => we_base_util::WE_MESSAGE_NOTICE]);
		} else {
			$this->resp->setData('success', 0);
			$this->resp->setData('message', ['text' => g_l('validation', '[edit_service][saved_failure]') . (we_validation_base::$error ? '\n' . we_validation_base::$error : ''),
				'type' => we_base_util::WE_MESSAGE_ERROR]);
		}
	}

	private function validationDeleteService(): void{
		$service = new we_validation_service(we_base_request::int('id'), 'custom', we_base_request::string('category'), we_base_request::string('name'), we_base_request::string('host'), we_base_request::_(we_base_request::FILE, 'path'), we_base_request::string('s_method'), we_base_request::string('varname'), we_base_request::string('checkvia'), we_base_request::string('ctype'), we_base_request::string('additionalVars'), we_base_request::string('fileEndings'), we_base_request::bool('active'));
		if(we_validation_base::deleteService($service)){
			$this->resp->setData('success', 1);
			$this->resp->setData('messsage', ['text' => g_l('validation', '[edit_service][delete_success]'), 'type' => we_base_util::WE_MESSAGE_NOTICE]);
		} else {
			$this->selectedService = $service;
			$this->resp->setData('success', 0);
			$this->resp->setData('message', ['text' => g_l('validation', '[edit_service][delete_failure]'), 'type' => we_base_util::WE_MESSAGE_ERROR]);
		}
	}

	private function validationSelectService(): void{
		$this->selectedName = we_base_request::string('validationService');
	}

	private function validationNewService(): void{
		$this->selectedService = new we_validation_service(0, 'custom', 'xhtml', g_l('validation', '[edit_service][new]'), 'www.example', '/path', 'get', 'varname', 'url', 'text/html', '', '.html', true);
	}

	private function processDialogData(): void{
		$services = we_validation_base::getValidationServices(we_base_WE::getWeDoc()->Extension, 'edit');
		if($this->cmd === 'newService' && $this->selectedService){
			$services[] = $this->selectedService;
		}

		if($services){
			$selectArr = [];
			foreach($services as $service){
				$selectArr[$service->getName()] = htmlentities($service->name);
				$this->selectedService = $this->selectedService ?: $service;

				if($service->getName() === $this->selectedName){
					$this->selectedService = $service;
				}
			}
		} else {
			$selectArr = [];
		}

		$this->resp->setData('services', $selectArr);
		$this->resp->setData('selected', ($this->selectedService ? $this->selectedService->getName() : ''));
		$this->resp->setData('data', $this->selectedService);
	}

}
