<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c030d2b9427c $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-04-08 11:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_search extends we_search_base{
	const TAB_DOCUMENTS = 1;
	const TAB_TEMPLATES = 2;
	const TAB_MEDIA = 5;
	const TAB_ADVANCED = 3;
	const TABLE_FILE = 1;
	const TABLE_TEMPLATES = 2;
	const TABLE_OBJECTFILES = 3;
	const TABLE_OBJECT = 4;
	const TABLE_VERSION = 5;
	private const usedMediaLinksDefault = [
		'accessible' => [],
		'notaccessible' => [],
		'groups' => []
	];

	//for doclist!
	/** @var int number of searchfield-rows	 */
	var $height;

	/** @var string default order of the result columns	 */
	var $order = 'Text';

	/*	 * @var int default number of rows of the result columns	 */
	var $anzahl = 10;

	/*	 * @var int mode if the searchfields are displayed or not, default = 0 (not displayed)	 */
	var $mode = 0;

	/** @var string set view, either iconview (1) or listview (0, default)	 */
	var $setView = we_search_view::VIEW_LIST;

	/**  @var array with fields to search in	 */
	var $searchFields = [];

	/** @var array with operators	 */
	var $location = [];

	/** @var array with fields to search for	 */
	var $search = [];
	//private $collectionMetaSearches = [];

	/** @var int[] */
	private $usedMedia = [];

	/** @var array{accessible:array<string,mixed>,notaccessible:array<string,mixed>,groups:array<string,mixed>} */
	private $usedMediaLinks = self::usedMediaLinksDefault;

	/** @var int */
	public $founditems = 0;

	/** @var we_modules_view */
	public $View;

	public function __construct(we_modules_view $view){
		parent::__construct();
		$this->View = $view;
	}

	/**
	 *
	 * @param we_search_modelBase $model
	 * @param string $whichSearch
	 * @return we_search_resultBase[]
	 */
	function searchProperties(we_search_modelBase $model, string $whichSearch): array{//FIXME: we should never return arrays leave this in search table!
		$results = $versionsFound = $saveArrayIds = [];
		'@phan-var array{Table:string,foundInVersions?:array{string,int,int}[],Nr:int,isNr:int,UID:int,docID:int,docTable:string,Text:string,Path:string,ParentID:int,IsFolder:int,IsProtected:int,temp_template_id:int,TemplateID:int,ContentType:string,SiteTitle:string,CreatorID:int,Creation_Date:string,Mod_Date:string,Publish_Date:?string,Extension:?string,TableID:int,VersionID:int,media_alt:?string,media_title:?string,media_filesize:int,IsUsed:int,remTable:?string,remCT:?string,remClass:int,IsPublished:int,IsModified:int}[] $results';
		$_SESSION['weS']['search']['foundItems' . $whichSearch] = 0; // will be obsolete
		$searchFields = $model->getProperty('currentSearchFields');
		$searchText = array_map('trim', $model->getProperty('currentSearch'));
		$location = $model->getProperty('currentLocation');
		$folderID = $model->getPropertyInt('currentFolderID');
		$order = (string) $model->getProperty('currentOrder');
		$view = (string) $model->getProperty('currentSetView');
		$searchstart = $model->getPropertyInt('currentSearchstart');
		$anzahl = $model->getPropertyInt('currentAnzahl');
		$tables = $model->getProperty('currentSearchTables');
		$searchForField = $model->getProperty('currentSearchForField');
		$searchForContentType = $model->getProperty('currentSearchForContentType');

		// FIXME: set whichserach as new model property once in the beginning and forget about such rubish like that...
		$tab = we_base_request::int('tab', 0) ?: (we_base_request::int('tabnr', 0) ?: ($model->getPropertyInt('activTab') ?: we_search_frames::TAB_DOCUMENTS));
		if(we_base_request::bool('we_cmd', true, 'newSearch')){
			if(
				($model->IsFolder != 1 && (($whichSearch === we_search_view::SEARCH_DOCS && $tab === 1) || ($whichSearch === we_search_view::SEARCH_TMPL && $tab === 2) ||
				($whichSearch === we_search_view::SEARCH_ADV && $tab === 3)) || ($whichSearch === we_search_view::SEARCH_MEDIA && $tab === 5)) ||
				(we_base_request::int('cmdid')) ||
				(($view = we_base_request::string('view')) === 'searchGetSearchResult')
			){
				$this->createTempTable();
				$opAND = in_array($whichSearch, [we_search_view::SEARCH_ADV, we_search_view::SEARCH_MEDIA]);

				foreach($tables as $table){
					$this->settable($table);
					$workspaces = we_users_util::get_ws($table);
					'@phan-var int[] $workspaces';

					if(($searchWhere = $this->getSearchString($table, $tables, $opAND, $searchFields, $whichSearch, $searchForField, $searchText, $location, $searchForContentType, $view))){
						$whereQuery = $searchWhere;

						if($folderID){
							$whereQuery[] = we_search_search::ofFolderAndChildsOnly([$folderID], $table);
						}

						if($table === VERSIONS_TABLE){
							$workspacesTblFile = we_users_util::get_ws(FILE_TABLE);
							'@phan-var int[] $workspacesTblFile';
							if(we_modules_info::isActive(we_modules_info::OBJECT)){
								$workspacesObjFile = we_users_util::get_ws(OBJECT_FILES_TABLE);
								'@phan-var int[] $workspacesObjFile';
							}
						}

						if($workspaces){
							$whereQuery[] = we_search_search::ofFolderAndChildsOnly($workspaces, $table);
						}

						//query for restrict users for FILE_TABLE, VERSIONS_TABLE AND OBJECT_FILES_TABLE
						switch($table){
							case FILE_TABLE:
							case OBJECT_FILES_TABLE :
							case OBJECT_TABLE :
								$whereQuery[] = '(WETABLE.RestrictOwners IN(0,' . we_users_user::getID() . ') OR FIND_IN_SET(' . we_users_user::getID() . ',WETABLE.Owners))';
								break;

							case VERSIONS_TABLE:
								$_SESSION['weS']['search']['onlyObjects'] = true;
								$_SESSION['weS']['search']['onlyDocs'] = true;
								$_SESSION['weS']['search']['ObjectsAndDocs'] = true;

								$_SESSION['weS']['search']['onlyObjectsRestrUsersWhere'] = ' AND (WETABLE.RestrictOwners=0 OR WETABLE.CreatorID=' . we_users_user::getID() . ' OR FIND_IN_SET(' . we_users_user::getID() . ',WETABLE.Owners))' .
									(empty($workspacesObjFile) ? '' : ' AND ' . (implode(' AND ', $searchWhere) . ' AND ' . self::ofFolderAndChildsOnly($workspacesObjFile, OBJECT_FILES_TABLE)));

								$_SESSION['weS']['search']['onlyDocsRestrUsersWhere'] = ' AND (WETABLE.RestrictOwners=0 OR WETABLE.CreatorID=' . we_users_user::getID() . ' OR FIND_IN_SET(' . we_users_user::getID() . ',WETABLE.Owners))' .
									(empty($workspacesTblFile) ? '' : ' AND ' . (implode(' AND ', $searchWhere) . ' AND ' . self::ofFolderAndChildsOnly($workspacesTblFile, FILE_TABLE)));

								/* 	if(!$isCheckedFileTable && $isCheckedObjFileTable){
								  $_SESSION['weS']['search']['onlyDocs'] = false;
								  $whereQuery .= ' AND ' . DB_WE::$inst->escape($table) . '.documentTable="' . OBJECT_FILES_TABLE . '" ';
								  $_SESSION['weS']['search']['ObjectsAndDocs'] = false;
								  }
								  if($isCheckedFileTable && !$isCheckedObjFileTable){
								  $_SESSION['weS']['search']['onlyObjects'] = false;
								  $whereQuery .= ' AND ' . DB_WE::$inst->escape($table) . '.documentTable="' . FILE_TABLE . '" ';
								  $_SESSION['weS']['search']['ObjectsAndDocs'] = false;
								  } */
								break;
						}
						$where = implode(' AND ', array_filter($whereQuery));
						$this->where = $where;
						$this->insertInTempTable($where, $table);

						// when MediaSearch add attrib_alt, attrib_title, IsUsed to SEARCH_TEMP_TABLE
						if($whichSearch === we_search_view::SEARCH_MEDIA){
							$this->insertMediaAttribsToTempTable();
						}
					}
				}
			}
		}
		foreach($this->selectFromTempTable($searchstart, $anzahl, $order) as $cur){
			if(!empty($cur['VersionID'])){
				$versionsFound[$cur['VersionID']] = [
					$cur['ContentType'],
					$cur['docID'],
					$cur['VersionID']
				];
			}
			if(empty($cur['VersionID']) && !empty($cur['ContentType']) && !empty($cur['docID']) && !isset($saveArrayIds[$cur['ContentType']][$cur['docID']])){
				$saveArrayIds[$cur['ContentType']][$cur['docID']] = $cur['docID'];
				$results[] = array_merge(['Table' => '', 'foundInVersions' => []], $cur);
			}
		}

		// FIXME: is this obsolete?
		foreach($versionsFound as $v){
			foreach($results as &$val){
				if(isset($val['foundInVersions']) && isset($val['docID']) && $val['docID'] == $v[1] && $val['ContentType'] == $v[0]){
					$val['foundInVersions'][] = $v[2];
				}
			}

			foreach($this->selectFromTempTable($searchstart, $anzahl, $order) as $cur){
				if(!isset($saveArrayIds[$cur['ContentType']][$cur['VersionID']])){
					$saveArrayIds[$cur['ContentType']][$cur['VersionID']] = $cur['VersionID'];
					$results[] = array_merge(['Table' => ''], $cur);
				}
			}
		}

		$_SESSION['weS']['search']['foundItems' . $whichSearch] = $this->founditems = $this->getResultCount();

		if($this->founditems === 0){
			return [];
		}
		if($versionsFound){
			$versionFields = 'ID, ID AS VersionID, version AS Text, documentID as docID, documentTable AS docTable, logDate AS Creation_Date, active AS VersionActive, "' . VERSIONS_TABLE . '" AS "Table"';
			$versions = DB_WE::$inst->getAllFirstq('SELECT ' . $versionFields . ' FROM ' . VERSIONS_TABLE . ' WHERE ID IN(' . implode(',', array_keys($versionsFound)) . ') ORDER BY version DESC', true, MYSQLI_ASSOC);
			$versionsFull = array_map(function ($v){
				return [
				'VersionID' => (int) $v['VersionID'],
				'Text' => (string) $v['Text'],
				'docID' => (int) $v['docID'],
				'docTable' => (string) $v['docTable'],
				'Creation_Date' => (string) $v['Creation_Date'],
				'VersionActive' => (bool) $v['VersionActive'],
				'Table' => (string) $v['Table'],
				];
			}, $versions);
		}

		$allResults = [];
		foreach($results as $v){
			if(!empty($v['foundInVersions'])){
				rsort($v['foundInVersions']);
			}
			$v['Description'] = '';
			switch(addTblPrefix((string) $v['docTable'])){
				case FILE_TABLE:
					$v['Description'] = fS('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DID=' . intval($v['docID']) . ' AND c.nHash=unhex(md5("Description")) AND c.DocumentTable="' . ($v['IsPublished'] && !$v['IsModified'] ? stripTblPrefix(FILE_TABLE) : TEMPORARY_DOCTARGET) . '"');
					break;
				default:
					$v['Description'] = '';
			}
			$allResults[] = $v;
			if(!empty($v['foundInVersions'])){
				foreach($v['foundInVersions'] as $versionID){
					if(!empty($versionsFull[(int) $versionID])){
						$allResults[] = array_merge($v, $versionsFull[(int) $versionID]);
					}
				}
			}
		}

		// fill results into result objects
		$resultObjects = [];
		foreach($allResults as $result){
			$resultObject = null;
			switch($whichSearch){
				case we_search_view::SEARCH_DOCS:
					$resultObject = new we_search_resultDocuments($this);
					break;
				case we_search_view::SEARCH_TMPL:
					$resultObject = new we_search_resultTemplates($this);
					break;
				case we_search_view::SEARCH_MEDIA:
					$resultObject = new we_search_resultMedia($this);
					$resultObject->setNumMedialinks($model->getPropertyInt('currentAnzahlMedialinks'));
					break;
				case we_search_view::SEARCH_ADV:
					$resultObject = $result['Table'] === VERSIONS_TABLE ? new we_search_resultVersion($this) : new we_search_resultAdvanced($this);
					break;
				default:
					continue 2;
			}

			$resultObject->setTable((string) $result['Table'])
				->setFoundInVersion($result['foundInVersions'] ?? [])
				->setUID((int) $result['UID'])
				->setDocID((int) $result['docID'])
				->setDocTable((string) $result['docTable'])
				->setText((string) $result['Text'])
				->setPath((string) $result['Path'])
				->setParentID(((int) $result['ParentID']))
				->setIsFolder((bool) $result['IsFolder'])
				->setIsProtected((bool) $result['IsProtected'])
				->setTempTemplateId((int) $result['temp_template_id'])
				->setTemplateID((int) $result['TemplateID'])
				->setContentType((string) $result['ContentType'])
				->setSiteTitle((string) $result['SiteTitle'])
				->setDescription((string) $result['Description'])
				->setCreatorID((int) $result['CreatorID'])
				->setCreationDate((string) $result['Creation_Date'])
				->setModDate((string) $result['Mod_Date'])
				->setPublishDate((string) $result['Publish_Date'])
				->setExtension((string) $result['Extension'])
				->setTableID((int) $result['TableID'])
				->setVersionID((int) $result['VersionID'])
				->setVersionActive((bool) ($result['VersionActive'] ?? false))
				->setMediaAlt((string) $result['media_alt'])
				->setMediaTitle((string) $result['media_title'])
				->setMediaFilesize((int) $result['media_filesize'])
				->setIsUsed((bool) $result['IsUsed'])
				->setRemTable((string) $result['remTable'])
				->setRemCT((string) $result['remCT'])
				->setRemClass((int) $result['remClass'])
				->setIsPublished((bool) $result['IsPublished'])
				->setIsModified((bool) $result['IsModified']);
			/*
			  // seems to be unused
			  ->setNr((string) $result['Nr'])
			  ->setIsNr((bool) $result['isNr']);
			 *
			 */
			$resultObject->processHtml();
			$resultObjects[] = $resultObject;
		}

		return $resultObjects;
	}

	function getModFields(): array{
		$modFields = [];
		foreach(we_versions_version::MOD_FIELDS as $k){
			if($k != 'status'){
				$modFields[$k] = $k;
			}
		}

		return $modFields;
	}

	/**
	 *
	 * @return array<int,string>
	 */
	function getUsers(): array{
		//@phan-suppress-next-line PhanPartialTypeMismatchReturn
		return DB_WE::$inst->getAllFirstq('SELECT ID,username FROM ' . USER_TABLE . ' WHERE Type="user"' .
				(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? '' : ' AND ParentID=' . $_SESSION['user']['Group']) . ' ORDER BY username', false);
	}

	function getFields(string $whichSearch = '', int $tableID = 0): array{
		$tableFields = array_filter([
			'ID' => g_l('searchtool', '[ID]'),
			'Text' => g_l('searchtool', '[text]'),
			'Path' => g_l('searchtool', '[Path]'),
			'ParentIDDoc' => we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE]) ? g_l('searchtool', '[ParentIDDoc]') : '',
			'ParentIDObj' => we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE]) ? g_l('searchtool', '[ParentIDObj]') : '',
			'ParentIDTmpl' => g_l('searchtool', '[ParentIDTmpl]'),
			'temp_template_id' => g_l('searchtool', '[temp_template_id]'),
			'MasterTemplateID' => g_l('searchtool', '[MasterTemplateID]'),
			'ContentType' => g_l('searchtool', '[ContentType]'),
			'HasReferenceToID' => g_l('searchtool', '[HasReferenceToID]'),
			'temp_category' => we_modules_info::isActive(we_modules_info::CATEGORY) ? g_l('searchtool', '[temp_category]') : '',
			'CreatorID' => g_l('searchtool', '[CreatorID]'),
			'CreatorName' => g_l('searchtool', '[CreatorName]'),
			'WebUserID' => g_l('searchtool', '[WebUserID]'),
			'WebUserName' => g_l('searchtool', '[WebUserName]'),
			'Content' => g_l('searchtool', '[Content]'),
			'Status' => g_l('searchtool', '[Status]'),
			'Speicherart' => g_l('searchtool', '[Speicherart]'),
			'Publish_Date' => g_l('searchtool', '[Published]'),
			'Creation_Date' => g_l('searchtool', '[CreationDate]'),
			'Mod_Date' => g_l('searchtool', '[ModDate]'),
			//'allModsIn' => g_l('versions', '[allModsIn]'),
			'modifierID' => g_l('versions', '[modUser]')
		]);

		switch($whichSearch){
			case we_search_view::SEARCH_MEDIA:
				$tableFields = array_merge($this->getFieldsMeta(true), $tableFields);
				unset(
					$tableFields['Text'],
		 $tableFields['ParentIDObj'],
		 $tableFields['ParentIDTmpl'],
		 $tableFields['temp_template_id'],
		 $tableFields['MasterTemplateID'],
		 $tableFields['WebUserID'],
		 $tableFields['WebUserName'],
		 $tableFields['Content'],
		 $tableFields['Status'],
		 $tableFields['Speicherart'],
		 $tableFields['Published'],
		 $tableFields['HasReferenceToID']);
				break;
			case we_doclist_view::DOCLIST_DOCUMENTS:
			case we_doclist_view::DOCLIST_TEMPLATES:
			case we_doclist_view::DOCLIST_OBJECTFILES:
			case we_doclist_view::DOCLIST_COLLECTION:
				unset(
					$tableFields['Path'],
		 $tableFields['ParentIDDoc'],
		 $tableFields['ParentIDObj'],
		 $tableFields['ParentIDTmpl'],
		 $tableFields['MasterTemplateID'],
		 $tableFields['HasReferenceToID']
				);
		}

		if(we_editor_functions::isSeeMode()){
			unset($tableFields['ParentIDTmpl']);
		}

		if(!we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])){
			unset(
				$tableFields['ParentIDTmpl'],
		$tableFields['temp_template_id'],
		$tableFields['MasterTemplateID']);
		}

		return $tableFields;
	}

	function getFieldsMeta(bool $usePrefix = false, bool $getTypes = false): array{
		$ret = [
			($usePrefix ? 'meta__' : '') . we_contents_root::TITLE_FIELD => ($getTypes ? 'text' : g_l('searchtool', '[metadata][field]')),
			($usePrefix ? 'meta__' : '') . we_contents_root::DESCRIPTION_FIELD => ($getTypes ? 'text' : g_l('searchtool', '[metadata][description]')),
			($usePrefix ? 'meta__' : '') . we_contents_root::KEYWORD_FIELD => ($getTypes ? 'text' : g_l('searchtool', '[metadata][keyword]')),
		];

		$all = DB_WE::$inst->getAllFirstq('SELECT tag,type FROM ' . METADATA_TABLE, false);
		foreach($all as $tag => $type){
			$ret[($usePrefix ? 'meta__' : '') . $tag] = $getTypes ? $type : 'Metadaten: ' . $tag;
		}

		return $ret;
	}

	function getFieldsStatus(): array{
		return [
			'jeder' => g_l('searchtool', '[jeder]'),
			'geparkt' => g_l('searchtool', '[geparkt]'),
			'veroeffentlicht' => g_l('searchtool', '[veroeffentlicht]'),
			'geaendert' => g_l('searchtool', '[geaendert]'),
			'veroeff_geaendert' => g_l('searchtool', '[veroeff_geaendert]'),
			'geparkt_geaendert' => g_l('searchtool', '[geparkt_geaendert]'),
			'deleted' => g_l('searchtool', '[deleted]')
		];
	}

	function getFieldsSpeicherart(): array{
		return [
			'jeder' => g_l('searchtool', '[jeder]'),
			'dynamisch' => g_l('searchtool', '[dynamisch]'),
			'statisch' => g_l('searchtool', '[statisch]')
		];
	}

	static function getLocation(string $whichField = '', string $select = '', int $size = 1): array{
		switch($whichField){
			default:
				return [
					self::OP_IS => g_l('searchtool', '[IS]'),
					self::OP_CONTAIN => g_l('searchtool', '[CONTAIN]'),
					self::OP_START => g_l('searchtool', '[START]'),
					self::OP_END => g_l('searchtool', '[END]'),
					self::OP_LO => g_l('searchtool', '[<]'),
					self::OP_LEQ => g_l('searchtool', '[<=]'),
					self::OP_HEQ => g_l('searchtool', '[>=]'),
					self::OP_HI => g_l('searchtool', '[>]'),
					self::OP_IN => g_l('searchtool', '[IN]'),
				];
			case 'text':
				return [
					self::OP_IS => g_l('searchtool', '[IS]'),
					self::OP_CONTAIN => g_l('searchtool', '[CONTAIN]'),
					self::OP_START => g_l('searchtool', '[START]'),
					self::OP_END => g_l('searchtool', '[END]'),
					self::OP_IN => g_l('searchtool', '[IN]'),
				];
			case 'date':
				return [
					self::OP_IS => g_l('searchtool', '[IS]'),
					self::OP_LO => g_l('searchtool', '[<]'),
					self::OP_LEQ => g_l('searchtool', '[<=]'),
					self::OP_HEQ => g_l('searchtool', '[>=]'),
					self::OP_HI => g_l('searchtool', '[>]'),
					self::OP_IN => g_l('searchtool', '[IN]'),
				];
			case 'meta':
				return [self::OP_IS => g_l('searchtool', '[IS]')];
		}
	}

	/**
	 * @return int[]
	 */
	public function getUsedMedia(): array{
		return $this->usedMedia;
	}

	/**
	 *
	 * @return array{accessible:array<string,mixed>,notaccessible:array<string,mixed>,groups:array<string,mixed>}
	 */
	public function getUsedMediaLinks(): array{
		return $this->usedMediaLinks;
	}

	/**
	 *
	 * @return array<int,string>
	 */
	function getDoctypes(): array{
		$dtq = we_doctype_docTypes::getDoctypeQuery();
		return DB_WE::$inst->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false); //@phan-suppress-current-line PhanPartialTypeMismatchReturn
	}

	private function searchInTitle(string $keyword, string $table): string{
		return ' (WETABLE.ID IN (SELECT ct.DID FROM ' . CONTENT_TABLE . ' ct WHERE ct.nHash=unhex(md5("Title")) AND ct.Dat LIKE "%' . DB_WE::$inst->escape(trim($keyword)) . '%" AND ct.DocumentTable IN ("' . stripTblPrefix($table) . '","' . TEMPORARY_DOCTARGET . '")) )';
	}

	protected function searchCategory(string $keyword, string $table): string{
		$query = '';
		if($keyword){
			switch($table){
				case TEMPLATES_TABLE:
					return ' 0 ';
				case FILE_TABLE:
					$query = 'SELECT ID,IF(temp_category,temp_category,Category) FROM ' . FILE_TABLE . ' WHERE ((Category!=NULL OR Category!="") AND Publish_Date>=Mod_Date AND IsPublished=1) OR (Publish_Date<Mod_Date AND (temp_category!= NULL OR temp_category!= "")) ';
					break;
				case VERSIONS_TABLE:
					$query = 'SELECT ID,Category FROM ' . VERSIONS_TABLE . ' WHERE Category!= NULL OR Category!= ""';
					break;
				case OBJECT_TABLE :
					$query = 'SELECT ID,DefaultCategory FROM ' . OBJECT_TABLE . ' WHERE DefaultCategory!= NULL OR DefaultCategory!= ""';
					break;
				case OBJECT_FILES_TABLE :
					$query = 'SELECT ID,Category FROM ' . OBJECT_FILES_TABLE . ' WHERE Category!=NULL OR Category!="" AND Publish_Date>=Mod_Date AND IsPublished=1';
					break;
			}
		} else {
			switch($table){
				case TEMPLATES_TABLE:
					return ' 0 ';
				case FILE_TABLE:
					$query = 'SELECT ID FROM ' . FILE_TABLE . ' WHERE ((Category=NULL OR Category="") AND Publish_Date>=Mod_Date AND IsPublished=1) OR (Publish_Date<Mod_Date AND (temp_category=NULL OR temp_category="")) ';
					break;
				case VERSIONS_TABLE:
					$query = 'SELECT ID FROM ' . VERSIONS_TABLE . ' WHERE Category=NULL OR Category=""';
					break;
				case OBJECT_TABLE :
					$query = 'SELECT ID FROM ' . OBJECT_TABLE . ' WHERE DefaultCategory= NULL OR DefaultCategory=""';
					break;
				case OBJECT_FILES_TABLE :
					$query = 'SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Category=NULL OR Category="" AND Publish_Date>=Mod_Date AND IsPublished=1';
					break;
			}
		}

		$res = DB_WE::$inst->getAllFirstq($query, false);

		if(!$keyword){
			return 'WETABLE.ID IN(' . $query . ')';
		}

		$res2 = [];
		foreach($res as $k => $v){
			$res2[$k] = array_filter(explode(',', (string) $v));
		}

		$whereIn = [];

		$keyword = path_to_id($keyword, CATEGORY_TABLE);

		foreach($res2 as $k => $v){
			foreach($v as $v2){
				//look if the value is numeric
				if(preg_match('=^[0-9]+$=i', $v2)){
					if($v2 == $keyword){
						$whereIn[] = intval($k);
					}
				}
			}
		}

		return ($whereIn ? 'WETABLE.ID IN(' . implode(',', $whereIn) . ')' : ' 0 ');
	}

	protected function searchSpecial(string $keyword, string $searchFields, string $searchlocation): string{
		$db = DB_WE::$inst;
		switch($searchFields){
			default:
			case 'CreatorName':
				$table = USER_TABLE;
				$field = 'username';
				$fieldFileTable = 'CreatorID';
				break;
			case 'WebUserName':
				$table = CUSTOMER_TABLE;
				$field = 'Username';
				$fieldFileTable = 'WebUserID';
				break;
		}

		switch($searchlocation){
			case self::OP_END :
				$searching = ' LIKE "%' . $db->escape($keyword) . '" ';
				break;
			case self::OP_START :
				$searching = ' LIKE "' . $db->escape($keyword) . '%" ';
				break;
			case self::OP_IS :
				$searching = " = '" . $db->escape($keyword) . "' ";
				break;
			case self::OP_IN:
				$searching = ' IN ("' . implode('","', array_map('trim', explode(',', $keyword))) . '") ';
				break;
			case self::OP_LO :
				$searching = ' < "' . $db->escape($keyword) . '" ';
				break;
			case self::OP_LEQ :
				$searching = ' <= "' . $db->escape($keyword) . '" ';
				break;
			case self::OP_HI :
				$searching = ' > "' . $db->escape($keyword) . '" ';
				break;
			case self::OP_HEQ :
				$searching = ' >= "' . $db->escape($keyword) . '" ';
				break;
			default:
				$searching = ' LIKE "%' . $db->escape($keyword) . '%" ';
				break;
		}

		$userIDs = $db->getAllq('SELECT ID FROM ' . $db->escape($table) . ' WHERE ' . $field . ' ' . $searching, true);

		return ($userIDs ? $fieldFileTable . ' IN (' . implode(',', $userIDs) . ')' : ' 0 ');
	}

	/* function addToSearchInMeta(string $search, string $field, string $location): void{
	  $this->collectionMetaSearches[] = [$search, $field, $location];
	  } */

	private function searchInAllMetas(string $keyword, string $table = ''): string{
		if($table !== FILE_TABLE){// FIXME: actually no meta search on Versions or unpublished docs!!
			return '0';
		}
		$names = [];
		foreach($this->getFieldsMeta() as $k => $v){
			if($v[0] && $v[1]){
				$names[] = (string) $k;
			}
		}
		if(empty($names)){
			return '0';
		}
		$n = array_map('md5', $names);
		$IDs = DB_WE::$inst->getAllq('SELECT c.DID FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND (c.nHash IN (x\'' . implode('\',x\'', $n) . '\') AND c.Dat LIKE "%' . DB_WE::$inst->escape($keyword) . '%")', true);

		return $IDs ? 'WETABLE.ID IN (' . implode(',', $IDs) . ')' : '0';
	}

	private function searchInMeta(string $keyword, string $searchField, string $searchlocation, string $table): string{
		if($table !== FILE_TABLE){// FIXME: actually no meta search on Versions or unpublished docs!!
			return '';
		}
		$db = DB_WE::$inst;
		$reverse = false;
		switch($searchlocation){
			case self::OP_END :
				$searching = ' LIKE "%' . $db->escape($keyword) . '" ';
				break;
			case self::OP_START :
				$searching = ' LIKE "' . $db->escape($keyword) . '%" ';
				break;
			case self::OP_IS :
				$reverse = $keyword === '##EMPTY##';
				$searching = " = '" . $db->escape($keyword) . "' ";
				break;
			case self::OP_IN:
				$searching = ' IN ("' . implode('","', array_map('trim', explode(',', $keyword))) . '") ';
				break;
			case self::OP_LO:
				$searching = ' < "' . $db->escape($keyword) . '" ';
				break;
			case self::OP_LEQ:
				$searching = ' <= "' . $db->escape($keyword) . '" ';
				break;
			case self::OP_HI:
				$searching = ' > "' . $db->escape($keyword) . '" ';
				break;
			case self::OP_HEQ:
				$searching = ' >= "' . $db->escape($keyword) . '" ';
				break;
			default:
				$searching = ' LIKE "%' . $db->escape(trim($keyword)) . '%" ';
				break;
		}

		$IDs = $db->getAllq('SELECT c.DID FROM ' . CONTENT_TABLE . ' c WHERE c.nHash=x\'' . md5($searchField) . '\' ' . ($reverse ? '' : 'AND c.Dat ' . $searching) . ' AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '"', true);

		return $IDs ? 'WETABLE.ID ' . ($reverse ? 'NOT' : '') . ' IN (' . implode(',', $IDs) . ')' : ' 0 ';
	}

	protected function getStatusFiles(string $status, string $table): string{//IMI: IMPORTANT: veröffentlichungsstatus grenzt die contenttypes auf djenigen ein, die solch einen status haben!!
		// also kann auch beim verlinkungsstatus auf media-docs eingegrenzt werden
		switch($status){
			case 'jeder' :
				$ret = 'WETABLE.ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '","' . we_base_ContentTypes::OBJECT_FILE . '")';
				break;
			case 'geparkt' :
				$ret = ($table === VERSIONS_TABLE ?
					'v.status="unpublished"' :
					'(WETABLE.IsPublished=0 AND WETABLE.ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '","' . we_base_ContentTypes::OBJECT_FILE . '"))');
				break;
			case 'veroeffentlicht' :
				$ret = ($table === VERSIONS_TABLE ?
					'v.status="published"' :
					'(WETABLE.Publish_Date>=WETABLE.Mod_Date AND WETABLE.IsPublished=1 AND WETABLE.ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '","' . we_base_ContentTypes::OBJECT_FILE . '"))');
				break;
			case 'geaendert' :
				$ret = ($table === VERSIONS_TABLE ?
					'v.status="saved"' :
					'(WETABLE.Publish_Date<WETABLE.Mod_Date AND WETABLE.IsPublished=1 AND WETABLE.ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '","' . we_base_ContentTypes::OBJECT_FILE . '"))');
				break;
			case 'veroeff_geaendert' :
				$ret = '((WETABLE.Publish_Date>=WETABLE.Mod_Date OR WETABLE.Publish_Date < WETABLE.Mod_Date AND WETABLE.IsPublished=1) AND WETABLE.ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '","' . we_base_ContentTypes::OBJECT_FILE . '") )';
				break;
			case 'geparkt_geaendert' :
				$ret = ($table === VERSIONS_TABLE ?
					'v.status!="published"' :
					'((WETABLE.IsPublished=0 OR WETABLE.Publish_Date<WETABLE.Mod_Date) AND WETABLE.ContentType IN ("' . we_base_ContentTypes::WEDOCUMENT . '","' . we_base_ContentTypes::HTML . '","' . we_base_ContentTypes::OBJECT_FILE . '") )');
				break;
			case 'dynamisch' :
				$ret = ($table !== FILE_TABLE && $table !== VERSIONS_TABLE ? '' :
					'(WETABLE.IsDynamic=1 AND WETABLE.ContentType="' . we_base_ContentTypes::WEDOCUMENT . '")');
				break;
			case 'statisch' :
				$ret = ($table !== FILE_TABLE && $table !== VERSIONS_TABLE ? '' :
					'(WETABLE.IsDynamic=0 AND WETABLE.ContentType="' . we_base_ContentTypes::WEDOCUMENT . '")');
				break;
			case 'deleted' :
				$ret = ($table !== VERSIONS_TABLE ? '' :
					'v.status="deleted"' );
				break;
			default:
			case 'default':
				$ret = '1';
		}

		return $ret;
	}

	private function searchModifier(int $id): string{
		return ($id ? 'WETABLE.modifierID=' . $id : '');
	}

	private function searchModFields(string $text): string{
		if(!in_array($text, we_versions_version::MOD_FIELDS)){
			return ' 0 ';
		}

		$ids = DB_WE::$inst->getAllq('SELECT ID FROM ' . VERSIONS_TABLE . ' WHERE modifiedFields IS NOT NULL AND FIND_IN_SET("' . $text . '",modifications)', true);
		return $ids ? 'WETABLE.ID IN (' . implode(',', $ids) . ') ' : ' 0 ';
	}

	protected function searchContent(string $keyword, string $table): string{
		$db = DB_WE::$inst;
		switch($table){
			case FILE_TABLE:
				return ' WETABLE.ID IN(
SELECT ff.ID FROM ' .
					FILE_TABLE . ' ff JOIN ' .
					CONTENT_TABLE . ' cc ON (cc.DocumentTable IN ("' . $db->escape(stripTblPrefix(FILE_TABLE)) . '","' . TEMPORARY_DOCTARGET . '") AND cc.DID=ff.ID)
WHERE
	cc.Dat LIKE "%' . $this->db->escape(trim($keyword)) . '%" AND
	((1 ' . we_users_util::makeOwnersSql(false, 'ff.') . ') ' . we_selector_file::getWsQuery(FILE_TABLE, false, 'ff.') . ')
GROUP BY ff.ID
)';
			case TEMPLATES_TABLE:
				$contents = 'SELECT cc.DID FROM ' . CONTENT_TABLE . ' cc WHERE cc.Dat LIKE "%' . $this->db->escape(trim($keyword)) . '%" AND cc.nHash=unhex(md5("data")) AND cc.DocumentTable="' . $db->escape(stripTblPrefix(TEMPLATES_TABLE)) . '"';

				return ($contents ? ' WETABLE.ID IN(' . $contents . ')' : '');
			case VERSIONS_TABLE:
				//skip searching versioned data!
				return '';
			case OBJECT_FILES_TABLE :
				$Ids = $regs = [];

				$classes = $db->getAllq('SELECT ID FROM ' . OBJECT_TABLE, true);

				//published objects
				foreach($classes as $i){
					$obj_table = OBJECT_X_TABLE . intval($i);
					$tableInfo = $db->metadata($obj_table);
					'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int}> $tableInfo';
					$fields = [];
					foreach($tableInfo as $t){
						if(preg_match('/(.+?)_(.*)/', $t['name'], $regs)){
							if($regs[1] != 'OF' && $regs[1] != 'variant'){
								$fields[] = ['name' => $t['name'],
									'type' => $regs[1],
									'length' => $t['len']
								];
							}
						}
					}
					if(!$fields){
						continue;
					}
					$where = [];
					foreach($fields as $v){
						$where[] = 'o.`' . $v['name'] . '` LIKE "%' . $db->escape(trim($keyword)) . '%"';
					}

					$Ids = array_merge($Ids, $db->getAllq('SELECT
	`off`.ID
FROM ' .
							$db->escape($obj_table) . ' o JOIN ' .
							OBJECT_FILES_TABLE . ' `off` ON (o.OF_ID=`off`.ID AND o.OF_DocumentTable IN ("tblObjectFiles","' . TEMPORARY_DOCTARGET . '") )
WHERE ' .
							implode(' OR ', $where) . ' AND
				((1 ' . we_users_util::makeOwnersSql(false, '`off`.') . ') ' . we_selector_file::getWsQuery(OBJECT_FILES_TABLE, false, '`off`.') . ') GROUP BY `off`.ID', true));
				}

				return ($Ids ? ' WETABLE.ID IN (' . implode(',', $Ids) . ')' : '');
		}

		return '';
	}

	function searchMediaLinks(int $useState = 0, bool $holdAllLinks = true, array $inIDs = [], bool $returnQuery = false): string{
		$db = DB_WE::$inst;
		$this->usedMediaLinks = self::usedMediaLinksDefault;
		$this->usedMedia = $tmpMediaLinks = $groups = [];
		$inIDs = array_filter($inIDs);
		$fields = $holdAllLinks ? 'fl.ID,fl.DocumentTable,fl.remObj,fl.isTemp,c.Name AS element' : 'DISTINCT fl.remObj';
		$query = 'SELECT ' . $fields . '
FROM ' .
			FILELINK_TABLE . ' fl ' . ($holdAllLinks ? ' LEFT JOIN ' .
			CONTENT_TABLE . ' c ON (c.nHash=fl.nHash AND c.DocumentTable=fl.DocumentTable AND c.DID=fl.remObj)' : '') . '
WHERE
	fl.type="media" AND
	fl.remTable="' . stripTblPrefix(FILE_TABLE) . '" ' . ($inIDs ? ' AND
	fl.remObj IN (' . implode(',', $inIDs) . ')' : '') . ' AND
	fl.position=0';

		if($holdAllLinks){
			$result = $db->getAllq($query);

			$types = [
				FILE_TABLE => g_l('global', '[documents]'),
				TEMPLATES_TABLE => g_l('global', '[templates]'),
				OBJECT_FILES_TABLE => g_l('global', '[objects]'),
				OBJECT_TABLE => g_l('searchtool', '[classes]'),
				VFILE_TABLE => g_l('global', '[vfile]'),
				CATEGORY_TABLE => g_l('global', '[categorys]'),
				NEWSLETTER_TABLE => g_l('menu_moduleInformation', '[newsletter][text]'),
				CUSTOMER_TABLE => g_l('menu_moduleInformation', '[customer][text]'),
				GLOSSARY_TABLE => g_l('menu_moduleInformation', '[glossary][text]'),
				NAVIGATION_TABLE => g_l('menu_moduleInformation', '[navigation][text]'),
			];

			foreach($result as $rec){
				$tmpMediaLinks[$rec['remObj']][] = [$rec['ID'], $rec['DocumentTable'], $rec['isTemp'], $rec['element']];
				$groups[$rec['DocumentTable']][] = $rec['ID'];
				$this->usedMedia[] = (int) $rec['remObj'];
			}

			// get some more information about referencing objects
			$accessible = $paths = $isModified = $isUnpublished = $ct = $onclick = $type = $mod = $isTmpPossible = []; // TODO: make these arrays elements of one array
			foreach($groups as $k => $v){// FIXME: ct is obslete?
				'@phan-var string $k';
				$realTable = addTblPrefix($k);
				switch($realTable){
					case FILE_TABLE:
					case OBJECT_FILES_TABLE :
						$all = $db->getAllq('SELECT ID,Path,(Mod_Date>Publish_Date) AS IsModified,ContentType,IsPublished FROM ' . $realTable . ' WHERE ID IN (' . implode(',', array_unique($v)) . ') AND ((1 ' . we_users_util::makeOwnersSql(false) . ') ' . we_selector_file::getWsQuery($realTable, false) . ')');
						foreach($all as $rec){
							$accessible[$k][$rec['ID']] = true;
							$paths[$k][$rec['ID']] = $rec['Path'];
							$isModified[$k][$rec['ID']] = $rec['IsPublished'] && $rec['IsModified'];
							$isUnpublished[$k][$rec['ID']] = (int) $rec['IsPublished'] === 0;
							$ct[$k][$rec['ID']] = $rec['ContentType'];
							$onclick[$k][$rec['ID']] = 'WE().layout.openToEdit(\'' . $realTable . '\',\'' . $rec['ID'] . '\',\'' . $rec['ContentType'] . '\')';
							$type[$k][$rec['ID']] = 'we_doc';
							$mod[$k][$rec['ID']] = '';
							$isTmpPossible[$k][$rec['ID']] = true;
						}
						break;
					case TEMPLATES_TABLE:
					case OBJECT_TABLE:
						$all = $db->getAllq('SELECT ID,Path,ContentType FROM ' . $realTable . ' WHERE ID IN (' . implode(',', array_unique($v)) . ') AND ((1 ' . we_users_util::makeOwnersSql(false) . ') ' . we_selector_file::getWsQuery($realTable, false) . ')');
						foreach($all as $rec){
							$accessible[$k][$rec['ID']] = true;
							$paths[$k][$rec['ID']] = $rec['Path'];
							$ct[$k][$rec['ID']] = $rec['ContentType'];
							$onclick[$k][$rec['ID']] = 'WE().layout.openToEdit(\'' . $realTable . '\',\'' . $rec['ID'] . '\',\'' . $rec['ContentType'] . '\')';
							$type[$k][$rec['ID']] = 'we_doc';
							$mod[$k][$rec['ID']] = '';
							$isTmpPossible[$k][$rec['ID']] = false;
						}
						break;
					case VFILE_TABLE:
						if(we_base_permission::hasPerm([we_base_perm::WORK_COLLECTIONS_CAN_SEE])){
							$all = $db->getAllFirstq('SELECT ID,Path FROM ' . $realTable . ' WHERE ID IN (' . implode(',', array_unique($v)) . ')', false); //no ws fo collections
							foreach($all as $id => $path){
								$accessible[$k][$id] = true;
								$paths[$k][$id] = $path;
								$ct[$k][$id] = we_base_ContentTypes::COLLECTION;
								$onclick[$k][$id] = 'WE().layout.openToEdit(\'' . $realTable . '\',\'' . $id . '\',\'' . we_base_ContentTypes::COLLECTION . '\')';
								$type[$k][$id] = 'we_doc';
								$mod[$k][$id] = '';
								$isTmpPossible[$k][$id] = false;
							}
						}
						break;
					case CUSTOMER_TABLE:
					case GLOSSARY_TABLE:
					case NAVIGATION_TABLE:
					case NEWSLETTER_TABLE:
						$paths[$k] = id2pathA($v, $realTable);
						$modules = [
							CUSTOMER_TABLE => 'customer',
							GLOSSARY_TABLE => 'glossary',
							NAVIGATION_TABLE => 'navigation',
							NEWSLETTER_TABLE => 'newsletter'
						];
						foreach(array_keys($paths[$k]) as $key){
							$accessible[$k][$key] = true;
							$onclick[$k][$key] = 'weSearch.openModule(\'' . $modules[$realTable] . '\',' . $key . ')';
							$type[$k][$key] = 'module';
							$mod[$k][$key] = $modules[$realTable];
							$isTmpPossible[$k][$key] = false;
						}
						break;
					case CATEGORY_TABLE:
						if(we_base_permission::hasPerm([we_base_perm::WORK_CATEGORY_EDIT])){
							$paths[$k] = id2pathA($v, $realTable);
							foreach(array_keys($paths[$k]) as $key){
								$accessible[$k][$key] = true;
								$onclick[$k][$key] = 'weSearch.openCategory(' . $key . ')';
								$type[$k][$key] = 'cat';
								$mod[$k][$key] = '';
								$isTmpPossible[$k][$key] = false;
							}
						}
				}
			}

			foreach($tmpMediaLinks as $m_id => $v){
				$this->usedMediaLinks['accessible']['mediaID_' . $m_id] = [];
				$this->usedMediaLinks['notaccessible']['mediaID_' . $m_id] = [];
				foreach($v as $val){// FIXME: table, ct are obsolete when onclick works
					if(!isset($this->usedMediaLinks['accessible']['mediaID_' . $m_id][$types[addTblPrefix($val[1])]][$val[0] . $val[3]])){
						if(isset($accessible[$val[1]][$val[0]])){
							$this->usedMediaLinks['accessible']['mediaID_' . $m_id][$types[addTblPrefix($val[1])]][$val[0] . $val[3]] = [
								'exists' => isset($accessible[$val[1]][$val[0]]),
								'referencedIn' => intval($val[2]) === 0 ? 'main' : 'temp',
								'isTempPossible' => $isTmpPossible[$val[1]][$val[0]],
								'id' => $val[0],
								'element' => $val[3],
								'type' => $type[$val[1]][$val[0]] ?? '',
								'table' => addTblPrefix($val[1]),
								'ct' => $ct[$val[1]][$val[0]] ?? '',
								'mod' => $mod[$val[1]][$val[0]],
								'onclick' => $onclick[$val[1]][$val[0]] ?? 'WE().util.showMessage(\'not implemented yet: ' . $val[1] . '\', WE().consts.message.WE_MESSAGE_ERROR, window)',
								'path' => $paths[$val[1]][$val[0]] ?? '',
								'isModified' => $isModified[$val[1]][$val[0]] ?? false,
								'isUnpublished' => $isUnpublished[$val[1]][$val[0]] ?? false
							];
						} else {
							$this->usedMediaLinks['notaccessible']['mediaID_' . $m_id][$types[addTblPrefix($val[1])]][$val[0] . $val[3]] = true;
						}
					} else {
						$this->usedMediaLinks['accessible']['mediaID_' . $m_id][$types[addTblPrefix($val[1])]][$val[0] . $val[3]]['referencedIn'] = 'both';
					}
				}

				foreach($types as $type){
					if(!empty($this->usedMediaLinks['accessible']['mediaID_' . $m_id][$type]) || !empty($this->usedMediaLinks['notaccessible']['mediaID_' . $m_id][$type])){
						$this->usedMediaLinks['groups']['mediaID_' . $m_id][] = $type;
					}
				}
			}
		} else {
			$this->usedMedia = array_map('intval', $db->getAllq($query, true));
		}

		if(!$useState || !$returnQuery){
			return '0';
		}

		return $this->usedMedia ? ('WETABLE.ID ' . ($useState === 2 ? 'NOT ' : ' ') . 'IN(' . implode(',', $this->usedMedia) . ')') : ($useState === 2 ? '' : ' 0 ');
	}

	function searchHasReferenceToId(int $id, string $table): string{
		if(!$id || !$table){
			return '0';
		}

		$db = DB_WE::$inst;

		return ($ids = $db->getAllq('SELECT DISTINCT ID FROM ' . FILELINK_TABLE . ' WHERE type="media" AND DocumentTable="' . $db->escape(stripTblPrefix($table)) . '" AND remObj=' . $id, true)) ? ' ID IN (' . implode(',', array_unique($ids)) . ') ' : '0';
	}

	/**
	 * @param int $searchstart
	 * @param int $anzahl
	 * @param string $order
	 * @return array{Nr:int,isNr:int,UID:int,docID:int,docTable:string,Text:string,Path:string,ParentID:int,IsFolder:int,IsProtected:int,temp_template_id:int,TemplateID:int,ContentType:string,SiteTitle:string,CreatorID:int,Creation_Date:string,Mod_Date:string,Publish_Date:?string,Extension:?string,TableID:int,VersionID:int,media_alt:?string,media_title:?string,media_filesize:int,IsUsed:int,remTable:?string,remCT:?string,remClass:int,IsPublished:int,IsModified:int}[]
	 */
	function selectFromTempTable(int $searchstart, int $anzahl, string $order): array{
		$sortierung = explode(' ', trim($order) ?: 'Text');
		[$sortIsNr, $sortNr] = (isset($sortierung[1]) ? ['', 'DESC'] : ['DESC', '']);
		if(!$sortierung[0]){
			throw new we_exception_database('Sort invalid');
		}

		// FIXME: isNr and Nr seem to be unused
		//@phan-suppress-next-line PhanPartialTypeMismatchReturn
		return $this->db->getAllq('SELECT *,
	ABS(' . $sortierung[0] . ') AS Nr,
	(' . $sortierung[0] . " REGEXP '^[0-9]') AS isNr
FROM " .
				SEARCHRESULT_TABLE . '
WHERE
	UID=' . we_users_user::getID() . '
ORDER BY
	IsFolder DESC,
	isNr ' . $sortIsNr . ',
	Nr ' . $sortNr . ',
	' . $sortierung[0] . ' ' . $sortNr . '
LIMIT ' . ($searchstart > 0 ? $searchstart : 0) . ',' . $anzahl);
	}

//FIXME path is only implemented for filetable
	protected function insertInTempTable(string $where = '', string $table = '', string $path = ''): void{
		$this->table = ($table ?: ($this->table ?: ''));
		if(!$this->table){
			return;
		}

		$this->where = '1 ' . ($where ? (((substr(trim($where), 0, 4) !== 'AND ') ? 'AND ' : ' ') . trim($where)) : ($this->where ? 'AND ' . $this->where : ''));

		switch($this->table){
			case FILE_TABLE:
				if($path){
					$this->where .= ' AND WETABLE.Path LIKE "' . $this->db->escape($path) . '%" ';
				}
				$where = str_replace('WETABLE.', 'f.', $this->where);
				$this->db->query('REPLACE INTO ' . SEARCHRESULT_TABLE .
					' (UID,docID,docTable,Text,Path,ParentID,IsFolder,IsProtected,temp_template_id,TemplateID,ContentType,Creation_Date,CreatorID,Mod_Date,Publish_Date,Extension,SiteTitle)
SELECT
					' . we_users_user::getID() . ',f.ID,"' . stripTblPrefix(FILE_TABLE) . '",f.Text,f.Path,f.ParentID,f.IsFolder,f.IsProtected,f.temp_template_id,f.TemplateID,f.ContentType,f.Creation_Date,f.CreatorID,f.Mod_Date,f.Publish_Date,f.Extension,c.Dat
FROM
	`' . FILE_TABLE . '` f LEFT JOIN
	`' . CONTENT_TABLE . '` c ON (f.ID=c.DID AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND c.nHash=unhex(md5("Title")) )
WHERE ' . $where . ' GROUP BY ID');

				$this->db->query('UPDATE ' .
					SEARCHRESULT_TABLE . ' st JOIN ' .
					CONTENT_TABLE . ' c ON (c.DID=st.docID AND c.DocumentTable="' . TEMPORARY_DOCTARGET . '")
SET
	st.SiteTitle=c.Dat
WHERE
	c.nHash=unhex(md5("Title")) AND
	st.UID=' . we_users_user::getID());
				break;
			case VERSIONS_TABLE:
				if($_SESSION['weS']['search']['onlyDocs'] || $_SESSION['weS']['search']['ObjectsAndDocs']){
					$this->db->query('REPLACE INTO ' . SEARCHRESULT_TABLE . ' (UID,docID,docTable,Text,Path,ParentID,TemplateID,ContentType,Creation_Date,CreatorID,Extension,TableID,VersionID) SELECT ' . we_users_user::getID() . ',v.documentID,v.documentTable,CONCAT_WS("",v.Filename,v.Extension),v.Path,v.ParentID,v.TemplateID,v.ContentType,v.Creation_Date,v.modifierID,v.Extension,v.TableID,v.ID FROM ' . VERSIONS_TABLE . ' v LEFT JOIN ' . FILE_TABLE . ' f ON v.documentID=f.ID WHERE v.documentTable="' . stripTblPrefix(FILE_TABLE) . '" AND ' . str_replace('WETABLE.', (stristr($this->where, "status='deleted'") ? 'f.' : 'v.'), $this->where . ' ' . $_SESSION['weS']['search']['onlyDocsRestrUsersWhere']) . ' GROUP BY ID');
				}
				if(we_modules_info::isActive(we_modules_info::OBJECT) && ($_SESSION['weS']['search']['onlyObjects'] || $_SESSION['weS']['search']['ObjectsAndDocs'])){
					$this->db->query('REPLACE INTO ' . SEARCHRESULT_TABLE . ' (UID,docID,docTable,Text,Path,ParentID,TemplateID,ContentType,Creation_Date,CreatorID,Extension,TableID,VersionID)
SELECT ' . we_users_user::getID() . ',v.documentID,v.documentTable,CONCAT_WS("",v.Filename,v.Extension),v.Path,v.ParentID,v.TemplateID,v.ContentType,v.Creation_Date,v.modifierID,v.Extension,v.TableID,v.ID FROM ' . VERSIONS_TABLE . ' v LEFT JOIN ' . OBJECT_FILES_TABLE . ' `of` ON v.documentID=`of`.ID WHERE v.documentTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" AND ' . str_replace('WETABLE.', (stristr($this->where, "status='deleted'") ? '`of`.' : 'v.'), $this->where . ' ' . $_SESSION['weS']['search']['onlyObjectsRestrUsersWhere'] . ' GROUP BY ID'));
				}
				unset($_SESSION['weS']['search']['onlyObjects'], $_SESSION['weS']['search']['onlyDocs'], $_SESSION['weS']['search']['ObjectsAndDocs'], $_SESSION['weS']['search']['onlyObjectsRestrUsersWhere'], $_SESSION['weS']['search']['onlyDocsRestrUsersWhere']);
				break;

			case TEMPLATES_TABLE:
				$this->db->query('REPLACE INTO ' . SEARCHRESULT_TABLE . ' (UID,docID,docTable,Text,Path,ParentID,IsFolder,ContentType,SiteTitle,Creation_Date,CreatorID,Mod_Date,Extension) SELECT ' . we_users_user::getID() . ',ID,"' . stripTblPrefix(TEMPLATES_TABLE) . '",Text,Path,ParentID,IsFolder,ContentType,Path,Creation_Date,CreatorID,Mod_Date,Extension FROM `' . TEMPLATES_TABLE . '` t WHERE ' . str_replace('WETABLE.', 't.', $this->where) . ' GROUP BY ID');
				break;

			case VFILE_TABLE:
				$this->db->query('REPLACE INTO ' . SEARCHRESULT_TABLE . ' (UID,docID,docTable,Text,Path,ParentID,IsFolder,ContentType,Creation_Date,CreatorID,Mod_Date,remTable,remCT,remClass) SELECT ' . we_users_user::getID() . ',ID,"' . stripTblPrefix(VFILE_TABLE) . '",Filename,Path,ParentID,IsFolder,ContentType,Creation_Date,CreatorID,Mod_Date,remTable,remCT,remClass FROM `' . VFILE_TABLE . '` v WHERE ' . str_replace('WETABLE.', 'v.', $this->where) . ' GROUP BY ID');
				break;

			case OBJECT_FILES_TABLE :
				$this->db->query('REPLACE INTO ' . SEARCHRESULT_TABLE . ' (UID,docID,docTable,Text,Path,ParentID,IsFolder,ContentType,Creation_Date,CreatorID,Mod_Date,Publish_Date,TableID) SELECT ' . we_users_user::getID() . ',ID,"' . stripTblPrefix(OBJECT_FILES_TABLE) . '",Filename,Path,ParentID,IsFolder,ContentType,Creation_Date,CreatorID,Mod_Date,Publish_Date,TableID FROM ' . OBJECT_FILES_TABLE . ' `of` WHERE ' . str_replace('WETABLE.', '`of`.', $this->where) . ' GROUP BY ID');
				break;

			case OBJECT_TABLE :
				$this->db->query('REPLACE INTO ' . SEARCHRESULT_TABLE . ' (UID,docID,docTable,Text,Path,ParentID,IsFolder,ContentType,Creation_Date,CreatorID,Mod_Date) SELECT ' . we_users_user::getID() . ',ID,"' . stripTblPrefix(OBJECT_TABLE) . '",Filename,Path,ParentID,IsFolder,ContentType,Creation_Date,CreatorID,Mod_Date FROM `' . OBJECT_TABLE . '` WHERE ' . str_replace('WETABLE.', 'o.', $this->where) . ' GROUP BY ID');
				break;
		}
	}

	private function insertMediaAttribsToTempTable(): void{
		if(!fB('SELECT 1 FROM ' . SEARCHRESULT_TABLE . ' WHERE UID=' . we_users_user::getID() . ' AND docTable="' . stripTblPrefix(FILE_TABLE) . '"')){
			return;
		}
//it is easier to have 3 queries than one with left joins
		foreach(['title' => 'c.Dat', 'alt' => 'c.Dat', 'filesize' => 'c.BDID'] as $cur => $field){
			$this->db->query('UPDATE ' .
				SEARCHRESULT_TABLE . ' t JOIN ' .
				CONTENT_TABLE . ' c  ON t.docID=c.DID
SET
	t.media_' . $cur . '=' . $field . '
WHERE
	t.UID=' . we_users_user::getID() . ' AND
	t.docTable="' . stripTblPrefix(FILE_TABLE) . '" AND
	c.nHash=x\'' . md5($cur) . '\' AND
	c.Type="attrib" AND
	c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '"'
			);
		}

		//FIXME: remove this query - only temporary, searchMediaLinks should use the join as well
		$IDs = $this->db->getAllq('SELECT docID FROM ' . SEARCHRESULT_TABLE . ' WHERE UID=' . we_users_user::getID() . ' AND docTable="' . stripTblPrefix(FILE_TABLE) . '"', true);
		$this->searchMediaLinks(0, true, $IDs); // we write $this->usedMediaLinks here, where we allready have final list of found media
		if($this->usedMedia){
			$this->db->query('UPDATE ' . SEARCHRESULT_TABLE . ' SET IsUsed=1 WHERE UID=' . we_users_user::getID() . ' AND docID IN(' . implode(',', $this->usedMedia) . ') AND docTable="' . stripTblPrefix(FILE_TABLE) . '" LIMIT 1');
		}
	}

	function createTempTable(): void{
		$this->db->query('DELETE FROM ' . SEARCHRESULT_TABLE . ' WHERE UID=' . we_users_user::getID());
	}

	private function searchString(string $searchname, string $searchfield, string $searchlocation, string $tablename): string{
		$this->table = $tablename;
		$sql = [];

		$whatParentID = '';
		switch($searchfield){
			case 'ParentIDDoc':
			case 'ParentIDObj':
			case 'ParentIDTmpl':
				$whatParentID = $searchfield;
				$searchfield = 'ParentID';
				break;
			case 'ID':
				if($tablename === VERSIONS_TABLE){
					$searchfield = 'documentID';
					break;
				}
			case 'CreatorID':
			case 'WebUserID':
				if(!is_numeric($searchname)){
					$arr = array_filter(explode(',', $searchname), function ($var){
						return is_numeric($var);
					});
					if(empty($arr)){
						return '0';
					}
					$searchname = implode(',', $arr);
				}
				break;
			case 'temp_template_id' :
				if($tablename === VERSIONS_TABLE){
					$searchfield = 'TemplateID';
				}
				break;
			case 'Mod_Date' :
				if($tablename === VERSIONS_TABLE){
					$searchfield = 'logDate';
				}
				break;
			case 'Text':
				if($tablename === VERSIONS_TABLE){
					$searchfield = 'Filename';
				}
				break;
		}
		if(empty($searchname)){
			return '';
		}

		$tableInfo = DB_WE::$inst->metaFieldNames($this->table);
		//filter fields for each table
		foreach($tableInfo as $cur){
			if($searchfield != $cur){
				continue;
			}
			$searchfield = 'WETABLE.' . $cur;

			if(
				($whatParentID === 'ParentIDDoc' && in_array($this->table, [FILE_TABLE, VERSIONS_TABLE])) ||
				($whatParentID === 'ParentIDObj' && in_array($this->table, [OBJECT_FILES_TABLE, VERSIONS_TABLE])) ||
				($whatParentID === 'ParentIDTmpl' && $this->table === TEMPLATES_TABLE)){
				if($this->table === VERSIONS_TABLE){
					switch($whatParentID){
						case 'ParentIDDoc':
							$this->table = FILE_TABLE;
							break;
						case 'ParentIDObj':
							$this->table = OBJECT_FILES_TABLE;
							break;
					}
				}
				$searchname = path_to_id($searchname, $this->table);
				$searching = "='" . $searchname . "' ";
				$sql[] = $this->sqlwhere($searchfield, $searching);
			} elseif(
				($cur === 'MasterTemplateID' && $this->table === TEMPLATES_TABLE) ||
				($cur === 'temp_template_id' && $this->table === FILE_TABLE) ||
				($cur === 'TemplateID' && $this->table === VERSIONS_TABLE)){
				$searchname = path_to_id($searchname, TEMPLATES_TABLE);
				$searching = "='" . $searchname . "' ";

				if(($cur === 'temp_template_id' && $this->table === FILE_TABLE) || ($cur === 'TemplateID' && $this->table === VERSIONS_TABLE)){
					$sql[] = $this->sqlwhere('WETABLE.TemplateID', $searching);
				} else {
					$sql[] = $this->sqlwhere($searchfield, $searching);
				}
			} elseif(in_array($cur, ['Publish_Date', 'Creation_Date', 'Mod_Date'], true)){
				if($cur === 'Publish_Date' && $this->table === FILE_TABLE || $this->table === OBJECT_FILES_TABLE || $cur !== 'Publish_Date'){
					if($this->table === VERSIONS_TABLE && ($cur === 'Creation_Date' || $cur === 'Mod_Date')){
						$searchfield = $this->table . '.logDate';
					}
					$timestampStart = $searchname . ' 00:00:00';
					$timestampEnd = $searchname . ' 23:59:59';

					if(isset($searchlocation)){
						switch($searchlocation){
							case self::OP_IS:
								$sql[] = $this->sqlwhere($searchfield, ' BETWEEN "' . $timestampStart . '" AND "' . $timestampEnd . '" ');
								break;
							case self::OP_LO:
								$sql[] = $this->sqlwhere($searchfield, ' < "' . $timestampStart . '" ');
								break;
							case self::OP_LEQ:
								$sql[] = $this->sqlwhere($searchfield, ' <= "' . $timestampEnd . '" ');
								break;
							case self::OP_HI:
								$sql[] = $this->sqlwhere($searchfield, ' > "' . $timestampEnd . '" ');
								break;
							case self::OP_HEQ:
								$sql[] = $this->sqlwhere($searchfield, ' >= "' . $timestampStart . '" ');
								break;
						}
					}
				}
			} elseif(isset($searchlocation)){
				switch($searchlocation){
					case self::OP_END:
						$sql[] = $this->sqlwhere($searchfield, ' LIKE "%' . $this->db->escape($searchname) . '" ');
						break;
					case self::OP_START:
						$sql[] = $this->sqlwhere($searchfield, ' LIKE "' . $this->db->escape($searchname) . '%" ');
						break;
					case self::OP_IN:
						$searchname = strtr($searchname, ['\_' => '_', '\%' => '%']);
						$sql[] = $this->sqlwhere($searchfield, ' IN ("' . implode('","', array_map('trim', explode(',', $searchname))) . '") ');
						break;
					case self::OP_IS:
						$searchname = strtr($searchname, ['\_' => '_', '\%' => '%']);
						$sql[] = $this->sqlwhere($searchfield, '="' . $this->db->escape($searchname) . '" ');
						break;
					case self::OP_LO:
						$sql[] = $this->sqlwhere($searchfield, ' < "' . $this->db->escape($searchname) . '" ');
						break;
					case self::OP_LEQ:
						$sql[] = $this->sqlwhere($searchfield, ' <= "' . $this->db->escape($searchname) . '" ');
						break;
					case self::OP_HI:
						$sql[] = $this->sqlwhere($searchfield, ' > "' . $this->db->escape($searchname) . '" ');
						break;
					case self::OP_HEQ:
						$sql[] = $this->sqlwhere($searchfield, ' >= "' . $this->db->escape($searchname) . '" ');
						break;
					default:
						$sql[] = $this->sqlwhere($searchfield, ' LIKE "%' . $this->db->escape($searchname) . '%" ');
						break;
				}
			}
			//found col, return
			return implode(' AND ', $sql);
		}

		return '';
	}

	/**
	 * @param int[] $folderID
	 * @param string $table
	 * @return string
	 */
	private static function ofFolderAndChildsOnly(array $folderID, string $table): string{//move this to view class; or verse visa
		$_SESSION['weS']['search']['countChilds'] = [];
		$ids = [];
		self::getChildsOfParentId($folderID, $ids, $table);

		return $ids ? 'WETABLE.ParentID IN (' . implode(',', array_unique($ids)) . ')' : '';
	}

	private static function getChildsOfParentId(array $folders, array &$ret, string $table): void{
		if($table === VERSIONS_TABLE){ //we don't have parents & folders
			return;
		}

		$ret = array_merge($ret, $folders);
		$ids = DB_WE::$inst->getAllq('SELECT ID FROM `' . DB_WE::$inst->escape($table) . '` WHERE ParentID IN(' . implode(',', $folders) . ') AND IsFolder=1', true);

		if($ids){
			self::getChildsOfParentId($ids, $ret, $table);
		}
	}

	function getResultCount(): int{
		return fI('SELECT COUNT(1) FROM ' . SEARCHRESULT_TABLE . ' WHERE UID=' . we_users_user::getID());
	}

	static function cleanOldEntries(): void{
		DB_WE::$inst->query('DELETE FROM ' . SEARCHRESULT_TABLE . ' WHERE UID IN (SELECT ID FROM ' . USER_TABLE . ' WHERE Ping IS NULL OR (Ping+INTERVAL ' . we_base_constants::PING_TOLERANZ . ' second)<NOW() )');
	}

	private function getSearchString(string $table, array $tables, bool $opAND, array $searchFields, string $whichSearch, array $searchForField, array $searchText, array $location, array $searchForContentType, string $view): array{
		$where = $where_OR = [];

		foreach($searchFields as $i => $curField){
			'@phan-var string $curField';
			$done = false;

			// process empty searchTexts: decide wether to ignore empty serachTexts (continue) or to search for 0, '' respectively
			$searchString = (string) $searchText[$i];
			if(!$searchString){
				switch($whichSearch){
					case we_search_view::SEARCH_MEDIA:
						switch($curField){
							case 'keyword':
								$where[] = 1;
								continue 3;
							default:
								if($location[$i] === self::OP_IS && preg_match('|^meta__|', $curField)){
									$searchString = '##EMPTY##'; // FIXME: old stuff..., find better solution
									break;
								}
								continue 3;
						}
						break;
					default:
						switch($curField){
							case 'temp_category':
								break;
							default:
								// ignore condition when searchTexts is empty
								continue 3;
						}
						break;
				}
			}

			/* if(!empty($searchString) ) */{
				if(!in_array($curField, ['Status', 'Speicherart', 'HasReferenceToID'], true)){
					$searchString = strtr($searchString, ['\\' => '\\\\', '_' => '\_', '%' => '\%']);
				}

				if($table === FILE_TABLE && $whichSearch === we_search_view::SEARCH_MEDIA){
					$done = true;
					switch($curField){
						case 'keyword':
							foreach((array_filter($searchForField, function ($var){
								return ($var == 1);
							})) as $field => $v){
								switch($field){
									case 'title': // IMPORTANT: in media search options are generally AND-linked, but not "search in Title, Text, Meta!
										$where_OR[] = ($this->searchInTitle($searchString, $table) ?: '');
										break;
									case 'text':
										$where_OR[] = 'WETABLE.Filename LIKE "%' . DB_WE::$inst->escape(trim($searchString)) . '%" ';
										break;
									case 'meta':
										$where_OR[] = ($this->searchInAllMetas($searchString, $table) ?: '');
										break;
								}
							}
							break;
						case 'ContentType':
							$contentTypes = [];
							foreach($searchForContentType as $type => $v){
								if($v){
									switch($type){
										case 'image':
											$contentTypes[] = we_base_ContentTypes::IMAGE;
											break;
										case 'video':
											$contentTypes[] = we_base_ContentTypes::VIDEO;
											break;
										case 'audio':
											$contentTypes[] = we_base_ContentTypes::AUDIO;
											break;
										case 'other':
											$contentTypes[] = we_base_ContentTypes::APPLICATION;
											$contentTypes[] = we_base_ContentTypes::PDF;
											break;
									}
								}
							}
							$where[] = 'WETABLE.ContentType IN ("' . implode('","', $contentTypes ?: [
									we_base_ContentTypes::IMAGE,
									we_base_ContentTypes::VIDEO,
									we_base_ContentTypes::AUDIO,
									we_base_ContentTypes::APPLICATION,
									we_base_ContentTypes::PDF
								]) . '")';
							break;
						case 'IsUsed':
							switch($searchString){
								case '-1':
									$where[] = 1;
									break;
								default:
									$where[] = $this->searchMediaLinks((int) $searchString, $view !== we_search_view::VIEW_ICONS, [], true);
							}
							break;
						case 'IsProtected':
							switch($searchString){
								case '1':
									$where[] = 'WETABLE.IsProtected=1';
									break;
								case '2':
									$where[] = 'WETABLE.IsProtected=0';
									break;
								case '-1':
									$where[] = 'WETABLE.IsProtected IN (0,1)';
									break;
							}
							break;
						default:
							$done = false;
							if(str_starts_with($curField, 'meta__')){
								$where[] = $this->searchInMeta($searchString, substr($curField, 6) ?: '', $location[$i], $table);
								$done = true;
							}
					}
				}

				if(!$done){
					switch($curField){
						case 'Content':
							if($table !== OBJECT_TABLE){
								$w = $this->searchContent($searchString, $table);
								if($w === ''){
									//not found in content
								} elseif($opAND){
									$where[] = $w;
								} else {
									$where_OR[] = $w;
								}
							}
							break;
						case 'modifierID':
							$w = $this->searchModifier((int) $searchString);
							if($w === ''){
								$where[] = ' 0 ';
							} elseif($opAND){
								$where[] = $w;
							} else {
								$where_OR[] = $w;
							}

							break;
						case 'allModsIn':
							if($table === VERSIONS_TABLE){
								$where[] = $this->searchModFields($searchString);
							}
							break;
						case 'Title':
							$w = $this->searchInTitle($searchString, $table);
							if($w === ''){
								$where[] = ' 0 ';
							} elseif($opAND){
								$where[] = $w;
							} else {
								$where_OR[] = $w;
							}
							break;
						case 'Status':
						case 'Speicherart':
							switch($table){
								case VERSIONS_TABLE:
									$where[] = $this->getStatusFiles($searchString, $table);

									$docTableChecked = (in_array(FILE_TABLE, $tables, true));
									$objTableChecked = (in_array(OBJECT_FILES_TABLE, $tables, true));
									if($objTableChecked && $docTableChecked){
										$where[] = '(v.documentTable IN ("' . stripTblPrefix(FILE_TABLE) . '","' . stripTblPrefix(OBJECT_FILES_TABLE) . '")) ';
									} elseif($docTableChecked){
										$where[] = 'v.documentTable="' . stripTblPrefix(FILE_TABLE) . '" ';
									} elseif($objTableChecked){
										$where[] = 'v.documentTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" ';
									}
									break;
								case FILE_TABLE:
								case OBJECT_FILES_TABLE :
									$w = $this->getStatusFiles($searchString, $table);
									if($w === ''){
										$where[] = ' 0 ';
									} elseif($opAND){
										$where[] = $w;
									} else {
										$where_OR[] = $w;
									}

									break 2;
								default:
									break 2;
							}

							break;
						case 'CreatorName':
						case 'WebUserName':
							if(isset($curField) && isset($location[$i])){
								$w = $this->searchSpecial($searchString, $curField, $location[$i]);
								if($w === ''){
									$where[] = ' 0 ';
								} elseif($opAND){
									$where[] = $w;
								} else {
									$where_OR[] = $w;
								}
							}
							break;
						case 'temp_category':
							$w = $this->searchCategory($searchString, $table);
							if($w === ''){
								$where[] = ' 0 ';
							} elseif($opAND){
								$where[] = $w;
							} else {
								$where_OR[] = $w;
							}

							break;
						case 'HasReferenceToID':
							$where[] = $searchString ? (($searchId = path_to_id($searchString)) ? $this->searchHasReferenceToId($searchId, $table) : 0) : '';
							break;
						default:
							//if($whichSearch != "AdvSearch"){
							$w = $this->searchString($searchString, $curField, $location[$i], $table);
							if($w === ''){
								$where[] = ' 0 ';
							} elseif($opAND){
								$where[] = $w;
							} else {
								$where_OR[] = $w;
							}
						//}
					}
				}
			}
		}
		$tmp = implode(' OR ', array_filter($where_OR));
		if($tmp){
			$where[] = '(' . $tmp . ')';
		}

		return array_filter($where);
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'buttonSelectValue' => g_l('button', '[select][value]'),
				'confirmDel' => g_l('searchtool', '[confirmDel]'),
				'nameForSearch' => g_l('searchtool', '[nameForSearch]'),
				'no_perms' => (g_l('tools', '[no_perms]')),
				'nothingCheckedAdv' => g_l('searchtool', '[nothingCheckedAdv]'),
				'nothingCheckedTmplDoc' => g_l('searchtool', '[nothingCheckedTmplDoc]'),
				'nothing_to_delete' => (g_l('tools', '[nothing_to_delete]')),
				'confirmDeleteFiles' => g_l('searchtool', '[confirm_delete_files]'),
				'nothing_to_save' => (g_l('tools', '[nothing_to_save]')),
				'predefinedSearchdelete' => (g_l('searchtool', '[predefinedSearchdelete]')),
				'predefinedSearchmodify' => (g_l('searchtool', '[predefinedSearchmodify]')),
				'publish_docs' => g_l('searchtool', '[publish_docs]'),
				'resetVersionsSearchtool' => g_l('versions', '[resetVersionsSearchtool]'),
				'searchtool__notChecked' => g_l('searchtool', '[notChecked]'),
				'searchtool__publishOK' => g_l('searchtool', '[publishOK]'),
				'versionsNotChecked' => g_l('versions', '[notChecked]'),
				'versionsResetAllVersionsOK' => g_l('versions', '[resetAllVersionsOK]'),
		]);
	}

}
