<?php
declare(strict_types=1);

/** Funktion zum Blaettern
 * 	Funktion zum erstellen einer Linkliste, die das Blaettern durch eine bestimmte Zahl von Ergebnissen erlaubt
 *
 * 	@author Jan Gorba <jan.gorba@webedition.de>
 * 	@link http://www.webedition.de/
 * 	@version 1.0_php4 (de)
 *
 *
 * 	@advice Tabulatorweite ist 4
 */
/*
  Alle moeglichen Varianten
  Anfang -- Zurueck - 1, 2, 3 ... 6, 7, 8 ... 11, 12, 13 - Weiter -- Ende
  Zurueck - 1, 2, 3 ... 6, 7, 8 ... 11, 12, 13 - Weiter
  Anfang - 1, 2, 3 ... 6, 7, 8 ... 11, 12, 13 - Ende
  1, 2, 3 ... 6, 7, 8 ... 11, 12, 13

  Anfang -- Zurueck - ... 6, 7, 8 ... - Weiter -- Ende
  Zurueck ... 6, 7, 8 ... Weiter
  Anfang ... 6, 7, 8 ... Ende
  ... 6, 7, 8 ...

  Anfang -- Zurueck - 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13 - Weiter -- Ende
  Zurueck - 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13 - Weiter
  Anfang - 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13 - Ende
  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13

  Variablen fuer Links
  Variable						Beispiel						Ausgabebeispiel
  $delimiter1						--								Anfang -- Zurueck
  $delimiter2						-								Zurueck - 1
  $delimiter3						,								1, 2, 3
  $delimiter4						...								1, 2, 3 ... 6, 7, 8

  $link_active_text				Seite							Seite 1, Seite 2, Seite 3
  $link_active_bracket_left		[								6, [7], 8
  $link_active_bracket_right		]								6, [7], 8
  $link_active_more				style='font-size:1.2em;'		<a href='...' style='font-size:1.2em;'>

  $link_text						Seite							Seite 1, Seite 2, Seite 3
  $link_bracket_left				>								>1<, >2<, >3<
  $link_bracket_right				<								>1<, >2<, >3<
  $link_more						target='_blank'					<a href='...' target='_blank'>

  $link_first_text				Anfang							Anfang -- Zurueck
  $link_last_text					Ende							Weiter -- Ende
  $link_first_last_more			style='font-family:arial;'		<a href='...' style='font-family:arial;'>

  $link_next_text					Weiter							Weiter -- Ende
  $link_prev_text					Zurueck							Anfang -- Zurueck
  $link_next_prev_more			class='navigation'				<a href='...' class='navigation'>

  $show_count						3								6, 7, 8
  $show_continuous				true/false						true: 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13
  false: abhaengig von $show_outer
  $show_outer						true/false						true: 1, 2, 3 ... 6, 7, 8 ... 11, 12, 13
  false: ... 6, 7, 8 ...
  $show_next_prev					true/false						true: Weiter/Zurueck anzeigen, false: nicht anzeigen
  $show_first_last				true/false						true: Anfang/Ende anzeigen, false: nicht anzeigen
  $show_active_link				true/false						true: 6, [7], 8, false: 6, 8
  $show_single_link				true/false						true: [1], false: (nichts)

  $link_href						bilder.php?id=$id&page=			<a href='bilder.php?id=BMW&page=2'>
 */

class we_shop_pager{
	// Wer hier Werte setzt, der sollte sicher sein, dass sie richtig sind, geprueft werden sie nicht!
	// Also lieber die dafuer vorgesehenen Methoden nutzen.
	//////////////////////////////////////////////////
	// Erforderliche Variablen
	//////////////////////////////////////////////////
	/** @var int Zahl der aktuellen Seite (Seite 1 hat Wert 0)	 */
	private $active_page = 0;

	/** @var int Anzahl aller Eintraege	 */
	private $entries = 0;

	/** Eintraege, die pro Seite gezeigt werden sollen
	 *
	 * 	@var int
	 */
	private $epp = 12;

	/** URI zur Datei, plus Variablen, fuer Links
	 *
	 * 	@var string
	 */
	private $link_href = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=shop&pnt=edit_shop_revenueTop&page=';
	//////////////////////////////////////////////////
	// Optionale Variablen
	//////////////////////////////////////////////////

	/** @var string Trennzeichen, siehe 'Variablen fuer Links'	 */
	private $delimiter1 = ' - ';

	/** @var string	 */
	private $delimiter2 = ' - ';

	/** @var string Trennzeichen, siehe 'Variablen fuer Links'	 */
	private $delimiter3 = '  ';

	/** @var string Trennzeichen, siehe 'Variablen fuer Links'	 */
	private $delimiter4 = ' ... ';

	/** @var string Text vor aktivem Link, siehe 'Variablen fuer Links'	 */
	private $link_active_text = '';

	/** Zeichen vor aktivem Link, siehe 'Variablen fuer Links'
	 *
	 * 	@var string
	 */
	private $link_active_bracket_left = '[';

	/** @var string Zeichen hinter aktivem Link, siehe 'Variablen fuer Links'	 */
	private $link_active_bracket_right = ']';

	/** @var string Zusaetzliche Angaben zum aktiven Link, siehe 'Variablen fuer Links'	 */
	private $link_active_more = '';

	/** @var string Zeichen vor nicht-aktiven Links, siehe 'Variablen fuer Links'	 */
	private $link_text = '';

	/** @var string Zeichen vor nicht-aktiven Links, siehe 'Variablen fuer Links'	 */
	private $link_bracket_left = '';

	/** @var string Zeichen hinter nicht-aktiven Links, siehe 'Variablen fuer Links'	 */
	private $link_bracket_right = '';

	/** @var string Zusaetzliche Angaben zu nicht-aktiven Links, siehe 'Variablen fuer Links'	 */
	private $link_more = '';

	/** @var string Text fuer 'Weiter' (naechste Seite), siehe 'Variablen fuer Links'	 */
	private $link_next_text = 'Vorwaerts';

	/**  @var string Text fuer 'Zurueck' (vorherige Seite), siehe 'Variablen fuer Links'	 */
	private $link_prev_text = 'Zurueck';

	/** @var string Zusaetzliche Angaben zu Links 'Weiter' und 'Zurueck', siehe 'Variablen fuer Links'	 */
	private $link_first_last_more = '';

	/** @var string Text fuer 'Anfang' (erste Seite), siehe 'Variablen fuer Links'	 */
	private $link_first_text = 'Start';

	/** @var string Text fuer 'Ende' (letzte Seite), siehe 'Variablen fuer Links'	 */
	private $link_last_text = 'Ende';

	/** @var string Zusaetzliche Angaben zu Links 'Anfang' und 'Ende', siehe 'Variablen fuer Links'	 */
	private $link_next_prev_more = '';

	/** @var int Anzahl Links, die nebeneinander gezeigt werden, siehe 'Variablen fuer Links'	 */
	private $show_count = 3;

	/** @var bool Alle Links zeigen ja/nein, siehe 'Variablen fuer Links'	 */
	private $show_continuous = false;

	/** @var bool Aeussere Links zeigen ja/nein, siehe 'Variablen fuer Links'	 */
	private $show_outer = true;

	/** 'Weiter' und 'Zurueck' zeigen ja/nein, siehe 'Variablen fuer Links'
	 *
	 * 	@var bool
	 */
	private $show_next_prev = true;

	/** @var bool 'Anfang' und 'Ende' zeigen ja/nein, siehe 'Variablen fuer Links'	 */
	private $show_first_last = true;

	/** @var bool Link der aktuellen Seite zeigen ja/nein, siehe 'Variablen fuer Links'	 */
	private $show_active_link = true;

	/** @var bool Wenn nur eine Seite, Link zeigen ja/nein, siehe 'Variablen fuer Links'	 */
	private $show_single_link = false;

	/** Um wie viele Seiten Weiter/Zurueck springen soll
	 *
	 * 	@var int
	 */
	private $steps_next_prev = 1;
	//////////////////////////////////////////////////
	// Voll und ganz interne Variablen
	//////////////////////////////////////////////////

	/** @var int Anzahl aller Links	 */
	private $link_count = 0;

	/** @var int Anzahl der Links, die den aktuellen umgeben  */
	private $link_count_surround = 0;

	/** @var int Seitenzahl fuer vorherige Seite */
	private $page_prev = 0;

	/** @var int Seitenzahl fuer naechste Seite	 */
	private $page_next = 0;

	/** @var int Seitenzahl fuer erste Seite */
	private $page_first = 0;

	/** @var int Seitenzahl fuer letzte Seite */
	private $page_last = 0;

	/** @var array Array aller Links, String mit Links bei Uebergabe	 */
	private $links = [];

	/** @var string Array aller Links, String mit Links bei Uebergabe	 */
	private $linksStr = '';

	/** Erstellt einen normalen webEdition pager f�r den Shop
	 *
	 * 	@access public
	 * 	@param string $url URL der Seite
	 * 	@param int $actPage Aktuelle Seitenzahl (0 = erste Seite)
	 * 	@param int $nrOfPage Anzahl Eintr�ge pro Seite
	 * 	@param int $anz Anzahl Eintr�ge gesamt
	 */
	public static function getStandardPagerHTML(string $url, int $actPage, int $nrOfPage, int $anz): string{
		$offset = $actPage * $nrOfPage;

		return '<table class="default"><tr><td>' .
			($actPage > 0 ?
			we_html_button::create_button(we_html_button::BACK, $url . '&actPage=' . ($actPage - 1)) :
			we_html_button::create_button(we_html_button::BACK, '', '', '', true)) .
			'</td><td style="padding:0xp 23px;"><b>' . (($anz) ? $offset + 1 : 0) . '-' .
			(($anz - $offset) < $nrOfPage ?
			$anz : $offset + $nrOfPage) .
			'&nbsp;&nbsp;' . g_l('global', '[from]') . '&nbsp;&nbsp;' . $anz . '</b></td><td>' .
			(($offset + $nrOfPage) < $anz ?
			we_html_button::create_button(we_html_button::NEXT, $url . '&actPage=' . ($actPage + 1)) :
			we_html_button::create_button(we_html_button::NEXT, '', '', '', true)) .
			'</td></tr></table>';
	}

	/** Erstellt ein Objekt fuer die Blaetterfunktion
	 *
	 * 	@access public
	 * 	@param (int) $active_page						Zahl der aktuellen Seite, erhaelt man ungefaehr so: "$active_page=isset($_GET['ap']) ? $_GET['ap'] : 0;"
	 * 	@param (int) $entries							Anzahl aller Ergebnisse; erhaelt man ungefaehr so: "$entries=num_rows($resource);"
	 */
	function __construct(int $active_page, int $entries){
		$this->set_Active_Page($active_page);
		$this->set_Entries($entries);
	}

	/** Erstellt die Liste mit Links
	 *
	 * 	@access public
	 * 	@return (mixed)									Gibt den erstellten String der Links oder false bei Fehlern zurueck
	 */
	function create(): array{
		// Array neu initialisieren
		$this->links = [];

		// Erst einmal testen, ob die Werte okay sind
		if($this->entries == 0){
			$this->links[0] = '';
			return $this->links;
		}

		// Anzahl aller moeglichen Links zu den Seiten ermitteln
		$this->link_count = $this->entries % $this->epp == 0 ? (int) $this->entries / $this->epp : (int) ceil($this->entries / $this->epp);

		// Ermitteln, wie viele Links den aktuellen umgeben werden
		$this->link_count_surround = (int) floor($this->show_count / 2);

		// Wenn sinnvoll ($show_outer===true, aber $this->link_count<=$show_count*3), dann ueberschreiben wir hier $show_continuous
		if($this->show_outer === true &&
			$this->link_count <= $this->show_count * 3){
			$this->show_continuous = true;
		}

		// Angabe der aktuelllen Seite pruefen, gegebenenfalls neu setzen
		if($this->link_count < $this->active_page){
			$this->active_page = $this->link_count - 1;
		} elseif(!is_numeric($this->active_page) ||
			$this->active_page < 0){
			$this->active_page = 0;
		}

		// Jetzt berechnen wir erst einmal die Seitenzahlen fuer die erste, letzte, naechste und vorherige Seite
		// Sollte eine Seite nicht existieren, dann ist der Wert null (nicht 0!)
		$this->page_next = $this->active_page + 1;
		if($this->page_next >= $this->link_count - 1 &&
			$this->active_page == $this->link_count - 1){
			$this->page_next = 0;
		}
		$this->page_prev = $this->active_page - 1;
		if($this->page_prev <= 0 &&
			$this->active_page == 0){
			$this->page_prev = 0;
		}
		$this->page_first = 0;
		if($this->active_page == 0){
			$this->page_first = 0;
		}
		$this->page_last = $this->link_count - 1;
		if($this->active_page == $this->link_count - 1){
			$this->page_last = 0;
		}


		// Wenn kein Blaettern zu Stande kommt, dann mal schauen, ob der Nutzer den einzelnen Link sehen will
		if(($this->show_single_link === false ||
			$this->show_active_link === false) &&
			$this->entries <= $this->show_count){
			$this->links = [];
			return $this->links;
		}

		// Alle Seitenzahlen durchgehen (egal, ob am Ende gezeigt oder nicht) und Seitenzahlen im Array speichern
		for($i = 0; $i < $this->link_count; $i++){
			$this->links[] = $i;
		}
		// Links aus Seitenzahlen erstellen
		$this->make_links();

		// Wenn alle Seiten angezeigt werden sollen...
		// ...oder Anzahl Links kleiner oder gleich der zu zeigenden ist, dann die Links speichern
		if($this->show_continuous ||
			$this->link_count <= $this->show_count){
			$this->implodeLinks_and_add_delimiter3();
		} else {
			// Oder wenn nicht alle Links angezeigt werden sollen...
			// ...und nicht die aeusseren, dann Links speichern
			if($this->show_outer === false){
				// Wenn aktiver Link einer der aeusseren ist, dann dieses gesondert zeigen
				if($this->active_page <= $this->link_count_surround){
					$this->links = $this->get_Outer_Links('left');
					$this->implodeLinks_and_add_delimiter3();
					$this->add_Delimiter4(false, true);
				} elseif($this->active_page >= $this->page_last - $this->link_count_surround){
					$this->links = $this->get_Outer_Links('right');

					$this->implodeLinks_and_add_delimiter3();
					$this->add_Delimiter4(true, false);
				}
				// Ansonsten die $show_count relevanten Links ermitteln und speichern
				else {
					$this->links = $this->get_Inner_Links();
					$this->implodeLinks_and_add_delimiter3();
					$this->add_Delimiter4(true, true);
				}
			}
			// Oder wenn nicht alle Links angezeigt werden sollen...
			// ...und die aeusseren, dann Links speichern
			elseif($this->show_outer === true){
				// Alle moeglichen Links sammeln
				$temp = [
					$this->get_Outer_Links('left'),
					$this->get_Inner_Links(),
					$this->get_Outer_Links('right')
				];

				// Alle drei Arrays in eins packen
				// Keine array_*-Funktion, da diese die Indizes loeschen
				$temp2 = $temp[0];
				for($i = 1; $i < 3; $i++){
					foreach($temp[$i] as $k => $v){
						$temp2[$k] = $v;
					}
				}
				$temp = $temp2;
				ksort($temp);
				$temp2 = [];

				// Zusammenhaengende Links (Abstand zum naechsten gleich 1) in eigene Arrays packen
				$last_index = 0;
				$j = 0;
				foreach($temp as $k => $v){
					if($k - $last_index > 1){
						$j++;
					}

					$temp2[$j][] = $v;

					$last_index = $k;
				}

				// Arrays zusammenhaengender Links zu Strings zusammenfassen
				// Kein Funktionsaufruf, da diese auf $this->links zugreifen
				$temp = [];

				foreach($temp2 as $cur){
					$temp[] = implode($this->delimiter3, $cur);
				}
				$this->linksStr = implode($this->delimiter4, $temp);
			}
		}

		// Links fuer Weiter/Zurueck und Anfang/Ende hinzufuegen, na ja, wenn es denn gewuenscht ist
		$this->add_Next_Prev();
		$this->add_First_Last();

		return $this->links;
	}

	/** Macht aus den im Array $links gepeicherten IDs Links mit allem Dran und Drum
	 *
	 * 	@access private
	 */
	private function make_links(){

		foreach($this->links as $i => &$cur){
			$temp = $i + 1;
			$cur = '<a href="' . $this->link_href[$i] . '"' .
				($i == $this->active_page ?
				$this->link_active_more . '>' . $this->link_active_bracket_left . $this->link_active_text . $temp . $this->link_active_bracket_right :
				$this->link_more . '>' . $this->link_bracket_left . $this->link_text . $temp . $this->link_bracket_right
				) . '</a>';
		}

		// Jetzt lassen wir den aktiven Link verschwinden - wenn denn gewuenscht
		if($this->show_active_link === false){
			unset($this->links[$this->active_page]);
		}
		$this->links = array_values($this->links);
	}

	/** Gibt Array der inneren Links zurueck
	 *
	 * 	@access private
	 */
	private function get_Inner_Links(){
		$temp = [];
		$from = $this->active_page - $this->link_count_surround;
		if($from < 0){
			$from = 0;
		}
		$to = $from + $this->show_count;
		if($to > $this->link_count - 1){
			$to = $this->link_count - 1;
		}

		for($i = $from; $i < $to; $i++){
			$temp[$i] = $this->links[$i];
		}

		return $temp;
	}

	/** Gibt Array mit aeusseren Links (links 'left' oder rechts 'right') zurueck
	 *
	 * 	@access private
	 */
	private function get_Outer_Links($side = null){
		$temp = [];

		if($side === 'left'){
			for($i = 0; $i < $this->show_count; $i++){
				$temp[$i] = $this->links[$i];
			}
		} elseif($side === 'right'){
			// Der erste Workaround, da sonst ein Link zu wenig gezeigt wird, wenn aktiver ausgeblendet
			if($this->show_active_link === false){
				$from = $this->link_count - $this->show_count - 1;
			} else {
				$from = $this->link_count - $this->show_count;
			}
			$to = $from + $this->show_count;

			for($i = $from; $i < $to; $i++){
				$temp[$i] = $this->links[$i];
			}
		}

		return $temp;
	}

	/** Macht aus den im Array $links gepeicherten IDs Links mit allem Dran und Drum (siehe Beispiele)
	 *
	 * 	@access private
	 */
	private function add_Delimiter1(){
		if($this->page_first !== null){
			$this->linksStr = $this->delimiter1 . $this->linksStr;
		}
		if($this->page_last !== null){
			$this->linksStr .= $this->delimiter1;
		}
	}

	/** Macht aus den im Array $links gepeicherten IDs Links mit allem Dran und Drum (siehe Beispiele)
	 *
	 * 	@access private
	 */
	private function add_Delimiter2(){
		if($this->page_prev !== null &&
			// Verhindern, dass Delimiter2 gezeigt wird, wenn Delimiter4 schon da ist (pure Schoenheit)
			!str_starts_with($this->linksStr, $this->delimiter4)){
			$this->linksStr = $this->delimiter2 . $this->linksStr;
		}
		if($this->page_next !== null &&
			!str_ends_with($this->linksStr, $this->delimiter4)){
			$this->linksStr .= $this->delimiter2;
		}
	}

	/** Macht aus den im Array $links gepeicherten IDs Links mit allem Dran und Drum (siehe Beispiele)
	 *
	 * 	@access private
	 */
	private function implodeLinks_and_add_delimiter3(): void{
		$this->linksStr = implode($this->delimiter3, $this->links);
	}

	/** Macht aus den im Array $links gepeicherten IDs Links mit allem Dran und Drum (siehe Beispiele)
	 *
	 * 	@access private
	 */
	private function add_Delimiter4(bool $left = false, bool $right = false){
		if($left){
			$this->linksStr = $this->delimiter4 . $this->linksStr;
		}
		if($right){
			$this->linksStr .= $this->delimiter4;
		}
	}

	/** Fuegt dem Array Links fuer Weiter/Zurueck hinzu - wenn denn gewuenscht
	 *
	 * 	@access private
	 */
	private function add_Next_Prev(){
		if($this->show_next_prev === true){
			$this->add_Delimiter2();

			$temp1 = $temp2 = '';
			if($this->page_prev !== null){
				$temp1 = '<a href="' . $this->link_href . ($this->page_prev - $this->steps_next_prev + 1) . '"' . $this->link_next_prev_more . '>' . $this->link_prev_text . '</a>';
			}
			if($this->page_next !== null){
				$temp2 = '<a href="' . $this->link_href . ($this->page_next + $this->steps_next_prev - 1) . '"' . $this->link_next_prev_more . '>' . $this->link_next_text . '</a>';
			}
			$this->linksStr = $temp1 . $this->linksStr . $temp2;
		}
	}

	/** Fuegt dem Array Links fuer Anfang/Ende hinzu - wenn denn gewuenscht
	 *
	 * 	@access private
	 */
	private function add_First_Last(){
		if($this->show_first_last === true){
			$this->add_Delimiter1();

			$temp1 = $temp2 = '';
			if($this->page_first !== null){
				$temp1 = "<a href='{$this->link_href}{$this->page_first}'{$this->link_first_last_more}>" . $this->link_first_text . '</a>';
			}
			if($this->page_last !== null){
				$temp2 = "<a href='{$this->link_href}{$this->page_last}'{$this->link_first_last_more}>" . $this->link_last_text . '</a>';
			}
			$this->linksStr = $temp1 . $this->linksStr . $temp2;
		}
	}

	/** Setzt Wert fuer Zahl der aktuelle Seite
	 * 	<b>Achtung</b> 10 hier, bedeutet bei der Ausgabe Seite 11
	 * 	Dieser Wert MUSS dynamisch festgelegt werden
	 * 	'ap' MUSS mit der letzten Variable bei $link_href uebereinstimmen (ap = active page)
	 *
	 * 	@access public
	 * 	@param (int) $active_page						Zahl der aktuellen Seite, erhaelt man ungefaehr so: "$active_page=isset($_GET['ap']) ? $_GET['ap'] : 0;"
	 * 	@return (bool)									Gibt false zurueck, wenn $active_page keinen gueltigen Wert hat
	 */
	function set_Active_Page(int $active_page): void{
		$this->active_page = $active_page;
	}

	/** Setzt Wert fuer Gesamtzahl aller Eintraege
	 *
	 * 	@access public
	 * 	@param (int) $entries							Anzahl aller Ergebnisse; erhaelt man ungefaehr so: "$entries=num_rows($resource);"
	 * 	@return (bool)									Gibt false zurueck, wenn $entries keinen gueltigen Wert hat
	 */
	function set_Entries(int $entries): void{
		$this->entries = $entries;
	}

	/** Setzt Wert fuer Anzahl Ergebnis pro Seite, die angezeigt werden sollen
	 *
	 * 	@access public
	 * 	@param (int) $epp								Anzahl Ergebnis pro Seite, die angezeigt werden sollen
	 * 	@return (bool)									Gibt false zurueck, wenn $epp keinen gueltigen Wert hat
	 */
	function set_Entries_Per_Page(int $epp): bool{
		if($epp >= 0){
			$this->epp = $epp;
			return true;
		}
		return false;
	}

	/** Setzt Wert fuer Link, zu dem verwiesen werden soll
	 * 	<b>WICHTIG</b> Die Variable, die den Wert der aktuellen Seite enthaelt muss ganz am Ende stehen und dahinter ein = (siehe Beispiele)!!
	 * 	Die Seitenzahl wird immer hinten angehangen, weshalb das unbedingt beachtet werden muss!
	 *
	 * 	@access public
	 * 	@param (string) $link_href						Link, zu dem verwiesen werden soll
	 * 	@return (bool)									Gibt false zurueck, wenn $link_href keinen gueltigen Wert hat
	 */
	function set_Link_Href(string $link_href): void{
		if(!str_ends_with($link_href, '=')){
			$link_href .= '=';
		}
		$this->link_href = trim($link_href);
	}

	/** Setzt Trennzeichen zwischen Links Anfang/Zurueck und Weiter/Ende
	 *
	 * 	@access public
	 * 	@param (string) $delimiter1						Zeichen, die zwischen Links Anfang/Zurueck und Weiter/Ende angezeigt werden (siehe 'Alle moeglichen Varianten')
	 * 	@return (bool)									Gibt false zurueck, wenn $delimiter1 keinen gueltigen Wert hat
	 */
	function set_Delimiter1(string $delimiter1 = ' -- '): void{
		$this->delimiter1 = $delimiter1;
	}

	/** Setzt Trennzeichen zwischen Links Zurueck und den Seitenzahlen sowie Weiter und den Seitenzahlen
	 *
	 * 	@access public
	 * 	@param (string) $delimiter2						Zeichen, die zwischen Links Zurueck und den Seitenzahlen sowie Weiter und den Seitenzahlen angezeigt werden (siehe 'Alle moeglichen Varianten')
	 * 	@return (bool)									Gibt false zurueck, wenn $delimiter2 keinen gueltigen Wert hat
	 */
	function set_Delimiter2(string $delimiter2 = ' - '): void{
		$this->delimiter2 = $delimiter2;
	}

	/** Setzt Trennzeichen zwischen Links der einzelnen Seiten
	 *
	 * 	@access public
	 * 	@param (string) $delimiter3						Zeichen, die zwischen Links der einzelnen Seiten gezeigt weden (siehe 'Alle moeglichen Varianten')
	 * 	@return (bool)									Gibt false zurueck, wenn $delimiter3 keinen gueltigen Wert hat
	 */
	function set_Delimiter3(string $delimiter3 = ', '): void{
		$this->delimiter3 = $delimiter3;
	}

	/** Setzt Trennzeichen zwischen Links der einzelnen Seiten
	 *
	 * 	@access public
	 * 	@param (string) $delimiter4						Zeichen, die zwischen Links der einzelnen Seiten gezeigt werden (siehe 'Alle moeglichen Varianten')
	 * 	@return (bool)									Gibt false zurueck, wenn $delimiter4 keinen gueltigen Wert hat
	 */
	function set_Delimiter4(string $delimiter4 = ' ... '): void{
		$this->delimiter4 = $delimiter4;
	}

	/** Setzt Wert fuer Zeichen vor und hinter aktivem Link
	 *
	 * 	@access public
	 * 	@param (string) $link_active_bracket_left		Zeichen vor aktivem Link (siehe 'Variablen')
	 * 	@param (string) $link_active_bracket_right		Zeichen hinter aktivem Link (siehe 'Variablen')
	 * 	@return (bool)									Gibt false zurueck, wenn $link_active_bracket_left oder $link_active_bracket_right keinen gueltigen Wert hat
	 */
	function set_Link_Active_Bracket(string $link_active_bracket_left = '[', string $link_active_bracket_right = ']'): void{
		$this->link_active_bracket_left = trim($link_active_bracket_left);
		$this->link_active_bracket_right = trim($link_active_bracket_right);
	}

	/** Setzt erweiterte Einstellungen des aktiven Links
	 *
	 * 	@access public
	 * 	@param (string) $link_active_more				Erweitere Einstellungen des aktiven Links (JavaScript, CSS, target, ...)
	 * 	@return (bool)									Gibt false zurueck, wenn $link_active_more keinen gueltigen Wert hat
	 */
	function set_Link_Active_More(string $link_active_more = ''): void{
		$link_active_more = ' ' . trim($link_active_more);
		$this->link_active_more = $link_active_more;
	}

	/** Setzt Wert fuer Zeichen vor und hinter jeden (nicht aktiven) Link
	 *
	 * 	@access public
	 * 	@param (string) $link_bracket_left				Zeichen vor jedem (nicht aktiven) Link (siehe 'Variablen')
	 * 	@param (string) $link_bracket_right				Zeichen hinter jedem (nicht aktiven) Link (siehe 'Variablen')
	 * 	@return (bool)									Gibt false zurueck, wenn $link_bracket_left keinen gueltigen Wert hat
	 */
	function set_Link_Bracket_Left(string $link_bracket_left = '', string $link_bracket_right = ''): void{
		$this->link_bracket_left = trim($link_bracket_left);
		$this->link_bracket_right = trim($link_bracket_right);
	}

	/** Setzt erweiterte Einstellungen fuer jeden (nicht aktiven) Link
	 *
	 * 	@access public
	 * 	@param (string) $link_more						Erweitere Einstellungen des jedes (nicht aktiven) Links (JavaScript, CSS, target, ...)
	 * 	@return (bool)									Gibt false zurueck, wenn $link_more keinen gueltigen Wert hat
	 */
	function set_Link_More(string $link_more = ''): void{
		$link_more = ' ' . trim($link_more);
		$this->link_more = $link_more;
	}

	/** Setzt Text fuer Links Zurueck/Weiter
	 *
	 * 	@access public
	 * 	@param (string) $link_next_text					Text des Links Weiter
	 * 	@param (string) $link_prev_text					Text des Links Zurueck
	 * 	@return (bool)									Gibt false zurueck, wenn $link_next_text oder $link_prev_text keinen gueltigen Wert hat
	 */
	function set_Link_Next_Prev_Text(string $link_next_text = 'Weiter', string $link_prev_text = 'Zur&uuml;ck'): void{
		$this->link_next_text = $link_next_text;
		$this->link_prev_text = $link_prev_text;
	}

	/** Setzt erweiterte Einstellungen fuer Links Zurueck/Weiter
	 *
	 * 	@access public
	 * 	@param (string) $link_next_prev_more			Erweitere Einstellungen Links Zurueck/Weiter (JavaScript, CSS, target, ...)
	 * 	@return (bool)									Gibt false zurueck, wenn $link_next_prev_more keinen gueltigen Wert hat
	 */
	function set_Link_Next_Prev_More(string $link_next_prev_more = ''): void{
		$link_next_prev_more = ' ' . trim($link_next_prev_more);
		$this->link_next_prev_more = $link_next_prev_more;
	}

	/** Setzt Text fuer Links Anfang/Ende
	 *
	 * 	@access public
	 * 	@param (string) $link_first_text				Text des Links Anfang
	 * 	@param (string) $link_last_text					Text des Links Ende
	 * 	@return (bool)									Gibt false zurueck, wenn $link_first_text oder $link_last_text keinen gueltigen Wert hat
	 */
	function set_Link_First_Last_Text(string $link_first_text = 'Anfang', string $link_last_text = 'Ende'): void{
		$this->link_first_text = $link_first_text;
		$this->link_last_text = $link_last_text;
	}

	/** Setzt erweiterte Einstellungen fuer Links Anfang/Ende
	 *
	 * 	@access public
	 * 	@param (string) $link_first_last_more			Erweitere Einstellungen Links Anfang/Ende (JavaScript, CSS, target, ...)
	 * 	@return (bool)									Gibt false zurueck, wenn $link_first_last_more keinen gueltigen Wert hat
	 */
	function set_Link_First_Last_More(string $link_first_last_more = ''): void{
		$link_first_last_more = ' ' . trim($link_first_last_more);
		$this->link_first_last_more = $link_first_last_more;
	}

	/** Setzt den Wert, wie viele Links gezeigt werden sollen
	 *
	 * 	@access public
	 * 	@param (int) $show_count						Gibt Zahl der Links an, die nebeneinander gezeigt werden (ungerade Zahlen sind idealer)
	 * 	@return (bool)									Gibt false zurueck, wenn $show_count keinen gueltigen Wert hat
	 */
	function set_Show_Count(int $show_count = 3): bool{
		if($show_count >= 3){
			$this->show_count = ceil($show_count);
			return true;
		}
		return false;
	}

	/** Setzt den Wert, um wie viele Seiten bei Weiter/Zurueck gesprungen werden soll
	 *
	 * 	@access public
	 * 	@param int $steps_next_prev					Gibt Zahl der Seiten an, um die bei Weiter/Zurueck gesprungen werden soll
	 * 	@return (bool)								Gibt false zurueck, wenn $steps_next_prev keinen gueltigen Wert hat
	 */
	function set_Steps_Next_Prev(int $steps_next_prev = 0): bool{
		if($steps_next_prev >= 1){
			$this->steps_next_prev = intval(round($steps_next_prev));
			return true;
		}
		return false;
	}

	/** Setzt den Wert, ob alle Links auf einmal gezeigt werden sollen
	 *
	 * 	@access public
	 * 	@param (bool) $show_continuous					Wenn true, dann werden alle Links auf einmal gezeigt
	 * 	@return (bool)									Gibt false zurueck, wenn $show_outer keinen gueltigen Wert hat
	 */
	function set_Show_Continuous(bool $show_continuous = false): void{
		$this->show_continuous = $show_continuous;
	}

	/** Setzt den Wert, ob aeussere Links der Seiten gezeigt werden sollen
	 *
	 * 	@access public
	 * 	@param (bool) $show_outer						Wenn true, dann werden aeusseren Links der Seiten angezeigt (siehe 'Alle moeglichen Varianten')
	 * 	@return (bool)									Gibt false zurueck, wenn $show_outer keinen gueltigen Wert hat
	 */
	function set_Show_Outer(bool $show_outer = true): void{
		$this->show_outer = $show_outer;
	}

	/** Setzt den Wert, ob Link der aktuellen Seite gezeigt werden soll
	 *
	 * 	@access public
	 * 	@param (bool) $show_active_link					Wenn true, dann wird der Link der aktuellen Seite angezeigt
	 * 	@return (bool)									Gibt false zurueck, wenn $show_active_link keinen gueltigen Wert hat
	 */
	function set_Show_Active_Link(bool $show_active_link = true): void{
		$this->show_active_link = $show_active_link;
	}

	/** Setzt den Wert, ob Links fuer Zurueck/Weiter gezeigt werden sollen
	 *
	 * 	@access public
	 * 	@param (bool) $show_next_prev					Wenn true, dann werden Links fuer Zurueck/Weiter angezeigt
	 * 	@return (bool)									Gibt false zurueck, wenn $show_next_prev keinen gueltigen Wert hat
	 */
	function set_Show_Next_Prev(bool $show_next_prev = true): void{
		$this->show_next_prev = $show_next_prev;
	}

	/** Setzt den Wert, ob Links fuer Anfang/Ende gezeigt werden sollen
	 *
	 * 	@access public
	 * 	@param (bool) $show_first_last					Wenn true, dann werden Links fuer Anfang/Ende angezeigt
	 * 	@return (bool)									Gibt false zurueck, wenn $show_first_last keinen gueltigen Wert hat
	 */
	function set_Show_First_Last(bool $show_first_last = true): void{
		$this->show_first_last = $show_first_last;
	}

	/** Setzt den Wert, ob ein einzelner Link gezeigt werden sollen
	 *
	 * 	@access public
	 * 	@param (bool) $show_single_link					Wenn true, wird auch bei nur einer Seite ein Link zurueckgegeben, sonst nichts
	 * 	@return (bool)									Gibt false zurueck, wenn $show_single_link keinen gueltigen Wert hat
	 */
	function set_Show_Single_Link(bool $show_single_link = false): void{
		$this->show_single_link = $show_single_link;
	}

	/** Gibt Zahl zu zeigender Ergebnisse pro Seite zurueck
	 *
	 * 	@access public
	 * 	@return (int)									Zahl zu zeigender Ergebnisse pro Seite
	 */
	function get_Epp(): int{
		return $this->epp;
	}

	/** Gibt (eventuell korrigierte) Zahl der aktuellen Seite zurueck
	 *
	 * 	@access public
	 * 	@return (int)									(eventuell korrigierte) Zahl der aktuellen Seite
	 */
	function get_Active_Page(): int{
		return $this->active_page;
	}

}
