<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 74610b487d71 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-03 00:24 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifPosition extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		//	content is not needed in this tag
		self::attributeMissing($attribs, ['type' => false, 'position' => false], __CLASS__);

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$position = self::getAttribute('position', $attribs, [], we_base_request::STRING_LIST);
		$size = count($position);
		$operator = self::getAttribute('operator', $attribs, '', we_base_request::STRING);

		switch($type){
			case 'listdir': //	inside a listdir
			case 'listview' : //	inside a listview, we take direct global listview object
				foreach($position as $pos){
					$tmp = self::op($pos, $size, $operator, we_tag_listview::$current->count, we_tag_listview::$current->anz);
					if($tmp !== null){
						return $tmp;
					}
				}
				break;

			case 'block' : //	look in function we_tag_block for details
				if(($reference = self::getAttribute('reference', $attribs, '', we_base_request::STRING))){
					foreach(array_reverse(we_tag_block::$position) as $arr){
						if($arr['oname'] === $reference){
							$reference = $arr;
							break;
						}
					}
				} else {
					$reference = end(we_tag_block::$position);
				}
				if(empty($reference)){
					return false;
				}
				foreach($position as $pos){
					$tmp = self::op($pos, $size, $operator, (int) $reference['position'], (int) $reference['size']);
					if($tmp !== null){
						return $tmp;
					}
				}
				break;

			default:
				return false;
		}
		return false;
	}

	protected static function op(string $position, int $size, string $operator, int $LVposition, int $LVsize): ?bool{
		switch($position){
			case 'first' :
				if($size == 1 && $operator != ''){
					switch($operator){
						case 'equal':
							return $LVposition == 1;
						case 'less':
							return $LVposition < 1;
						case 'less|equal':
							return $LVposition <= 1;
						case 'greater':
							return $LVposition > 1;
						case 'greater|equal':
							return $LVposition >= 1;
					}
				} else {
					if($LVposition == 1){
						return true;
					}
				}
				break;
			case 'last' :
				if($size == 1 && $operator != ''){
					switch($operator){
						case 'equal':
							return $LVposition == $LVsize;
						case 'less':
							return $LVposition < $LVsize;
						case 'less|equal':
							return $LVposition <= $LVsize;
						case 'greater|equal':
							return $LVposition >= $LVsize;
					}
				} else {
					if($LVposition == $LVsize){
						return true;
					}
				}
				break;
			case 'odd':
				if($LVposition % 2 != 0){
					return true;
				}
				break;
			case 'even':
				if($LVposition % 2 == 0){
					return true;
				}
				break;

			default:
				$position = intval($position); // Umwandeln in integer
				if($size == 1 && $operator != ''){
					switch($operator){
						case 'equal':
							return $LVposition == $position;
						case 'less':
							return $LVposition < $position;
						case 'less|equal':
							return $LVposition <= $position;
						case 'greater':
							return $LVposition > $position;
						case 'greater|equal':
							return $LVposition >= $position;
						case 'every':
							return ($LVposition % $position == 0);
					}
				} else {
					if(($operator === 'every' && ($LVposition % $position == 0)) || $LVposition == $position){
						return true;
					}
				}
				break;
		}
		//no decission yet
		return null;
	}

}
