<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 493097bf2738 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-11-18 00:41 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_link extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		/* self::attributeMissing($attribs, ['name' => false], __CLASS__); */
		$name = self::getAttribute('name', $attribs, self::getPostName('link'), we_base_request::STRING);
		$text = self::getAttribute('text', $attribs, '', we_base_request::STRING);
		$imageid = self::getAttribute('imageid', $attribs, 0, we_base_request::INT);
		$id = self::getAttribute('id', $attribs, 0, we_base_request::INT);
		self::$doc = we_base_WE::getWeDoc();

		// check if target document exists (Bug #7167)
		if($id != 0 && (!fI('SELECT 1,IsPublished=1 AS IsPublished FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id)))){
			$id = 0;
		}
		if($imageid != 0 && (!fB('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($imageid)))){
			$imageid = 0;
			$id = 0;
		}

		$attribs = self::removeAttribs($attribs, ['text', 'id', 'imageid', 'to', 'nameto', '_name_orig']);

		if(self::$doc instanceof we_document_triggerDocument){
			$link = self::$doc->getElementA($attribs['name']);
		} else {
			$link = we_unserialize(self::$doc->getElementS($name));
		}
		'@phan-var array{name:string,text:string,img_src:string,ctype:?string,img_id:int|string,width:string,height:string,border:string,alt:?string,img_title:?string} $link';
		if(!we_base_WE::$editmode){
			return self::$doc->getFieldLink($link, $attribs, false, !we_base_WE::$mainDoc['doc'] ? self::$doc->ParentID : we_base_WE::$mainDoc['doc']->ParentID, !we_base_WE::$mainDoc['doc'] ? self::$doc->Path : we_base_WE::$mainDoc['doc']->Path);
		}

		if(!$link){
			$link = [
				'id' => ($id ?? ''),
				'ctype' => ((!empty($imageid)) ? we_base_link::CONTENT_INT : we_base_link::CONTENT_TEXT),
				'img_id' => ((!empty($imageid) ) ? $imageid : ''),
				'type' => (isset($id) ? we_base_link::TYPE_INT : we_base_link::TYPE_EXT),
				'href' => (isset($id) ? '' : we_base_link::EMPTY_EXT),
				'text' => (!empty($imageid) ? (!empty($text) ? $text : g_l('global', '[new_link]')) : '')
			];

			// Link should only displayed if it's a preset link
			if($id || $imageid != 0 || $text){
				$_SESSION['weS']['WE_LINK'] = $link;
				self::$doc->changeLink($name);
				self::$doc->saveInSession(we_base_WE::$transaction);
			}
		}

		$img = new we_document_image();
		$content = we_document_base::getLinkContent($link, $img);

		$startTag = self::$doc->getLinkStartTag($link, $attribs, we_base_WE::$mainDoc['doc']->ParentID, we_base_WE::$mainDoc['doc']->Path, $img);
		$editbut = we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:setScrollTo(); we_cmd('edit_link', {transaction: '" . we_base_WE::$transaction . "', fieldname: '" . $name . "'})");
		$delbut = we_html_button::create_button(we_html_button::TRASH, "javascript:setScrollTo(); we_cmd('delete_link', '" . $name . "')");

		return we_html_element::span(['class' => 'we we_link'], ($startTag ?: '') . ($content ?: $text) . ($startTag ? '</a>' : '') . $editbut . $delbut);
	}

}
