<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_sum extends we_weTag_tag{
	/** @var array<string, float> */
	private static $sum = [];

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);
		$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$result = self::$sum[$name] ?? 0;

		return we_base_util::formatNumber($result, self::getAttribute('num_format', $attribs, '', we_base_request::STRING), self::getAttribute('decimals', $attribs, 2, we_base_request::INT));
	}

	public static function getSum(string $name): ?float{
		return self::$sum[$name] ?? null;
	}

	public static function addSum(string $name, float $value): void{
		if(!isset(self::$sum[$name])){
			self::$sum[$name] = 0;
		}
		self::$sum[$name] += $value;
	}

}
