<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_voting extends we_weTag_tag{
	/** @var string */
	public static $namespace = '';

	/** @var ?we_voting_voting */
	public static $currentVoting = null;

	public static function parse(array $attribs, string $content): string{
		return '<?php ' . we_weTag_parser::printTag('voting', $attribs) . ';?>' . $content . '<?php ' . we_weTag_parser::printTag('condition', ['_type' => 'stop']) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(!we_modules_info::isActive(we_modules_info::VOTING)){
			return self::modulMissingError('Voting', __CLASS__);
		}
		self::$doc = we_base_WE::getWeDoc();
		if(self::getAttribute('_type', $attribs, '', we_base_request::STRING) === 'stop'){
			self::$namespace = '';
			self::$currentVoting = null;
			return '';
		}

		self::attributeMissing($attribs, ['name' => false], __CLASS__);

		$id = self::getAttribute('id', $attribs, 0, we_base_request::INT);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$version = self::getAttribute('version', $attribs, 0, we_base_request::INT);

		self::$namespace = $name;

		if(self::$doc->issetElement($name)){
			self::$currentVoting = new we_voting_voting(self::$doc->getElementI($name));
		} else if($id != 0){
			self::$currentVoting = new we_voting_voting($id);
		} else {
			$voting_matches = [];
			self::$currentVoting = (preg_match_all('/_we_voting_answer_([0-9]+)_?([0-9]+)?/', implode(',', array_keys($_REQUEST)), $voting_matches) ?
				new we_voting_voting((int) $voting_matches[1][0]) :
				new we_voting_voting());
		}

		self::$currentVoting->setDefVersion(max(--$version, 0));
		return '';
	}

}
