<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 80c72f54723f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-03-29 16:27 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_tree_customer extends we_tree_base{
	private static $sortField = '';

	public function __construct(we_base_jsCmd $jsCmd){
		$this->autoload = false;
		parent::__construct($jsCmd);
		$this->addSorted = false;
	}

	public static function setSort(string $sort): void{
		self::$sortField = $sort;
	}

	protected function customJSFile(): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_tree.js', 'initTree();');
	}

	public static function getItems(int $pid, int $offset = 0, int $segment = 500, bool $sort = false): array{
		if(empty($pid)){
			$_SESSION['we']['custTree'] = [];
		}
		return (empty(self::$sortField) ?
			static::getItemsFromDB($pid, $offset, $segment) :
			static::getSortFromDB($pid, self::$sortField, $offset, $segment));
	}

	private static function getItemsFromDB(int $ParentID = 0, int $offset = 0, int $segment = 500, string $elem = 'ID,Forename,Surname', string $addWhere = ''): array{
		$prevoffset = max(0, $offset - $segment);
		$items = ($offset && $segment ? [
			[
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => CUSTOMER_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'published' => 1,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $prevoffset
			]
			] : []);


		$settings = new we_customer_settings();
		$settings->load();

		$ws = we_users_util::get_ws(CUSTOMER_TABLE);
		$where = ' WHERE 1 ' .
			($ws ? ' AND ' . implode('', $ws) : '') .
			' ' . $addWhere;

		foreach(DB_WE::$inst->getAllq('SELECT ' . $settings->treeTextFormatSQL . ' AS treeFormat,' . $elem . ',LoginDenied FROM ' . CUSTOMER_TABLE . ' ' . $where . ' ' . self::getSortOrder($settings) . ($segment ? ' LIMIT ' . $offset . ',' . $segment : '')) as $rec){
			$isFolder = !empty($rec['IsFolder']);
			$items[] = [
				'typ' => ($isFolder ? self::TYP_FOLDER : self::TYP_ITEM),
				'contenttype' => ($isFolder ? we_base_ContentTypes::CUSTOMERGROUP : we_base_ContentTypes::CUSTOMER),
				'disabled' => 0,
				'path' => '',
				'published' => $rec['LoginDenied'] ? 0 : 1,
				'tooltip' => ((int) $rec['ID']),
				'offset' => $offset,
				'id' => ((int) $rec['ID']),
				'parentid' => 0,
				'isfolder' => 0,
				'text' => htmlspecialchars($rec['treeFormat'])
			];
		}

		$total = fI('SELECT COUNT(1) FROM ' . CUSTOMER_TABLE . ' ' . $where);
		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			$items[] = [
				'id' => 'next_' . $ParentID,
				'parentid' => 0,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => CUSTOMER_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}

		return $items;
	}

	private static function getSortFromDB(int $pid, string $sort, int $offset = 0, int $segment = 500): array{
		$items = $select = $grouparr = $orderarr = $havingarr = $sort_defs = [];
		$noVal = g_l('modules_customer', '[no_value]');

		$settings = new we_customer_settings();
		$settings->load();
		$ws = we_users_util::get_ws(CUSTOMER_TABLE);

		$pos = $pid ? ($pid * -1) - 1 : 0;
		[$level, $parent, $name] = $pid && isset($_SESSION['we']['custTree'][$pos]) ? $_SESSION['we']['custTree'][$pos] : [0, 0, null];
		//get all names of previous "dirs"
		$pidarr = $name !== null ? [$name] : [];
		while($parent > 0){
			[, $parent, $pname] = $_SESSION['we']['custTree'][$parent];
			$pidarr[] = $pname;
		}
		$pidarr = array_reverse($pidarr);

		if(isset($settings->SortView[$sort])){
			$sort_defs = $settings->SortView[$sort];
		}

		$levelcount = count($sort_defs);
		$grp = '';
		foreach($sort_defs as $c => $sortdef){
			if(!empty($sortdef['function'])){
				$fkt = sprintf(we_customer_settings::FUNCTION_TABLE[$sortdef['function']], $sortdef['field']);
				$select[] = $fkt . ' AS ' . $sortdef['field'] . '_' . $sortdef['function'];

				$grouparr[] = $sortdef['field'] . '_' . $sortdef['function'];
				$orderarr[] = $sortdef['field'] . '_' . $sortdef['function'] . ' ' . $sortdef['order'];
				if(isset($pidarr[$c])){
					$havingarr[] = ($pidarr[$c] === $noVal ?
						'(' . $fkt . "='' OR " . $fkt . ' IS NULL)' :
						$fkt . "='" . $pidarr[$c] . "'");
				}
				if($level < $levelcount && $c == $level){
					$grp = $fkt;
				}
				continue;
			}
			$select[] = $sortdef['field'];
			$grouparr[] = $sortdef['field'];
			$orderarr[] = $sortdef['field'] . ' ' . $sortdef['order'];
			if(!empty($pidarr[$c])){
				$havingarr[] = ($pidarr[$c] === $noVal ?
					'(' . $sortdef['field'] . "='' OR " . $sortdef['field'] . ' IS NULL)' :
					$sortdef['field'] . "='" . $pidarr[$c] . "'");
			}
			if($level < $levelcount && $c == $level){
				$grp = $sortdef['field'];
			}
		}

		$select = array_filter($select);
		$elements = DB_WE::$inst->getAllq('SELECT
	' . ($level < $levelcount ? 'COUNT(1) AS Anz,' : '') .
			$settings->treeTextFormatSQL . ' AS treeFormat,
	ID,
	LoginDenied,
	Forename,
	Surname' .
			($select ? ',' . implode(',', $select) : '' ) . '
FROM ' .
			CUSTOMER_TABLE .
			' WHERE ' .
			($ws ? implode('', $ws) : '1') .
			($havingarr ? ' AND ' . implode(' AND ', $havingarr) : '') .
			($grp ? ' GROUP BY ' . $grp : '') . ' ORDER BY ' . implode(',', $orderarr) . self::getSortOrder($settings, ($orderarr ? ',' : '')) . (($level == $levelcount && $segment) ? ' LIMIT ' . $offset . ',' . $segment : ''));

		$gid = -1 * count($_SESSION['we']['custTree']);

		if($level < $levelcount){
			foreach($elements as $e){
				$gname = $e[(string) $grouparr[$level]] ?? $noVal;
				$_SESSION['we']['custTree'][] = [$level + 1, $pos, $gname];
				$groupTotal = $e['Anz'];
				$items[] = [
					'id' => --$gid,
					'parentid' => $pid,
					'path' => '',
					'text' => ($e[(string) $grouparr[$level]] ?? $noVal) . ' (' . $groupTotal . ')',
					'contenttype' => we_base_ContentTypes::FOLDER_COMMON,
					'isfolder' => 1,
					'typ' => self::TYP_FOLDER,
					'disabled' => 0,
					'open' => false,
				];
			}
			return $items;
		}
		//		$level == $levelcount
		$prevoffset = max(0, $offset - $segment);
		if($offset && $segment){
			$items[] = [
				'id' => 'prev_' . $pid,
				'parentid' => $pid,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => CUSTOMER_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'published' => 1,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $prevoffset
			];
		}
		foreach($elements as $e){
			$tt = $e['treeFormat'];

			$items[] = [
				'id' => $e['ID'],
				'parentid' => $pid,
				'path' => '',
				'text' => htmlspecialchars($tt),
				'contenttype' => we_base_ContentTypes::CUSTOMER,
				'isfolder' => 0,
				'typ' => self::TYP_ITEM,
				'disabled' => 0,
				'published' => $e['LoginDenied'] ? 0 : 1,
				'tooltip' => $e['ID']
			];
		}

		$total = fI('SELECT COUNT(ID) FROM ' . CUSTOMER_TABLE .
			' WHERE ' .
			($ws ? implode('', $ws) : '1') .
			($havingarr ? ' AND ' . implode(' AND ', $havingarr) : '') .
			($grp ? ' GROUP BY ' . $grp : ''));

		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			$items[] = [
				'id' => 'next_' . $pid,
				'parentid' => $pid,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => CUSTOMER_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}

		return $items;
	}

	public static function getSortOrder(we_customer_settings $settings, string $concat = 'ORDER BY'): string{
		$ret = ($settings->getSettings('default_order') ?
			($settings->formatFields ?
			implode(' ' . $settings->getSettings('default_order') . ',', $settings->formatFields) . ' ' . $settings->getSettings('default_order') :
			'Username ' . $settings->getSettings('default_order')) :
			'Username');
		return ($ret ? $concat . ' ' . $ret : '');
	}

}
