<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: a202423df34b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-03-22 21:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_tree_newsletter extends we_tree_base{

	protected function customJSFile(): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_tree.js', 'initTree();');
	}

	public static function getItems(int $ParentID, int $offset = 0, int $segment = 500, bool $sort = false): array{
		$wsQuery = '';
		$items = $aWsQuery = $parentpaths = [];

		if(($ws = we_users_util::get_ws(NEWSLETTER_TABLE))){
			'@phan-var int[] $ws';
			$wsPathArray = id2pathA($ws, NEWSLETTER_TABLE);
			foreach($wsPathArray as $path){
				'@phan-var string $path';
				$aWsQuery[] = ' Path LIKE "' . $path . '/%" ';
				$parentpaths = array_merge($parentpaths, we_tree_base::getParents($path));
			}
			$parentpaths = array_unique($parentpaths);
			if($parentpaths){
				$aWsQuery[] = 'Path IN(' . implode(',', $parentpaths) . ')';
			}

			$wsQuery = $aWsQuery ? '(' . implode(' OR ', $aWsQuery) . ') AND ' : '';
		}

		$prevoffset = max(0, $offset - $segment);
		if($offset && $segment){
			$items[] = [
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => NEWSLETTER_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'published' => 0,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $prevoffset
			];
		}

		foreach(DB_WE::$inst->getAllq('SELECT ID,ParentID,Path,Text,IsFolder FROM ' . NEWSLETTER_TABLE . '  WHERE ' . $wsQuery . ' ParentID=' . $ParentID . ' ORDER BY IsFolder DESC,(text REGEXP "^[0-9]") DESC,abs(text),Text ' . ($segment ? 'LIMIT ' . $offset . ',' . $segment : '' )) as $item){
			'@phan-var array{ID:int,ParentID:int,Path:string,Text:string,IsFolder:int} $item';
			$typ = [
				'typ' => ($item['IsFolder'] ? self::TYP_FOLDER : self::TYP_ITEM),
				'open' => false,
				'tooltip' => $item['ID'],
				'offset' => $offset,
				'disabled' => in_array($item['Path'], $parentpaths) ? 1 : 0,
				'text' => $item['Text'],
				'path' => $item['Path'],
				'published' => 1,
				'contenttype' => ($item['IsFolder'] == 1 ? we_base_ContentTypes::FOLDER_COMMON : we_base_ContentTypes::NEWSLETTER),
			];

			$fields = array_change_key_case($item);

			$items[] = array_merge($fields, $typ);
		}

		$total = fI('SELECT COUNT(1) FROM ' . NEWSLETTER_TABLE . '  WHERE ' . $wsQuery . ' ParentID=' . $ParentID);
		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			$items[] = [
				'id' => 'next_' . $ParentID,
				'parentid' => 0,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => NEWSLETTER_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}

		return $items;
	}

}
