<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: adcae6a39191 $
 * $Author: Lukas Imhof $
 * $Date: 2021-10-21 11:48 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_voting_view extends we_modules_view{
	/** @var we_voting_voting */
	var $Model;

	function __construct(){
		parent::__construct(we_modules_info::VOTING);
		$this->Model = new we_voting_voting();
	}

	function getCommonHiddens(array $cmds = []): string{
		return
			parent::getCommonHiddens($cmds) .
			we_html_element::hiddens([
				'vernr' => ($cmds['vernr'] ?? 0),
				'IsFolder' => ($this->Model->IsFolder ?? '0')
		]);
	}

	function getJSTop(): string{
		return
			parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/voting/voting_top.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	function getJSProperty(array $jsVars = []): string{
		$this->Model->Owners = (is_array($this->Model->Owners) ? array_filter($this->Model->Owners) : []);
		$owners = ($this->Model->Owners ?
			$this->db->getAllFirstq('SELECT ID,Path,IsFolder FROM ' . USER_TABLE . ' WHERE ID IN (' . implode(',', $this->Model->Owners) . ')', true, MYSQLI_ASSOC) :
			[]);
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/voting/voting_prop.js', '', ['id' => 'loadVarVoting', 'data-voting' => setDynamicVar([
					'isFolder' => $this->Model->IsFolder,
					'delBut' => we_html_button::create_button(we_html_button::TRASH, "javascript:top.content.setHot();" . we_base_constants::PLACEHOLDER),
					'delBut1' => we_html_button::create_button(we_html_button::TRASH, "javascript:top.content.setHot();if(answers_edit.itemCount>answers_edit.minCount){ " . we_base_constants::PLACEHOLDER . "; }else{ callAnswerLimit();}"),
					'answerCount' => $this->Model->QASet ? count($this->Model->QASet[0]['answers']) : 0,
					'QASet' => $this->Model->QASet,
					'QASetAdditions' => $this->Model->QASetAdditions,
					'showVariant' => we_base_request::int('vernr', 0),
					'allow' => [
						'freeText' => $this->Model->AllowFreeText,
						'images' => $this->Model->AllowImages,
						'media' => $this->Model->AllowMedia,
						'successor' => $this->Model->AllowSuccessors
					],
					'owners' => $owners,
					'blackList' => is_array($this->Model->BlackList) ? $this->Model->BlackList : []
		])]);
	}

	function processCommands(we_base_jsCmd $jscmd): void{
		switch(we_base_request::string("cmd")){
			case 'resetscores':
				foreach($this->Model->Scores as &$val){
					$val = 0;
				}
				break;
			case 'new_voting':
			case 'new_voting_group':
				if(!we_base_permission::hasPerm([we_base_perm::VOTING_NEW])){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model = new we_voting_voting();
				$this->Model->IsFolder = we_base_request::string('cmd') === 'new_voting_group';
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);
				break;
			case 'we_voting_edit':
				if(!we_base_permission::hasPerm([we_base_perm::VOTING_EDIT])){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					$_REQUEST['home'] = '1';
					$_REQUEST['pnt'] = 'edbody';
					break;
				}

				$this->Model = new we_voting_voting(we_base_request::int("cmdid"));

				if(!$this->Model->isAllowedForUser()){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					$this->Model = new we_voting_voting();
					$_REQUEST['home'] = true;
					break;
				}
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);
				break;
			case 'save_voting':
				if(!we_base_permission::hasPerm([we_base_perm::VOTING_NEW, we_base_perm::VOTING_EDIT])){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(we_voting_voting::filenameNotValid($this->Model->Text)){
					$jscmd->addMsg(g_l('modules_voting', '[wrongtext]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(!trim($this->Model->Text)){
					$jscmd->addMsg(g_l('modules_voting', '[name_empty]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->Active && $this->Model->ActiveTime && $this->Model->Valid < new DateTime('now')){
					$jscmd->addMsg(g_l('modules_voting', '[not_active]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$oldpath = $this->Model->Path;
				// set the path and check it
				$this->Model->setPath();

				if($this->Model->pathExists($this->Model->Path)){
					$jscmd->addMsg(g_l('modules_voting', '[name_exists]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				if($this->Model->isSelf()){
					$jscmd->addMsg(g_l('modules_voting', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$q_empty = $a_empty = true;
				if(!$this->Model->IsFolder && !empty($this->Model->QASet)){
					foreach($this->Model->QASet as $set){
						$q = trim($set['question']);
						if($q === ''){
							$q_empty = true;
							break;
						}
						$q_empty = false;

						foreach($set['answers'] as $ans){
							$q = trim($ans);
							if($q === ''){
								$a_empty = true;
								break;
							}
							$a_empty = false;
						}
					}

					if($q_empty){
						$jscmd->addMsg(g_l('modules_voting', '[question_empty]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					} else if($a_empty){
						$jscmd->addMsg(g_l('modules_voting', '[answer_empty]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					}
				}

				if($this->Model->ParentID && !fB('SELECT IsFolder FROM ' . VOTING_TABLE . ' WHERE ID=' . intval($this->Model->ParentID))){
					$jscmd->addMsg(g_l('modules_voting', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$newone = ($this->Model->ID == 0);
				$this->Model->save(we_base_request::bool('scores_changed'));

				if($this->Model->IsFolder && $oldpath != '' && $oldpath != '/' && $oldpath != $this->Model->Path){
					$entries = $this->db->getAllq('SELECT ID FROM ' . VOTING_TABLE . ' WHERE Path LIKE "' . $this->db->escape($oldpath) . '%" AND ID!=' . intval($this->Model->ID), true);
					foreach($entries as $id){
						$this->db->query('UPDATE ' . VOTING_TABLE . ' SET Path="' . $this->Model->evalPath(intval($id), VOTING_TABLE) . '" WHERE ID=' . $id);
					}
				}

				if($newone){
					$jscmd->addCmd('makeTreeEntry', [
						'id' => $this->Model->ID,
						'parentid' => $this->Model->ParentID,
						'text' => $this->Model->Text,
						'open' => false,
						'contenttype' => ($this->Model->IsFolder ? we_base_ContentTypes::FOLDER_COMMON : we_base_ContentTypes::VOTING),
						'table' => VOTING_TABLE,
						'published' => ($this->Model->isActive() ? 1 : 0)
					]);
					$jscmd->addCmd('drawTree');
				} else {
					$jscmd->addCmd('updateTreeEntry', [
						'id' => $this->Model->ID,
						'parentid' => $this->Model->ParentID,
						'text' => $this->Model->Text,
						'published' => ($this->Model->isActive() ? 1 : 0)
					]);
				}
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);
				$jscmd->addMsg(g_l('modules_voting', ($this->Model->IsFolder ? '[save_group_ok]' : '[save_ok]')), we_base_util::WE_MESSAGE_NOTICE);

				break;
			case 'delete_voting':
				if(!we_base_permission::hasPerm([we_base_perm::VOTING_DELETE])){
					$jscmd->addMsg(g_l('modules_voting', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if($this->Model->delete()){
					$jscmd->addCmd('deleteTreeEntry', $this->Model->ID);
					$jscmd->addMsg(g_l('modules_voting', ($this->Model->IsFolder == 1 ? '[group_deleted]' : '[voting_deleted]')), we_base_util::WE_MESSAGE_NOTICE);
					$this->Model = new we_voting_voting();
					$_REQUEST['home'] = 1;
					$_REQUEST['pnt'] = 'edbody';
				} else {
					$jscmd->addMsg(g_l('modules_voting', '[nothing_to_delete]'), we_base_util::WE_MESSAGE_ERROR);
				}

				break;
			case 'switchPage':
				break;
			case 'export_csv':
				$fname = rtrim(we_base_request::_(we_base_request::FILE, 'csv_dir'), '/') . '/voting_' . $this->Model->ID . '_export_' . time() . '.csv';

				$enclose = we_base_request::string('csv_enclose', '"');
				$enclose = $enclose == 0 ? '"' : '\'';
				$delimiter = we_base_request::string(';');
				$delimiter = ($delimiter === '\t' ? "\t" : $delimiter);
				switch(we_base_request::string('csv_lineend')){
					default:
					case we_exim_ExportCSV::LINEEND_WINDOWS:
						$lineend = "\r\n";
						break;
					case we_exim_ExportCSV::LINEEND_UNIX:
						$lineend = "\n";
						break;
					case we_exim_ExportCSV::LINEEND_MAC:
						$lineend = "\r";
						break;
				}

				$content = [];
				$questName = we_base_request::string('question_name');
				if($questName && ($tmp = we_base_request::string($questName . '_item0'))){
					$content[] = $enclose . addslashes($tmp) . $enclose . $delimiter;
				}
				$answerName = we_base_request::string('answers_name');
				$cnt = we_base_request::int('item_count');
				if($answerName && $cnt){
					for($i = 0; $i < $cnt; $i++){
						if(($tmp = we_base_request::string($answerName . '_item' . $i))){
							$content[] = $enclose . addslashes($tmp) . $enclose . $delimiter . $this->Model->Scores[$i];
						}
					}
				}
				we_base_file::save(WE_DOCUMENT_ROOT . $fname, implode($lineend, $content));
				$_REQUEST['lnk'] = $fname;
				break;
			case 'exportGroup_csv':
				$fname = we_base_file::addPreSlash(we_base_request::_(we_base_request::FILE, 'csv_dir') . '/votingGroup_' . $this->Model->ID . '_export_' . time() . '.csv');

				$enclose = we_base_request::string('csv_enclose', '"');
				$enclose = $enclose == 0 ? '"' : $enclose;
				$delimiter = we_base_request::string('csv_delimiter', ';');
				$delimiter = $delimiter === '\t' ? "\t" : $delimiter;
				switch(we_base_request::string('csv_lineend')){
					default:
					case 'windows':
						$lineend = "\r\n";
						break;
					case 'unix':
						$lineend = "\n";
						break;
					case 'mac':
						$lineend = "\r";
						break;
				}

				$allData = $this->Model->loadDB();
				$CSV_Charset = we_base_request::string('the_charset', 'UTF-8');
				$content = [mb_convert_encoding($enclose . trim(g_l('modules_voting', '[voting-session]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[voting-id]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[time]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[ip]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[user_agent]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[cookie]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[log_fallback]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[status]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[answerID]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[answerText]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[voting-successor]')) . $enclose . $delimiter .
						$enclose . trim(g_l('modules_voting', '[voting-additionalfields]')) . $enclose . $delimiter, $CSV_Charset, 'UTF-8'
					)
				];

				foreach($allData as $data){
					$cookie = g_l('modules_voting', $data['cookie'] ? '[enabled]' : '[disabled]');
					$fallback = g_l('global', $data['fallback'] ? '[yes]' : '[no]');

					if($data['status'] != we_voting_voting::SUCCESS){
						switch($data['status']){
							case we_voting_voting::ERROR :
								$mess = g_l('modules_voting', '[log_error]');
								break;
							case we_voting_voting::ERROR_ACTIVE :
								$mess = g_l('modules_voting', '[log_error_active]');
								break;
							case we_voting_voting::ERROR_REVOTE :
								$mess = g_l('modules_voting', '[log_error_revote]');
								break;
							case we_voting_voting::ERROR_BLACKIP :
								$mess = g_l('modules_voting', '[log_error_blackip]');
								break;
							default:
								$mess = g_l('modules_voting', '[log_error]');
						}
					} else {
						$mess = g_l('modules_voting', '[log_success]');
					}

					$lineFields = [
						$data['votingsession'],
						$data['voting'],
						date(g_l('weEditorInfo', '[date_format]'), $data['time']),
						inet_ntop($data['ip']),
						$data['agent'],
						$cookie,
						$fallback,
						$mess,
						$data['answer'],
						$data['answertext'],
						$data['successor'],
					];

					if($data['additionalfields'] != ''){
						$addData = we_unserialize($data['additionalfields']);

						if(is_array($addData) && !empty($addData)){
							foreach($addData as $values){
								$lineFields[] = $values;
							}
						} else {
							$lineFields[] = '-';
						}
					} else {
						$lineFields[] = '-';
					}
					$content[] = $enclose . mb_convert_encoding(implode($enclose . $delimiter . $enclose, array_map('trim', array_map('strval', $lineFields))), $CSV_Charset, 'UTF-8') . $enclose . $delimiter;
				}

				we_base_file::save(WE_DOCUMENT_ROOT . $fname, implode($lineend, $content));
				$_REQUEST['lnk'] = $fname;
				break;

			default:
		}

		$_SESSION['weS']['module_sessions']['voting'] = $this->Model;
	}

	function processVariables(): void{

		if(isset($_SESSION['weS']['module_sessions']['voting'])){
			$this->Model = $_SESSION['weS']['module_sessions']['voting'];
		}

		foreach($this->Model->persistent_slots as $key => $type){
			if(($v = we_base_request::_($type, $key, '__UNSET__')) !== '__UNSET__'){
				$this->Model->{$key} = $v;
			}
		}

		if(($page = we_base_request::int('page', -1)) >= 0){
			$this->page = $page;
		}

		$qaset = $qaADDset = [];
		$qname = we_base_request::string('question_name');
		$vcount = we_base_request::int('variant_count');
		$aname = we_base_request::string('answers_name');
		$icount = we_base_request::int('item_count');
		if($qname && $vcount && $aname && $icount){
			for($i = 0; $i < $vcount; $i++){
				if(($quest = we_base_request::string($qname . '_variant' . $i . '_' . $qname . '_item0', '')) !== ''){
					$set = [
						'question' => addslashes($quest),
						'answers' => [],
					];

					$an = $aname . '_variant' . $i . '_' . $aname . '_item';
					$anImage = $an . 'ImageID';
					$anMedia = $an . 'MediaID';
					$anSuccessor = $an . 'SuccessorID';
					$addset = ['imageID' => [], 'mediaID' => [], 'successorID' => []];
					for($j = 0; $j < $icount; $j++){
						if(($tmp = we_base_request::string($an . $j, '')) !== ''){
							$set['answers'][] = addslashes($tmp);
						}
						if(($tmp = we_base_request::string($anImage . $j, '')) !== ''){
							$addset['imageID'][] = ($tmp != 'Array' ? addslashes($tmp) : 0);
						}
						if(($tmp = we_base_request::string($anMedia . $j, '')) !== ''){
							$addset['mediaID'][] = ($tmp != 'Array' ? addslashes($tmp) : 0);
						}
						if(($tmp = we_base_request::string($anSuccessor . $j, '')) !== ''){
							$addset['successorID'][] = ($tmp != 'Array' ? addslashes($tmp) : 0);
						}
					}
					$qaset[] = $set;
					$qaADDset[] = $addset;
				}
			}
		}

		$this->Model->QASet = $qaset;
		$this->Model->QASetAdditions = $qaADDset;

		/* FIXME: this doesn't work! multi_edit assumes the value is the same as the displayed item, so you get an image-tag & textual user here which is really not what we need. To fix this, multi_edit.js must distinguish between "label" & value!
		 * if(($on = we_base_request::string('owners_name')) && ($oc = we_base_request::int('owners_count'))){
		  $this->voting->Owners = [];
		  $an = $on . '_variant0_' . $on . '_item';
		  for($i = 0; $i < $oc; $i++){
		  if(($tmp = we_base_request::string($an . $i))){
		  $up = str_replace(array(stripslashes($this->item_pattern), stripslashes($this->group_pattern)), '', $tmp);
		  $this->voting->Owners[] = path_to_id($up, USER_TABLE);
		  }
		  }
		  $this->voting->Owners = array_unique($this->voting->Owners);
		  }
		 */

		$ipset = [];
		if(($in = we_base_request::string('iptable_name')) && ($ic = we_base_request::int('iptable_count'))){
			$in = $in . '_variant0_' . $in . '_item';
			for($i = 0; $i < $ic; $i++){
				if(($tmp = we_base_request::string($in . $i, '')) !== ''){
					$ipset[] = addslashes($tmp);
				}
			}
			$this->Model->BlackList = $ipset;
		}

		$dates = we_base_util::convertDateInRequest('voting');
		if(!empty($dates['Publish'])){
			$this->Model->PublishDate = $dates['Publish'];
		}
		if(!empty($dates['Valid'])){
			$this->Model->Valid = $dates['Valid'];
		}

		if(we_base_request::float('scores_0', -1) >= 0 && ($ic = we_base_request::int('item_count')) && we_base_request::bool('scores_changed')){
			$this->Model->Scores = [];
			for($j = 0; $j < $ic; $j++){
				if(($tmp = we_base_request::float('scores_' . $j))){
					$this->Model->Scores[] = $tmp;
				}
			}
		}
	}

	public function getHomeScreen(we_base_jsCmd $jscmd): string{
		$hiddens = ['cmd' => "home",
			'pnt' => 'edbody'
		];
		$content = we_html_button::create_button('new_voting', "javascript:we_cmd('new_voting');", '', "", !we_base_permission::hasPerm([we_base_perm::VOTING_NEW])) .
			'<br/>' .
			we_html_button::create_button('new_voting_group', "javascript:we_cmd('new_voting_group');", '', "", !we_base_permission::hasPerm([we_base_perm::VOTING_NEW]));

		return parent::getActualHomeScreen(we_modules_info::VOTING, $jscmd, $content, we_html_element::form(['name' => we_base_constants::WE_FORM], $this->getCommonHiddens($hiddens) . we_html_element::hidden("home", 0)));
	}

}
