<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e826d47dd2c3 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-01-23 14:59 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_weTag_option{
	/** @var string	 */
	public $Name;

	/**
	 * value of this option, if differs from $Name
	 * @var string
	 */
	public $Value;

	/** @var bool attribute disabled of this option	 */
	public $Disabled = false;

	/** @var array<we_weTag_attribute> all allowed attributes, when selecting this option	 */
	public $AllowedAttributes;

	/** @var array<we_weTag_attribute> required attributes, when selecting this option	 */
	public $RequiredAttributes;

	/** @var string	 */
	public $Module;

	/**
	 * @param string $name
	 * @param mixed $value
	 * @param array<we_weTag_attribute> $allowedAttributes
	 * @param array<we_weTag_attribute> $requiredAttributes
	 */
	function __construct(string $name, $value = false, string $module = '', array $allowedAttributes = [], array $requiredAttributes = [], bool $disabled = false){
		$this->Name = $name;
		$this->Value = ($value === false) ? $name : $value;
		$this->Disabled = $disabled;

		// clean allowed and required attributes in case not all modules are installed
		$this->AllowedAttributes = $allowedAttributes;
		$this->RequiredAttributes = $requiredAttributes;
		$this->Module = $module;
	}

	/**
	 * @return array
	 */
	function getAssoziation(): array{
		return [$this->Value => $this->Name];
	}

	/**
	 * @return string
	 */
	function getName(): string{
		return $this->Name;
	}

	/**
	 * @return string[]
	 */
	function getAllowedAttributes(): array{
		$arr = [];
		foreach($this->AllowedAttributes as $attribute){
			if(empty($attribute)){
				continue;
			}
			if(!is_object($attribute)){
				t_e($attribute);
				continue;
			}
			if($attribute->useAttribute()){
				$arr[] = $attribute->getIdName();
			}
		}
		return $arr;
	}

	/**
	 * @return string[]
	 */
	function getRequiredAttributes(): array{
		$arr = [];
		foreach($this->RequiredAttributes as $attribute){
			if($attribute->useAttribute()){
				$arr[] = $attribute->getIdName();
			}
		}
		return $arr;
	}

	function addTypeAttribute(we_weTag_attribute $attr): void{
		array_unshift($this->AllowedAttributes, $attr);
	}

	/**
	 * checks if this attribute should be used, checks if needed modules are installed
	 * @return boolean
	 */
	function useOption(): bool{
		return ($this->Module === '' || we_modules_info::isActive($this->Module));
	}

}
