<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 879c3514b5e5 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-02-28 17:16 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_workflow_frames extends we_modules_frame{

	const TAB_PROPERTIES = 0;
	const TAB_OVERVIEW = 1;

	/** @var we_workflow_view */
	protected $View;

	/** @var we_workflow_tree */
	protected $Tree;

	/** @var int */
	protected $mode = 0;

	public function __construct(){
		$this->module = we_modules_info::WORKFLOW;
		parent::__construct();
		$this->Tree = new we_workflow_tree($this->jsCmd);
		$this->View = new we_workflow_view();
	}

	/**
	 *
	 * @param string $what
	 * @param int $mode
	 * @param int $step
	 * @param string $doctable
	 * @return string
	 */
	public function getHTML(string $what = '', int $mode = 0, int $step = 0, string $doctable = ''): string{
		$this->mode = $mode;
		switch($what){
			case self::FOOTER:
				return $this->getHTMLEditorFooter();
			case 'qlog':
				return $this->getHTMLLogQuestion();
			case 'log':
				return $this->getHTMLLog($mode, $doctable);
			case 'edit':
				return $this->getHTMLEditorBody();
			case 'frameset':
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode() . $this->View->getJSTop());
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLEditorHeader(): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorHeader();
		}

		$page = we_base_request::int('page', 0);
		$text = we_base_request::_(we_base_request::RAW, 'txt', g_l('modules_workflow', '[new_workflow]'));

		$we_tabs = new we_gui_tabs();

		if($this->mode == 0){
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, (string) (self::TAB_PROPERTIES), ['id' => 'tab_' . self::TAB_PROPERTIES, 'title' => g_l('tabs', '[module][properties]')]);
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_EDIT, false, (string) (self::TAB_OVERVIEW), ['id' => 'tab_' . self::TAB_OVERVIEW, 'title' => g_l('modules_workflow', '[menu][workflow]')]);
		} else {
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_INFO, true, (string) (self::TAB_PROPERTIES), ['id' => 'tab_' . self::TAB_PROPERTIES, 'title' => g_l('tabs', '[editor][information]')]);
		}

		$textPre = g_l('modules_workflow', ($this->mode == 1 ? '[document]' : '[workflow]'));
		$textPost = '/' . $text;

		$extraHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
				we_html_element::jsScript(JS_DIR . 'initTabs.js') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/workflow_top.js');

		$body = we_html_element::body(['onresize' => "weTabs.setFrameSize();",
					'onload' => "weTabs.setFrameSize();document.getElementById('tab_" . $page . "').className='tabActive';",
					'id' => 'eHeaderBody',
						], we_html_element::div(
								['id' => 'main'], we_html_element::div(
										['id' => 'headrow'], we_html_element::b(htmlspecialchars($textPre) . ':&nbsp;') .
										we_html_element::span(['id' => 'h_path', 'class' => 'header_small'], '<b id="titlePath">' . htmlspecialchars($textPost) . '</b>')
								) .
								$we_tabs->getHTML()
						)
		);

		return $this->getHTMLDocument($body, $extraHead);
	}

	protected function getHTMLEditorFooter(array $buttons = []): string{
		if(we_base_request::bool('home')){
			return parent::getHTMLEditorFooter($buttons);
		}

		$body = we_html_element::body([
					'id' => 'footerBody',
					'onload' => ($this->mode == 0 ? 'setStatusCheck()' : '')
						], we_html_element::form([], we_html_element::div(['class' => 'footertable'],
										we_html_element::div(['class' => 'main'],
												we_html_button::create_button(we_html_button::SAVE, "javascript:top.content.we_cmd('save_workflow');") .
												$this->View->getStatusHTML())
								)
						)
		);

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/workflow_frames.js'));
	}

	private function getHTMLLog(int $docID, string $doctable): string{
		return $this->getHTMLDocument(
						we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], we_workflow_view::getLogForDocument($docID, $doctable))
		);
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) < 0){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::int('offset', 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_workflow_tree::getItems($pid, $offset, $this->Tree->default_segment), 'sorted' => true]);

		return $this->getHTMLDocument(
						we_html_element::body([], we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hiddens([
											'wcmd' => '',
											'wopt' => ''])
								)
						), we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/workflow_frames.js')
		);
	}

	private function getHTMLLogQuestion(): string{
		$form = we_html_element::form(['name' => we_base_constants::WE_FORM], $this->View->getLogQuestion());
		$body = we_html_element::body(['onload' => "self.focus();"], $form);

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/workflow_frames.js'));
	}

	protected function getHTMLEditorBody(): string{
		if(we_base_request::bool('home')){
			return $this->View->getHomeScreen($this->jsCmd);
		}

		return $this->getHTMLDocument($this->View->getProperties($this->jsCmd), $this->View->getJSProperty());
	}

}
