<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e34127dc4f4b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-02-17 00:15 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * General Definition of WebEdition Workflow
 *
 */
class we_workflow_workflow extends we_workflow_base{
// Document-Type/Category based Workflow Type
	const DOCTYPE_CATEGORY = 'doctype';
// Directory based Workflow Type
	const FOLDER = 'folder';
// class+category based Workflow Type
	const OBJECT = 'object';
	//folder based
	const OBJECT_FOLDER = 'objectFolder';

	//properties
	/** @var int */
	var $ID = 0;

	/** @var string */
	var $Text;

	/** @var array */
	var $Type = [self::FOLDER];

	/** @var int[] */
	var $Folders = [0];

	/** @var int[] */
	var $DocType = [];

	/** @var int[] */
	var $Objects = [];

	/** @var int[] */
	var $Categories = [];

	/** @var int[] */
	var $ObjectFileFolders = [0];

	/** @var int[] */
	var $ObjCategories = [];

	/** @var bool */
	var $active = true;

	/** @var bool */
	var $EmailPath = false;

	/** @var bool */
	var $LastStepAutoPublish = false;

	/*	 * @var we_workflow_step[] steps for WorkFlow Definition 	 */
	var $steps = [];

	/** @var we_workflow_document default document object */
	var $documentDef;

	/** @var array documents array; format document[documentID]=document_name don't create array of objects 'cos want to save some memory */
	var $documents = [];

	/**
	 * Default Constructor
	 * Can load or create new Workflow Definition depends of parameter
	 */
	function __construct(int $workflowID = 0){
		parent::__construct(WORKFLOW_TABLE);

		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'Text' => we_base_request::STRING,
			'Type' => we_base_request::STRING_LIST, //FIXME this is confusing: this is an array of strings
			'DocType' => we_base_request::INTLISTA,
			'Folders' => we_base_request::INTLISTA,
			'ObjectFileFolders' => we_base_request::INTLISTA,
			'Objects' => we_base_request::INTLISTA,
			'Categories' => we_base_request::INTLISTA,
			'ObjCategories' => we_base_request::INTLISTA,
			'active' => we_base_request::BOOL,
			'EmailPath' => we_base_request::BOOL,
			'LastStepAutoPublish' => we_base_request::BOOL,
		];

		$this->Text = g_l('modules_workflow', '[new_workflow]');

		$this->AddNewStep();
		$this->AddNewTask();

		if($workflowID){
			$this->load($workflowID);
		}
	}

	/**
	 * Load workflow definition from database
	 */
	function load(int $id = 0, bool $loadData = true): bool{
		parent::load($id);
		if(!$this->ID){
			return false;
		}

		// get steps for workflow
		$this->steps = we_workflow_step::getAllSteps($this->ID);
		$this->loadDocuments();
		return true;
	}

	/**
	 * get all documents for workflow from database
	 */
	function loadDocuments(): void{
		$this->documents = $this->db->getAllq('SELECT
	w.ID,
	COALESCE(f.Text,of.Text) AS Text,
	COALESCE(f.ContentType,of.ContentType) AS contenttype
FROM ' .
			WORKFLOW_DOC_TABLE . ' w LEFT JOIN ' .
			FILE_TABLE . ' f ON (w.DocumentTable="tblFile" AND w.documentID=f.ID) LEFT JOIN ' .
			OBJECT_FILES_TABLE . ' of ON (w.DocumentTable="tblObjectFiles" AND w.documentID=of.ID)
WHERE
	w.workflowID=' . intval($this->ID) . ' AND
	w.Status="active"');
	}

	/**
	 * get all workflows from database (STATIC)
	 */
	function getAllWorkflows(): array{
		return $this->db->getAllq('SELECT ID FROM ' . WORKFLOW_TABLE . ' ORDER BY Text', true);
	}

	/**
	 * get all workflows from database
	 * @param int[] $all
	 * @return array<int,string>
	 */
	public static function getAllWorkflowsInfo(bool $active = true, array $types = [self::DOCTYPE_CATEGORY], array $all = []): array{
		//@phan-suppress-next-line PhanPartialTypeMismatchReturn
		return DB_WE::$inst->getAllFirstq('SELECT ID,Text FROM ' . WORKFLOW_TABLE . ' wf WHERE active=' . ((int) $active) . ' AND CONCAT(",",wf.Type,",") REGEXP \',(' . implode('|', $types) . '),\'' . ($all ? ' AND ID IN(' . implode(', ', $all) . ')' : '') . ' ORDER BY Text', false);
	}

	/**
	 * save complete workflow definition in database
	 * saving also all steps and tasks for current workflow
	 */
	function save(bool $force_new = false, bool $rebuild = false, array $skipWrite = []): bool{
		if(!parent::save()){
			return false;
		}
		// save all steps also

		$stepsList = [];

		foreach($this->steps as &$step){
			$step->workflowID = $this->ID;
			$step->save();

			$stepsList[] = $step->ID;
		}


		// !!! here we have to delete all other steps in database except this in array
		if($stepsList){
			$this->db->query('DELETE FROM ' . WORKFLOW_STEP_TABLE . ' WHERE workflowID = ' . intval($this->ID) . ' AND ID NOT IN (' . implode(', ', $stepsList) . ')');
		}

		//remove all documents from workflow
		foreach($this->documents as $val){
			$this->documentDef = new we_workflow_document($val['ID']);
			$this->documentDef->finishWorkflow(true);
			$this->documentDef->save();
		}
		return true;
	}

	/**
	 * delete workflow from database
	 * delete also all steps and tasks for current workflow
	 */
	function delete(): bool{
		if(!$this->ID){
			return false;
		}

		foreach(array_keys($this->steps) as $key){
			$this->steps[$key]->delete();
		}

		foreach($this->documents as $val){
			$this->documentDef = new we_workflow_document($val['ID']);
			$this->documentDef->delete();
		}

		parent::delete();

		return true;

		//$this->ID = -2; # status deleted
	}

	static function isDocInWorkflow(int $docID): bool{
		return fB('SELECT 1 FROM ' . WORKFLOW_DOC_TABLE . ' wf WHERE active=1 AND documentID=' . $docID . ' AND CONCAT(",",wf.Type,",") REGEXP \',(' . implode('|', [self::DOCTYPE_CATEGORY,
				self::FOLDER]) . '),\' LIMIT 1');
	}

	static function isObjectInWorkflow(int $docID): bool{
		return fB('SELECT 1 FROM ' . WORKFLOW_DOC_TABLE . ' wf WHERE active=1 AND documentID=' . $docID . ' AND CONCAT(",",wf.Type,",") REGEXP \',(' . implode('|', [self::OBJECT,
				self::OBJECT_FOLDER]) . '),\' LIMIT 1');
	}

	/**
	 * Get workflow for document
	 * @param int $doctype
	 * @param int[] $categories
	 * @param int $folder
	 * @param int[] $all
	 * @return int
	 */
	static function getDocumentWorkflow(int $doctype, array $categories, int $folder, array &$all): int{
		/**
		 * find by document type (has to be together with category)
		 */
		$all = ($doctype ?
			DB_WE::$inst->getAllq('SELECT ID FROM ' . WORKFLOW_TABLE . ' WHERE FIND_IN_SET(' . $doctype . ', DocType) AND FIND_IN_SET("' . self::DOCTYPE_CATEGORY . '",Type) AND active=1', true) :
			[]);
		/**
		 * find by category
		 */
		if($categories && we_modules_info::isActive(we_modules_info::CATEGORY)){
			$all = array_merge($all, DB_WE::$inst->getAllq('SELECT
	ID
FROM ' .
					WORKFLOW_TABLE . ' w JOIN ' .
					CATEGORY_TABLE . ' c ON FIND_IN_SET(c.ID, w.Categories)
WHERE
	w.active=1 AND
	' . ($doctype ? 'w.DocType=' . $doctype . ' AND ' : '') . '
	c.ID IN (' . implode(',', $categories) . ') AND
	FIND_IN_SET("' . self::DOCTYPE_CATEGORY . '",w.Type)
ORDER BY LENGTH(of.Path)
', true));
		}
		'@phan-var int[] $all';
		return ($all ? (int) $all[0] : // when we have found a document type-based workflow we can return
			(self::findWfIdForFolder($folder, $all) ?: 0));
	}

	private static function findWfIdForFolder(int $folderID, array &$all): int{
		//FIXME: $all should be checked if present
		$folders = [$folderID];
		while($folderID != 0){
			$folderID = fI('SELECT ParentID FROM ' . FILE_TABLE . ' WHERE ID=' . $folderID);
			$folders[] = $folderID;
		}
		$all = array_merge($all, ($tmp = DB_WE::$inst->getAllq('SELECT DISTINCT(w.ID)
FROM ' .
			WORKFLOW_TABLE . ' w JOIN ' .
			FILE_TABLE . ' f ON FIND_IN_SET(f.ID, w.Folders)
WHERE
	w.active=1 AND
	f.ID IN(' . implode(',', $folders) . ') AND
	FIND_IN_SET("' . self::FOLDER . '",w.Type)
ORDER BY LENGTH(f.Path)
	', true)));
		'@phan-var int[] $all';

		return $tmp ? (int) $tmp[0] : 0;
	}

	/**
	 * Get workflow for object
	 * @param int $object
	 * @param int[] $categories
	 * @param int $folderID
	 * @param int[] $all
	 */
	public static function getObjectWorkflow(int $object, array $categories, int $folderID, array &$all): int{
		$folders = [$folderID];
		while($folderID != 0){
			$folderID = fI('SELECT ParentID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $folderID);
			if($folderID){
				$folders[] = $folderID;
			}
		}

		$all = DB_WE::$inst->getAllq('SELECT DISTINCT(w.ID)
FROM ' .
			WORKFLOW_TABLE . ' w JOIN ' .
			OBJECT_FILES_TABLE . ' of ON FIND_IN_SET(of.ID, w.ObjectFileFolders)
WHERE
	w.active=1 AND
	of.ID IN(' . implode(',', $folders) . ') AND
	FIND_IN_SET("' . self::OBJECT_FOLDER . '",w.Type)
ORDER BY LENGTH(of.Path)
', true);

		if($categories && we_modules_info::isActive(we_modules_info::CATEGORY)){
			$all = array_merge($all, DB_WE::$inst->getAllq('SELECT ID
FROM ' .
					WORKFLOW_TABLE . ' w JOIN
		' . CATEGORY_TABLE . ' c ON FIND_IN_SET(c.ID,w.ObjCategories)
WHERE
	w.active=1 AND
	c.ID IN (' . implode(',', $categories) . ') AND
	FIND_IN_SET(' . $object . ', w.Objects) AND
	FIND_IN_SET("' . self::OBJECT . '",w.Type)
ORDER BY LENGTH(c.Path) DESC
', true));
		}
		'@phan-var int[] $all';

		return ($all ? $all[0] : 0);
	}

	function addNewStep(): void{
		$this->steps[] = new we_workflow_step();
	}

	function addNewTask(): void{
		foreach($this->steps as &$v){
			$v->tasks[] = new we_workflow_task();
		}
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'view' => [
					'delete_question' => g_l('modules_workflow', '[delete_question]'),
					'emty_log_question' => g_l('modules_workflow', '[emty_log_question]'),
					'nothing_to_delete' => (g_l('modules_workflow', '[nothing_to_delete]')),
					'nothing_to_save' => (g_l('modules_workflow', '[nothing_to_save]')),
					'save_changed_workflow' => g_l('modules_workflow', '[save_changed_workflow]'),
					'save_question' => g_l('modules_workflow', '[save_question]'),
				],
				'prop' => [
					'del_last_step' => (g_l('modules_workflow', '[del_last_step]')),
					'del_last_task' => (g_l('modules_workflow', '[del_last_task]')),
					'doctype_empty' => (g_l('modules_workflow', '[doctype_empty]')),
					'folders_empty' => (g_l('modules_workflow', '[folders_empty]')),
					'name_empty' => (g_l('modules_workflow', '[name_empty]')),
					'objects_empty' => (g_l('modules_workflow', '[objects_empty]')),
					'user_empty' => (g_l('modules_workflow', '[user_empty]')),
					'worktime_empty' => (g_l('modules_workflow', '[worktime_empty]')),
				]
		]);
	}

}
