/* global WE, top */

/**
 * webEdition CMS
 *
 * $Rev: 34c169e93d2d $
 * $Author: Lukas Imhof $
 * $Date: 2021-09-23 09:52 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
var deleteData = WE().util.getDynamicVar(document, 'loadVarDelete', 'data-deleteData');

function we_submitForm(target, url) {
	var f = window.document.we_form,
		selected = [],
		elem;
	if (!f.checkValidity()) {
		WE().util.showMessage(WE().consts.g_l.main.save_error_fields_value_not_valid, WE().consts.message.WE_MESSAGE_ERROR, window);

		return false;
	}

	for (elem of top.treeData.elements) {
		if (elem && elem.checked) {
			selected.push(elem.id);
		}
	}

	if (!selected.length) {
		WE().util.showMessage(WE().consts.g_l.main.nothing_to_delete, WE().consts.message.WE_MESSAGE_ERROR, window);
		return;
	}

	f.sel.value = selected.join(',');
	f.target = target;
	f.action = url;
	f.method = "post";
	f.submit();
	return true;
}

function confirmDel(found) {
	if (found) {
		WE().util.showConfirm(window, "", WE().consts.g_l.alert.found_in_workflow, [deleteData.wecmd0, "", deleteData.table, 1]);
	} else {
		we_cmd(deleteData.wecmd0, "", deleteData.table, 1);
	}
}

function initDeleteMultiple() {
	switch (deleteData.table) {
		case WE().consts.tables.FILE_TABLE:
			if (WE().util.hasPerm("DELETE_DOC_FOLDER") && WE().util.hasPerm("DELETE_DOCUMENT")) {
				top.treeData.setState(top.treeData.tree_states.select);
			} else if (WE().util.hasPerm("DELETE_DOCUMENT")) {
				top.treeData.setState(top.treeData.tree_states.selectitem);
			}
			break;
		case WE().consts.tables.TEMPLATES_TABLE:
			if (WE().util.hasPerm("DELETE_TEMP_FOLDER") && WE().util.hasPerm("DELETE_TEMPLATE")) {
				top.treeData.setState(top.treeData.tree_states.select);
			} else if (WE().util.hasPerm("DELETE_TEMPLATE")) {
				top.treeData.setState(top.treeData.tree_states.selectitem);
			}
			break;
		case WE().consts.tables.OBJECT_FILES_TABLE:
			if (WE().util.hasPerm("DELETE_OBJECTFILE")) {
				top.treeData.setState(top.treeData.tree_states.select);
			}
			break;
		case WE().consts.tables.VFILE_TABLE:
			// FIXME: implement prefs for collections
			//if(WE().util.hasPerm("DELETE_DOC_FOLDER") && WE().util.hasPerm("DELETE_DOCUMENT")){
			top.treeData.setState(top.treeData.tree_states.select);
			/*
			 } elseif(WE().util.hasPerm("DELETE_DOCUMENT")){
			 echo 'top.treeData.setState(top.treeData.tree_states["selectitem"]);';
			 }
			 *
			 */
			break;
		default:
			top.treeData.setState(top.treeData.tree_states.selectitem);
	}
}

function init() {
	if (deleteData.wecmd0 !== "delete_single_document") {
		initDeleteMultiple();
	}

	if (top.treeData.table !== deleteData.table) {
		top.treeData.table = deleteData.table;
		we_cmd("load", deleteData.table);
	} else {
		top.drawTree();
	}
}