/* global WE */

/**
 * webEdition CMS
 *
 * $Rev: 6b362ca58f07 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-02-14 23:00 +0100 $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';

WE().util.loadConsts(document, "g_l.metadatafields");
var phpdata = WE().util.getDynamicVar(document, 'loadVarEdit_metadatafields', 'data-phpdata');

function togglePropositionTable(sel, index) {
	var row = document.getElementById("proposalTable_" + index);
	row.style.display = sel.value === "none" ? "none" : "block";

	var fields = row.getElementsByTagName("INPUT");
	for (var i = 0; i < fields.length; i++) {
		fields[i].disabled = sel.value === "auto";
	}
}

function toggleType(sel, index) {
	var row = document.getElementById("proposalTable_" + index),
		selMode = document.forms[0].elements["metadataMode[" + index + "]"],
		checksDiv0 = document.getElementById('metadataProposalChecks0_' + index),
		checksDiv1 = document.getElementById('metadataProposalChecks1_' + index),
		modeDiv0 = document.getElementById('metadataModeDiv0_' + index),
		modeDiv1 = document.getElementById('metadataModeDiv1_' + index);

	row.style.display = (sel.value !== "textfield" ? "none" : (selMode.options[selMode.selectedIndex].value === "none" ? "none" : "block"));
	//selMode.disabled = sel.value === "textfield" ? false : true;
	checksDiv0.style.display = checksDiv1.style.display = modeDiv0.style.display = modeDiv1.style.display = sel.value === "textfield" ? 'block' : 'none';
}


function delRow(id) {
	var el = document.getElementById("row_" + id);
	if (el) {
		el.parentNode.removeChild(el);
	}
}

function addProposition(btn, index) {
	var
		plusRow = btn.parentNode.parentNode,
		newProp = getPropositionRow(index, (plusRow.parentNode.rows.length - 1));
	plusRow.parentNode.insertBefore(newProp, plusRow);
}

function delProposition(btn) {
	var prop = btn.parentNode.parentNode;
	prop.parentNode.removeChild(prop);
}

function addFieldToInput(sel, inpNr) {
	if (sel && sel.selectedIndex >= 0 && sel.options[sel.selectedIndex].parentNode.nodeName.toLowerCase() === "optgroup") {
		var
			_inpElem = document.forms[0].elements["metadataImportFrom[" + inpNr + "]"],
			_metaType = sel.options[sel.selectedIndex].parentNode.label.toLowerCase(),
			_str = _metaType + "/" + sel.options[sel.selectedIndex].value;
		_inpElem.value = _inpElem.value ? _inpElem.value + ("," + _str) : _str;
	}
	sel.selectedIndex = 0;
}

function addRow() {
	var
		container = document.getElementById("metadataTable"),
		newID = (container.rows.length),
		elem, elemTR, elemTD, cell, newRow, nestedTable, nestedRow, nestedCell;
	if (!container) {
		return;
	}
	elem = document.createElement("TABLE");
	elem.style.backgroundColor = '#f5f5f5';//margin-bottom:10px
	elem.style.marginBottom = '10px';
	elem.setAttribute("id", "elem_" + newID);

	newRow = document.createElement("TR");
	newRow.setAttribute("id", "metadataRow0_" + newID);
	cell = document.createElement("TD");
	cell.innerHTML = "<strong>" + WE().consts.g_l.metadatafields.tagname + "</strong>";
	cell.width = "210";
	cell.style.paddingTop = "12px";
	newRow.appendChild(cell);
	cell = document.createElement("TD");
	cell.innerHTML = "<strong>" + WE().consts.g_l.metadatafields.type + "</strong>";
	cell.width = "110";
	cell.style.paddingTop = "12px";
	cell.colspan = "2";
	newRow.appendChild(cell);
	elem.appendChild(newRow);

	newRow = document.createElement("TR");
	newRow.setAttribute("id", "metadataRow1_" + newID);
	cell = document.createElement("TD");
	cell.innerHTML = phpdata.tagInp.replace(/__we_new_id__/g, newID);
	cell.width = "210";
	newRow.appendChild(cell);
	cell = document.createElement("TD");
	cell.innerHTML = phpdata.typeSel.replace(/__we_new_id__/g, newID);
	cell.width = "200";
	newRow.appendChild(cell);
	cell = document.createElement("TD");
	cell.width = "30";
	cell.align = "right";
	cell.innerHTML = phpdata.trashButton.replace(/__we_new_id__/, newID);
	newRow.appendChild(cell);
	elem.appendChild(newRow);

	newRow = document.createElement("TR");
	newRow.setAttribute("id", "metadataRow2_" + newID);
	cell = document.createElement("TD");
	cell.style.paddingBottom = "6px";
	cell.innerHTML = '<div class="small">' + WE().consts.g_l.metadatafields.import_from + '</div>' + phpdata.importInp.replace(/__we_new_id__/, newID);
	newRow.appendChild(cell);
	cell = document.createElement("TD");
	cell.setAttribute("colspan", 2);
	cell.style.paddingBottom = "6px";
	cell.innerHTML = '<div class="small">' + WE().consts.g_l.metadatafields.fields + '</div>' + phpdata.fieldSel.replace(/__we_new_id__/g, newID);
	newRow.appendChild(cell);
	elem.appendChild(newRow);

	newRow = document.createElement("TR");
	newRow.setAttribute("id", "metadataRow3_" + newID);
	cell = document.createElement("TD");
	cell.style.paddingBottom = "1px";
	cell.innerHTML = '<div class="small" id="metadataModeDiv0_' + newID + '">' + WE().consts.g_l.metadatafields.proposals + '</div><div id="metadataModeDiv1_' + newID + '">' + phpdata.modeSel.replace(/__we_new_id__/g, newID) + '</div>';
	newRow.appendChild(cell);
	cell = document.createElement("TD");
	cell.setAttribute("colspan", 2);
	cell.innerHTML = '<div class="small" id="metadataProposalChecks0_' + newID + '">&nbsp;</div><div id="metadataProposalChecks1_' + newID + '">' + phpdata.csvCheck.replace(/__we_new_id__/g, newID) + phpdata.closedCheck.replace(/__we_new_id__/g, newID) + '</div>';
	newRow.appendChild(cell);
	elem.appendChild(newRow);

	newRow = document.createElement("TR");
	newRow.setAttribute("id", "metadataRow4_" + newID);
	cell = document.createElement("TD");
	cell.colSpan = "3";
	cell.style.paddingBottom = "16px";
	cell.paddingRight = "5px";
	nestedTable = document.createElement("TABLE");
	nestedTable.setAttribute("id", "proposalTable_" + newID);
	nestedTable.style.width = "100%";
	nestedTable.style.display = "none";
	//nestedTable.style.backgroundColor = "white";
	nestedTable.style.border = "1px solid gray";
	nestedTable.style.paddingTop = "8px";
	nestedTable.appendChild(getPropositionRow(newID, 0));
	nestedRow = document.createElement("TR");
	nestedCell = document.createElement("TD");
	nestedCell.width = "15%";
	nestedRow.appendChild(nestedCell);
	nestedCell = document.createElement("TD");
	nestedCell.innerHTML = phpdata.addPropositionBtn.replace(/__we_new_id__/, newID);
	nestedRow.appendChild(nestedCell);
	nestedCell = document.createElement("TD");
	nestedCell.width = "25";
	nestedRow.appendChild(nestedCell);
	nestedTable.appendChild(nestedRow);
	cell.appendChild(nestedTable);
	newRow.appendChild(cell);
	elem.appendChild(newRow);

	elemTR = document.createElement("TR");
	elemTR.setAttribute("id", "row_" + newID);
	elemTD = document.createElement("TD");
	elemTD.appendChild(elem);
	elemTR.appendChild(elemTD);
	container.appendChild(elemTR);
}

function getPropositionRow(indexMeta, indexProp) {
	var
		row = document.createElement("TR"),
		cell = document.createElement("TD");
	cell.width = "15%";
	row.appendChild(cell);

	cell = document.createElement("TD");
	cell.innerHTML = phpdata.proposalInp.replace(/__we_meta_id__/, indexMeta).replace(/__we_prop_id__/, indexProp);
	row.appendChild(cell);

	cell = document.createElement("TD");
	cell.width = "25";
	cell.innerHTML = phpdata.delPropositionBtn;
	row.appendChild(cell);

	return row;
}

function closeOnEscape() {
	return true;
}

function saveOnKeyBoard() {
	we_save();
	return true;

}

function we_save() {
	var _doc = document;
	var inputs = _doc.getElementsByTagName('INPUT');
	for (var i = 0; i < inputs.length; i++) {
		inputs[i].disabled = false;
	}

	var _z = 0;
	var _field = _doc.forms[0].elements['metadataTag[' + _z + ']'] !== undefined ? _doc.forms[0].elements['metadataTag[' + _z + ']'] : null;

	while (_field !== null) {
		if (!checkMetaFieldName(_field, _z)) {
			return;
		}
		_z++;
		_field = _doc.forms[0].elements['metadataTag[' + _z + ']'] !== undefined ? _doc.forms[0].elements['metadataTag[' + _z + ']'] : null;
	}

	_doc.getElementById('metadatafields_dialog').style.display = 'none';

	_doc.getElementById('metadatafields_save').style.display = '';

	_doc.we_form.save_metadatafields.value = 'true';
	_doc.we_form.submit();
}

function checkMetaFieldName(inpElem, nr) {
	var _val = inpElem.value;
	var _forbiddenNames = ",data,width,height,border,align,hspace,vspace,alt,name,title,longdescid,useMetaTitle,scale,play,autoplay,quality,attrib,salign,loop,controller,volume,hidden,";
	var _errtxt = "";
	if (_val === "") {
		_errtxt = WE().consts.g_l.metadatafields.error_meta_field_empty_msg;
		_errtxt = _errtxt.replace(/%s1/, nr + 1);
	} else if (_val.search(/[^a-zA-z0-9_]/) !== -1) {
		_errtxt = WE().consts.g_l.metadatafields.meta_field_wrong_chars_messsage;
		_errtxt = _errtxt.replace(/%s1/, _val);
	} else if (_forbiddenNames.indexOf("," + _val + ",") >= 0) {
		_errtxt = WE().consts.g_l.metadatafields.meta_field_wrong_name_messsage;
		_errtxt = _errtxt.replace(/%s1/, _val);
		_errtxt = _errtxt.replace(/%s2/, "\\n" + _forbiddenNames.substring(1, _forbiddenNames.length - 1).replace(/,/g, ", "));
	}


	if (_errtxt !== "") {
		inpElem.focus();
		inpElem.select();
		WE().util.showMessage(_errtxt, 4, WE().util.dialogTop(window));
		return false;
	}
	return true;
}
