/* global WE, top */

/**
 webEdition CMS
 *
 * $Rev: 0ea8c0c04490 $
 * $Author: lukasimhof $
 * $Date: 2023-01-16 12:20 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function Fileupload_sender_abstract(uploader) {
	this.uploader = uploader;

	this.doCommitFile = true;
	this.chunkSize = 256 * 1024;
	this.responseClass = 'we_fileupload_ui_base';
	this.typeCondition = [];
	this.patternFile = '';
	this.patternExt = '';
	this.charsFileForbidden = '';
	this.charsPathForbidden = '';
	this.regexFile = null;
	this.regexExt = null;
	this.regexFileForbidden = null;
	this.regexPathForbidden = null;
	this.extToMime = {};
	this.maxUploadSize = 0;
	this.rpcUrl = '';
	this.callback = function () {
		this.uploader.doc.forms[0].submit();
	};
	this.cmdUploadFinished = '';
	this.replaceCmdUploadFinished = '';
	this.dialogCallback = null;
	this.isCancelled = false;
	this.preparedFiles = [];
	this.uploadFiles = [];
	this.currentFile = -1;
	this.totalFiles = 0;
	this.totalWeight = 0;
	this.currentWeight = 0;
	this.currentWeightTag = 0;//FIXME: find better name
	this.isAutostartPermitted = false;
	this.partProperties = null;
	this.retry = 0;

	this.currentResponse = null;
	this.importedIDs = [];
	this.importedDocs = [];
	this.collectedMessages = {
		success: [],
		failure: [],
		rejected: []
	};

	this.init = function (conf) {
		this.controller = this.uploader.controller; // on init all components are initialized
		this.view = this.uploader.view;
		this.imageEdit = this.uploader.imageEdit;
		this.utils = this.uploader.utils;
		this.typeCondition = conf.typeCondition ? conf.typeCondition : this.typeCondition;
		this.patternFile = conf.patternFile ? conf.patternFile : this.patternFile;
		this.patternExt = conf.patternExt ? conf.patternExt : this.patternExt;
		this.charsFileForbidden = conf.charsFileForbidden ? conf.charsFileForbidden : this.charsFileForbidden;
		this.charsPathForbidden = conf.charsPathForbidden ? conf.charsPathForbidden : this.charsPathForbidden;
		this.regexFile = new RegExp('^' + this.patternFile + '$');
		this.regexExt = new RegExp('^' + this.patternExt + '$');
		this.regexFileForbidden = new RegExp(this.charsFileForbidden, 'g');
		this.regexPathForbidden = new RegExp(this.charsPathForbidden, 'g');
		this.doCommitFile = conf.doCommitFile !== undefined ? conf.doCommitFile : this.doCommitFile;
		this.chunkSize = typeof conf.chunkSize !== 'undefined' ? (conf.chunkSize * 1024) : this.chunkSize;
		this.callback = conf.callback ? conf.callback : this.callback;
		this.cmdUploadFinished = conf.cmdUploadFinished ? conf.cmdUploadFinished : this.cmdUploadFinished;
		this.responseClass = conf.responseClass ? conf.responseClass : this.responseClass;
		this.dialogCallback = conf.dialogCallback ? conf.dialogCallback : this.dialogCallback;
		this.maxUploadSize = typeof conf.maxUploadSize !== 'undefined' ? conf.maxUploadSize : this.maxUploadSize;
		this.rpcUrl = conf.rpcUrl ? conf.rpcUrl : this.rpcUrl;
	};

	this.onload = function () {
		this.onload_sub();
	};

	this.onload_sub = function () {
		// to be overridden
	};

	this.resetSender = function () {
		this.preparedFiles = [];
		this.imageEdit.memorymanagerReregisterAll();
		this.uploadFiles = [];
		this.currentFile = -1;
		this.isCancelled = false;
		this.totalFiles = this.totalWeight = this.currentWeight = this.currentWeightTag = 0;
		this.view.setExternalProgress(0);
		this.currentResponse = null;
		this.importedIDs = [];
		this.importedDocs = [];
		this.collectedMessages = {
			success: [],
			failure: [],
			rejected: []
		};
		// check for more props to reset!
	};

	this.prepareUpload = function () { // TODO: make this generic for use in importer too
		if (this.preparedFiles.length && typeof this.preparedFiles[0] === 'object' && this.preparedFiles[0] !== null && this.preparedFiles[0].getIsUploadable()) {
			this.preparedFiles[0].fileNum = 0;
			this.preparedFiles[0].preparedFilesIndex = 0;
			this.uploadFiles = [this.preparedFiles[0]];
			this.totalWeight = this.preparedFiles[0].size;
			this.totalFiles = 1;
			this.currentWeight = 0;
			this.totalChunks = Math.ceil(this.totalWeight / this.chunkSize);
			this.currentWeight = 0;
			this.currentWeightTag = 0;

			this.controller.processEvents(this.controller.ON.UPLOAD_START);

			return true;
		}

		return false;
	};

	this.getIsUploadableFiles = function () {
		for (var i = 0; i < this.preparedFiles.length; i++) {
			if (this.preparedFiles[i] && this.preparedFiles[i].getIsUploadable()) {
				return true;
			}
		}

		return false;
	};

	this.sendNextFile = function () {
		var cur = null;

		if (this.uploadFiles.length > 0) {
			this.currentFile = cur = this.uploadFiles.shift();

			if (this.view.elems.fileSelect && this.view.elems.fileSelect.value) {
				this.view.elems.fileSelect.value = '';
			}
			this.uploader.uploaderState = this.controller.STATE.UPLOADING;
			this.view.setShowElemProcess(cur, false);

			if (cur.isEditable) {
				this.prepareImageAndSend(cur);
				return;
			}

			if (!cur.getIsUploadable()) {
				this.checkRecountChunks(cur);
				this.sendNextFile();
				return;
			}

			// file is not yet read in any way so we read it from original file
			this.loadDataArrayAndSend(cur);
			return;
		}

		// no uploads left: start postprocess
		if (this.uploader.getUploaderState === this.controller.STATE.UPLOADING) {
			this.uploader.uploaderState = this.controller.STATE.FINISHED;
		} else {
			this.uploader.uploaderState = this.controller.STATE.NOTREADY;
		}

		this.postProcess();
		this.currentFile = null;
	};

	this.loadDataArrayAndSend = function (fileobj, e) {
		if (!e) {
			var reader = new FileReader();
			reader.onload = this.loadDataArrayAndSend.bind(this, fileobj);
			reader.readAsArrayBuffer(fileobj.file);
			return;
		}

		fileobj.dataArray = new Uint8Array(e.target.result);
		fileobj.size = fileobj.dataArray.byteLength;
		this.checkRecountChunks(fileobj);

		fileobj.isUploadStarted = true;
		this.controller.processEvents(this.controller.ON.FILE_SEND, fileobj.index, fileobj);
		this.sendNextChunk();
	};

	this.prepareImageAndSend = function (fileobj) {
		var editOptsLast,
				editOpts;

		editOptsLast = fileobj.img.processedOptions;
		this.imageEdit.setImageEditOptionsFile(fileobj);
		editOpts = fileobj.img.editOptions;
		this.view.setShowElemProcess(fileobj, false);

		// there was no need to load image before: load and send unchanged
		if (!fileobj.dataUrl && !fileobj.dataArray && !editOpts.doEdit &&
				!(this.maxUploadSize > 0 && fileobj.img.originalSize > this.maxUploadSize) &&
				this.imageEdit.MAX_LONGEST === -1) {
			this.loadDataArrayAndSend(fileobj);
			return;
		}

		// image could repetedly not be loaded or edited: upload unedited if not > this.maxUploadSize
		if (fileobj.img.countEditBeforeSend > 1) {
			if (this.maxUploadSize > 0 && fileobj.img.originalSize > this.maxUploadSize) {
				fileobj.isOrigSizeNOK = true;
				this.checkRecountChunks(fileobj);
				var message = WE().consts.g_l.fileupload.errorTooBigEditFailed;
				fileobj.failedRequest = message;
				this.collectedMessages.failure.push('<li>' + fileobj.fileName + ':<br/>' + message + '</li>');
				this.controller.processEvents(this.controller.ON.CHUNK_RESPONSE_NOK, fileobj.index, fileobj);
				this.sendNextFile();
				return;
			}

			// as file could not be loaded we do not know longest size: upload anyway and let php reject if needed
			if (editOpts.doEdit) {
				fileobj.notices.push(WE().consts.g_l.fileupload.errorEditFailed);
			}
			this.loadDataArrayAndSend(fileobj);
			return;
		}

		// reedit image if not yet edited or last used edit options are no longer valid
		if (!fileobj.dataUrl || !fileobj.dataUrl.length ||
				editOpts.doEdit != editOptsLast.doEdit || // IMPORTANT: make config string: "1,w,1280,270,90" to compare!!
				editOpts.scaleWhat != editOptsLast.scaleWhat ||
				editOpts.scale != editOptsLast.scale ||
				editOpts.rotate != editOptsLast.rotate ||
				editOpts.quality != editOptsLast.quality) {
			// image is not yet loaded, edited or edited using options others than actually valid ones!
			fileobj.img.countEditBeforeSend++;
			fileobj.isEdited = false;
			this.preparedFiles[fileobj.preparedFilesIndex].img.callback = 'prepareImageAndSend';
			this.preparedFiles[fileobj.preparedFilesIndex].tmpSize = this.preparedFiles[fileobj.preparedFilesIndex].size;
			this.imageEdit.processSingleImage(this.preparedFiles[fileobj.preparedFilesIndex]);
			return;
		}

		// FIXME: WHY is this done here and not at when editing is done??
		fileobj.dataArray = this.utils.dataURLToUInt8Array(fileobj.dataUrl);
		if (editOpts.doEdit || fileobj.img.landscape.correction) {
			if (fileobj.type === 'image/jpeg' && fileobj.img.jpgCustomSegments && fileobj.img.jpgCustomSegments.length) {
				fileobj.dataArray = this.imageEdit.jpgInsertSegments(fileobj.dataArray, null, null, fileobj.img.jpgCustomSegments, true);
			} else if (fileobj.type === 'image/png' && fileobj.img.pngTextChunks) {
				fileobj.dataArray = this.imageEdit.pngReinsertTextchunks(fileobj.dataArray, fileobj.img.pngTextChunks);
			}
		} else {
			/*
			 var extract = this.imageEdit.jpgGetSegmentsIfExist(fileobj.dataArray, [], true);
			 if(extract.success){
			 fileobj.img.jpgCustomSegments = extract.segments;
			 fileobj.dataArray = this.imageEdit.jpgInsertSegments(fileobj.dataArray, extract.headArray, extract.contentArray, extract.segments, true);
			 }
			 */
		}
		fileobj.dataUrl = null;
		fileobj.size = fileobj.dataArray.byteLength;

		this.checkRecountChunks(fileobj);

		if (!fileobj.getIsUploadable()) {
			this.sendNextFile();
			return;
		}

		fileobj.isUploadStarted = true;
		this.controller.processEvents(this.controller.ON.FILE_SEND, fileobj.index, fileobj);
		this.sendNextChunk();
	};

	this.checkRecountChunks = function (fileobj) {
		if (!fileobj.getIsUploadable()) {
			if (fileobj.lastMeasuredSize) {
				this.totalWeight = this.totalWeight - fileobj.lastMeasuredSize;
				this.totalChunks = Math.ceil(this.totalWeight / this.chunkSize);
			}
			return;
		}

		// we do remeasure last chunk anyway to be SURE it is correct!
		fileobj.totalParts = Math.ceil(fileobj.size / this.chunkSize);
		fileobj.lastChunkSize = fileobj.size % this.chunkSize;
		fileobj.lastChunkSize = fileobj.lastChunkSize ? fileobj.lastChunkSize : this.chunkSize;

		if (fileobj.lastMeasuredSize && parseInt(fileobj.lastMeasuredSize) !== parseInt(fileobj.size)) {
			this.totalWeight = this.totalWeight - fileobj.lastMeasuredSize + fileobj.size;
			this.totalChunks = Math.ceil(this.totalWeight / this.chunkSize);
		}

		fileobj.lastMeasuredSize = parseInt(fileobj.size);
	};

	this.sendNextChunk = function (resent) {//FIXME: why do we need this param???
		var oldPos,
				blob,
				cur = this.currentFile; // when using short syntax in line 1156 we must change some this to that = sender. FIXME: WHY?!

		if (this.isCancelled) {
			this.isCancelled = false;
			return;
		}
		if (resent) {
			this.retry++;
		} else {
			this.retry = 0;
		}

		if (cur.partNum < cur.totalParts || resent) {
			if (resent) {//this may fail on last chunk
				oldPos = (cur.partNum - 1) * this.chunkSize;
			} else {
				oldPos = cur.currentPos;
				cur.currentPos = oldPos + this.chunkSize;
				cur.partNum++;
			}
			blob = new Blob([cur.dataArray.subarray(oldPos, cur.currentPos)]);

			this.sendChunk(
					blob,
					(cur.fileName ? cur.fileName : cur.file.name),
					(cur.mimePHP !== 'none' ? cur.mimePHP : cur.type),
					(cur.partNum === cur.totalParts ? cur.lastChunkSize : this.chunkSize),
					cur.partNum,
					cur.totalParts,
					cur.size,
					cur.uniqueId
					);
		}
	};

	this.sendChunk = function (part, fileName, fileCt, partSize, partNum, totalParts, fileSize, uniqueId) {
		var xhr = new XMLHttpRequest(),
				fd = new FormData(),
				fsize = fileSize || 1,
				form = this.uploader.doc.we_form,
				cur = this.currentFile;

		this.partProperties = {
			partSize: partSize,
			partNum: partNum,
			totalParts: totalParts
		};

		var filePath = '';
		if (fileName.indexOf('/') !== -1) {
			var parts = fileName.split('/');
			fileName = parts.pop();
			filePath = parts.join('/');
		}

		fileCt = fileCt ? fileCt : 'text/plain';
		fd.append('fileinputName', this.uploader.fieldName);
		fd.append('doCommitFile', this.doCommitFile);
		fd.append('genericFilename', this.uploader.genericFilename);
		fd.append('weResponseClass', this.responseClass);
		fd.append('wePartNum', partNum);
		fd.append('wePartCount', totalParts);
		fd.append('wePartSize', partSize);
		fd.append('weRetry', this.retry);
		fd.append('weFileSize', fsize);
		fd.append('weFileName', fileName);
		fd.append('weFilePath', filePath.replace(this.regexPathForbidden, "_"));
		fd.append('weFileCt', fileCt);
		fd.append('weUniqueId', uniqueId);
		fd.append(this.currentFile.field !== undefined ? this.currentFile.field : this.uploader.fieldName, part, fileName);//FIXME: take fieldname allways from cur!

		switch (this.uploader.uiType) {
			case 'editFull':
				break;
			default:
				fd.append('fu_doc_focusX', cur.img.focusX);
				fd.append('fu_doc_focusY', cur.img.focusY);
				fd.append('fu_doc_importMetadata', (form.fu_doc_importMetadata ? form.fu_doc_importMetadata.value : form.fu_doc_importMetadata_ctrl.value));
		}

		fd = this.appendMoreData(fd);
		xhr.onreadystatechange = this.sendChunkAjaxCallback.bind(this);
		//FIXME: use XMLHttpRequestUpload?!
		xhr.timeout = 12000;
		xhr.open('POST', this.rpcUrl, true);
		xhr.send(fd);
	};

	this.sendChunkAjaxCallback = function (e) {
		var xhr = e.target;

		switch (xhr.readyState) {
			case 0://unsent
			case 1://opened
			case 2://headers received
			case 3://loading
				break;
			case 4:
				switch (xhr.status) {
					case 0://e.g. network error
						if (this.retry > 10) {
							WE().t_e('Uploader failed for more than 10 times');
							//do cancel
							return;
						}
						//extend retry interval
						window.setTimeout(this.sendNextChunk.bind(this, true), (this.retry * 3000));
						break;
					case 200:
						var resp = '';

						try {
							resp = JSON.parse(xhr.responseText);
						} catch (exception) {
							WE().t_e('Response unable to parse: ' + xhr.responseText);
							throw exception;
						}

						resp = resp.DataArray && resp.DataArray.data ? resp.DataArray.data : resp;
						this.processResponse(resp, this.partProperties);
						break;
					default:
						WE().t_e("Uploader received bad status from server: " + xhr.status);
				}
				break;
		}
	};

	this.appendMoreData = function (fd) {
		return fd;
		/*
		 parentID
		 sameName
		 */
	};

	this.processResponse = function (resp, args) {
		if (!this.isCancelled) {
			var cur = this.currentFile,
					notices;

			cur.uniqueId = resp.uniqueId;
			cur.mimePHP = resp.mimePhp;
			cur.currentWeightFile += args.partSize;
			this.currentWeight += args.partSize;

			switch (resp.status) {
				case 'continue':
					this.controller.processEvents(this.controller.ON.CHUNK_RESPONSE_OK, cur.index, cur);
					this.sendNextChunk();
					return;
				case 'success':
					this.currentWeightTag = this.currentWeight;
					cur.dataArray = null;
					cur.dataUrl = null;
					if (this.imageEdit.IS_MEMORY_MANAGMENT) {
						this.imageEdit.memorymanagerUnregister(cur);
					}
					this.controller.processEvents(this.controller.ON.CHUNK_RESPONSE_OK, cur.index, cur);
					this.controller.processEvents(this.controller.ON.FILE_RESPONSE_OK, cur.index, cur);
					this.doOnFileFinished(resp);
					notices = cur.notices ? '<br/>' + cur.notices.join('<br/>') : '';
					if (notices || resp.message) {
						this.collectedMessages.success.push('<li>' + resp.message + notices + '</li>');
					}
					if (resp.weDoc) {
						this.importedIDs.push(resp.weDoc.id);
						this.importedDocs.push = resp.weDoc;
					}
					this.currentResponse = resp;
					this.sendNextFile();
					return;
				case 'retransmit'://chunksize did not match at server
					if (this.retry < 10) {
						this.sendNextChunk(true);
						return;
					}
					//falls through
				case 'failure':
					this.currentWeight = this.currentWeightTag + cur.size;
					this.currentWeightTag = this.currentWeight;
					cur.dataArray = null;
					cur.dataUrl = null;
					if (this.imageEdit.IS_MEMORY_MANAGMENT) {
						this.imageEdit.memorymanagerUnregister(cur);
					}
					notices = cur.notices ? cur.notices.join('<br/>') + '<br/>' : '';
					cur.failedRequest = WE().consts.g_l.fileupload[resp.message] ? WE().consts.g_l.fileupload[resp.message] : resp.message;
					this.collectedMessages.failure.push('<li>' + resp.filename + ':<br/>' + notices + cur.failedRequest + '</li>');
					this.controller.processEvents(this.controller.ON.CHUNK_RESPONSE_NOK, cur.index, cur);
					this.currentResponse = resp;
					this.sendNextFile();

					return;
				default :
					return;
			}
		}
	};

	this.doOnFileFinished = function () {
		//to be overridden
	};

	this.postProcess = function () {
		//to be overriden
	};
}

function Fileupload_sender_base(uploader) {
	Fileupload_sender_abstract.call(this, uploader);

	this.uploader = uploader;
	this.totalWeight = 0;

	this.postProcess = function () {
		var resp = this.currentResponse,
				cur = this.currentFile,
				form = this.uploader.doc.forms.we_form;

		form.elements.weUniqueId.value = cur.uniqueId;
		form.elements.weFileCt.value = cur.mimePHP;
		form.elements.weFileName.value = cur.file.name;

		this.view.setStateItem(this.controller.STATE.ITEM_UPLOADED, 0);
		this.view.setStateGUI(this.controller.STATE.FINISHED);
		this.uploader.uploaderState = this.controller.STATE.FINISHED;

		if (this.cmdUploadFinished || this.replaceCmdUploadFinished) {
			var cmd = this.replaceCmdUploadFinished ? this.replaceCmdUploadFinished : this.cmdUploadFinished;
			var tmp = cmd.split(',');
			tmp.splice(1, 0, resp);
			window.setTimeout(this.controller.fireCmd, 100, [this.uploader.win, tmp]);
			//window.setTimeout(this.controller.fireCmd, 500, [this.uploader.win, ['close']]);
		}
	};
}
Fileupload_sender_base.prototype = Object.create(Fileupload_sender_abstract.prototype);
Fileupload_sender_base.prototype.constructor = Fileupload_sender_base;

function Fileupload_sender_bindoc(uploader) {
	Fileupload_sender_abstract.call(this, uploader);

	this.uploader = uploader;
	this.totalWeight = 0;

	this.appendMoreData = function (fd) {
		var form = this.uploader.doc.we_form;

		switch (uploader.uiType) {
			case 'wedoc':
			case 'editFull':
				fd.append('we_transaction', form.we_transaction.value);
				return fd;
		}

		fd.append('fu_file_sameName', form.fu_file_sameName.value);
		fd.append('fu_file_parentID', form.fu_file_parentID.value);
		fd.append('fu_file_parentDir', form.fu_file_parentDir.value);
		fd.append('fu_doc_categories', (form.fu_doc_categories ? form.fu_doc_categories.value : ''));
		fd.append('fu_doc_isSearchable', form.fu_doc_isSearchable.value);
		if (form.fu_doc_parseFile) {
			fd.append('fu_doc_parseFile', form.fu_doc_parseFile.value);
		}
		fd.append('fu_doc_thumbs', form.fu_doc_thumbs.value);
		fd.append('fu_doc_alt', form.fu_doc_alt.value);
		fd.append('fu_doc_title', form.fu_doc_title.value);

		return fd;
	};

	this.doOnFileFinished = function (resp) {};

	this.postProcess = function () {
		var resp = this.currentResponse,
				win = this.uploader.win,
				transaction = this.currentFile.transaction;

		this.preparedFiles = [];
		this.uploader.uploaderState = this.controller.STATE.FINISHED;

		switch (uploader.uiType) {
			case 'wedoc':
				if (resp.status === 'success') {
					this.currentFile = null;
					if (WE(true)) {
						win.we_cmd('update_file');
						WE().layout.we_setPath(null, resp.weDoc.path, resp.weDoc.text, 0, "published");
					}
				}
				break;
			case 'editFull':
				if (resp.status === 'success') {
					this.currentFile = null;

					if (win.document.getElementById('weImage')) {
						var src = '/webEdition/we_cmd.php?we_cmd[0]=we_base_showBinaryDoc&we_cmd[1]=image/*&we_cmd[2]=' + transaction;
						win.document.getElementById('weImage').src = src;
					}

					if (this.cmdUploadFinished && this.cmdUploadFinished[0] === 'we_save_document') {
						window.setTimeout(this.view.removeProgress, 2000, win);
					} else {
						this.view.removeProgress(win);
					}

					if (WE(true) && this.cmdUploadFinished) {
						if (this.cmdUploadFinished) {
							window.setTimeout(this.controller.fireCmd, 100, [win, this.cmdUploadFinished]);
						}
					}
				}
				break;
			default:
				if (resp.status === 'failure') {
					this.resetSender();
				} else {
					this.currentFile = null;

					for (var k in resp.weDoc) {
						resp[k] = resp.weDoc[k];
					}

					var cmd = this.cmdUploadFinished;
					if (cmd) {
						var tmp = cmd.split(',');
						tmp.splice(1, 0, resp);
						window.setTimeout(this.controller.fireCmd, 100, [WE().util.dialogTop(this.uploader.win).opener, tmp]);
					}
					window.setTimeout(this.controller.fireCmd, 100, [WE().util.dialogTop(this.uploader.win).opener, ['loadIfActive', WE().consts.tables.FILE_TABLE]]);
					window.setTimeout(this.controller.fireCmd, 500, [this.uploader.win, ['close']]);
				}
				break;
		}
	};
}
Fileupload_sender_bindoc.prototype = Object.create(Fileupload_sender_abstract.prototype);
Fileupload_sender_bindoc.prototype.constructor = Fileupload_sender_bindoc;

function Fileupload_sender_import(uploader) {
	Fileupload_sender_abstract.call(this, uploader);

	this.isCancelled = false;
	this.totalChunks = 0; //FIXME: apply consistent terminology to differ between currentfile and all files: and make it part of abstract

	this.prepareUpload = function () {
		if (this.currentFile === -1) { // callable just once!
			this.uploadFiles = [];
			this.totalWeight = 0;

			for (var i = 0,
					c = 0; i < this.preparedFiles.length; i++) {
				if (typeof this.preparedFiles[i] === 'object' && this.preparedFiles[i] !== null && this.preparedFiles[i].getIsUploadable()) {
					this.preparedFiles[i].fileNum = c++;
					this.preparedFiles[i].preparedFilesIndex = i;
					this.uploadFiles.push(this.preparedFiles[i]);
					this.totalWeight += this.preparedFiles[i].size;
					this.preparedFiles[i].lastMeasuredSize = this.preparedFiles[i].size;
				}
			}
			this.totalFiles = this.uploadFiles.length;

			if (this.totalFiles > 0) {
				this.currentWeight = 0;
				this.totalChunks = Math.ceil(this.totalWeight / this.chunkSize);
				this.currentWeight = 0;
				this.currentWeightTag = 0;
				this.controller.processEvents(this.controller.ON.UPLOAD_START);//NEW

				return true;
			}
		}

		return false;
	};

	this.appendMoreData = function (fd) {
		let
				form = this.uploader.doc.we_form,
				cur = this.currentFile;

		fd.append('weFormNum', cur.fileNum + 1);
		fd.append('weFormCount', this.totalFiles);
		fd.append('we_cmd[0]', 'import_files');
		fd.append('step', 1);

		fd.append('fu_file_sameName', form.fu_file_sameName.value);
		fd.append('fu_file_parentID', form.fu_file_parentID.value);
		fd.append('fu_doc_categories', (form.fu_doc_categories ? form.fu_doc_categories.value : ''));
		fd.append('fu_doc_isSearchable', form.fu_doc_isSearchable.value);
		fd.append('fu_doc_parseFile', form.fu_doc_parseFile.value);
		fd.append('fu_doc_thumbs', form.fu_doc_thumbs.value);
		fd.append('fu_doc_importDirs', (form.fu_doc_importDirs ? form.fu_doc_importDirs.value : ''));

		return fd;
	};

	this.postProcess = function (resp) {
		var resp = this.currentResponse,
				messages = [];

		this.view.reloadOpener();
		this.uploader.uploaderState = this.controller.STATE.FINISHED;

		this.view.setExternalProgress(100);
		this.view.setExternalProgressText('progress_title', '');

		if (this.collectedMessages.success.length) {
			messages.push(WE().consts.g_l.fileupload.successfullImports + '<br/><ul>' + this.collectedMessages.success.join('') + '</ul>');
		}
		if (this.collectedMessages.failure.length || this.collectedMessages.rejected.length) {
			messages.push(WE().consts.g_l.fileupload.failedImports + '<br/><ul>' + this.collectedMessages.failure.join('') + this.collectedMessages.rejected.join('') + '</ul>');
		}
		WE().util.showMessage(messages.join('<br/><br/>'), WE().consts.message.WE_MESSAGE_INFO, this.uploader.win);

		if (this.cmdUploadFinished || this.replaceCmdUploadFinished) {
			var cmd = this.cmdUploadFinished;
			if (cmd) {
				var tmp = cmd.split(',');
				resp.success = this.importedIDs; // change in commands to importedIDs
				tmp.splice(1, 0, resp);
				window.setTimeout(this.controller.fireCmd, 100, [WE().util.dialogTop(this.uploader.win), tmp]);
			}
			this.uploader.reset();
			window.setTimeout(this.controller.fireCmd, 1000, [this.uploader.win, ['close']]);
		}
		window.setTimeout(this.uploader.reset.bind(this.uploader), 1000);
	};
}
Fileupload_sender_import.prototype = Object.create(Fileupload_sender_abstract.prototype);
Fileupload_sender_import.prototype.constructor = Fileupload_sender_import;
