/* global top, WE */

/**
 * webEdition CMS
 *
 * $Rev: 3514922d72c4 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-12-15 16:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
var tmp = WE().util.getDynamicVar(document, 'loadVarSelectors', 'data-selector');
//FIMXE: remove when selectorbody is not an iframe anymore
var fileSelect = Object.prototype.toString.call(tmp) === '[object Array]' ? fileSelect : tmp;
WE().util.loadConsts(document, "g_l.fileselector");
WE().util.loadConsts(document, "selectors");
var entries = [];
var clickCount = 0;
WE().util.dialogTop(window).metaKeys = {
	ctrl: false,
	shift: false,
	doubleClick: false,
	inputClick: false,
	doubleTout: null
};
function applyOnEnter(evt){
	let _elemName = "target";
	if(evt.srcElement !== undefined){ // IE
		_elemName = "srcElement";
	}

	if(!(evt[_elemName].tagName === "SELECT" ||
			(evt[_elemName].tagName === "INPUT" && evt[_elemName].name !== "fname")
			)){
		WE().util.dialogTop(window).press_ok_button();
		return true;
	}

}
function closeOnEscape(){
	WE().util.dialogTop(window).exit_close();
}

function goBackDir(){
	setDir(WE().util.dialogTop(window).fileSelect.data.parentID);
}

function getEntry(id){
	for(let entry of WE().util.dialogTop(window).entries) {
		if(entry.ID == id){
			return entry;
		}
	}
	return {
		ID: 0,
		icon: "",
		text: "/",
		isFolder: 1,
		path: "/"
	};
}

function clearEntries(){
	entries = [];
}

function exit_close(){
	WE().util.dialogTop(window).doClose();
}

function doClick(id, ct){
	let dtop = WE().util.dialogTop(window);
	if(ct == 1){
		if(dtop.metaKeys.doubleClick){
			setDir(id);
			window.setTimeout(function(){
				dtop.metaKeys.doubleClick = false;
			}, 400);
		}
	} else if(dtop.fileSelect.options.multiple){
		if(dtop.metaKeys.shift){
			let
					oldid = dtop.fileSelect.data.currentID,
					currendPos = getPositionByID(id),
					firstSelected = getFirstSelected();
			if(currendPos > firstSelected){
				selectFilesFrom(firstSelected, currendPos);
			} else if(currendPos < firstSelected){
				selectFilesFrom(currendPos, firstSelected);
			} else {
				selectFile(id);
			}
			dtop.fileSelect.data.currentID = oldid;
		} else if(!dtop.metaKeys.ctrl){
			selectFile(id);
		} else if(isFileSelected(id)){
			unselectFile(id);
		} else {
			selectFile(id);
		}
	} else {
		selectFile(id);
	}
	if(dtop.metaKeys.ctrl){
		dtop.metaKeys.ctrl = 0;
	}
	if(dtop.metaKeys.shift){
		dtop.metaKeys.shift = 0;
	}
}

function setDir(id){
	let fspath = top.document.getElementById('fspath'),
			dtop = WE().util.dialogTop(window),
			e = getEntry(id);
	dtop.fileSelect.data.currentID = id;
	dtop.fileSelect.data.currentDir = id;
	dtop.fileSelect.data.currentPath = e.path;
	dtop.fileSelect.data.currentText = e.text;
	dtop.document.getElementsByName("fname")[0].value = e.text;
	dtop.fscmd.location.replace(dtop.queryString(WE().consts.selectors.CMD, id));
	if(fspath){
		fspath.innerHTML = e.path;
	}
}

function setRootDir(){
	setDir(WE().util.dialogTop(window).fileSelect.options.rootDirID);
}

function selectFile(id){
	let
			fspath = top.document.getElementById('fspath'),
			dtop = WE().util.dialogTop(window),
			a = dtop.document.getElementsByName("fname")[0];
	if(id){
		let e = getEntry(id);
		if(a.value != e.text &&
				a.value.indexOf(e.text + ",") === -1 &&
				a.value.indexOf("," + e.text + ",") === -1 &&
				a.value.indexOf("," + e.text + ",") === -1){
			a.value = a.value ? (a.value + "," + e.text) : e.text;
			if(fspath){
				fspath.innerHTML = e.path;
			}
		}
		if(dtop.fsbody.document.getElementById("line_" + id)){
			dtop.fsbody.document.getElementById("line_" + id).classList.add("selected");
		}
		dtop.fileSelect.data.currentPath = e.path;
		dtop.fileSelect.data.currentID = id;
		dtop.fileSelect.data.currentType = e.contentType;
	} else {
		a.value = "";
		dtop.fileSelect.data.currentPath = "";
	}
}

function addEntry(ID, text, isFolder, path, ct){
	entries.push({
		ID: ID,
		text: text,
		isFolder: isFolder,
		path: path,
		contentType: ct
	});
}

function writeBody(d){
	let body = '';
	for(let entry of entries) {
		body += '<tr' + ((entry.ID == WE().util.dialogTop(window).fileSelect.data.currentID) ? ' class="selected"' : '') + ' id="line_' + entry.ID + '"' + ' onclick="return selectorOnClick(event,' + entry.ID + ');"' + (entry.isFolder ? ' ondblclick="return selectorOnDblClick(' + entry.ID + ');"' : '') + ' >' +
				'<td class="selector selectoricon">' + WE().util.getTreeIcon(entry.contentType, false) + '</td>' +
				'<td class="selector filename"  title="' + entry.text + '"><div class="cutText">' + entry.text + '</div></td>' +
				'<td class="selector remain"></td></tr>';
	}
	d.innerHTML = '<table class="selector sort" data-skipSort="0"><thead><tr>' + WE().util.dialogTop(window).fileSelect.options.selectorHeader + '</tr></thead><tbody>' + body + '</tbody></table>';
	window.fsbody.$("table.sort").each(WE().layout.applyTableSort);
}

function getFirstSelected(){
	for(let i = 0; i < entries.length; i++) {
		if(WE().util.dialogTop(window).fsbody.document.getElementById("line_" + entries[i].ID).classList.contains("selected")){
			return i;
		}
	}
	return -1;
}

function unselectFile(id){
	let e = getEntry(id),
			dtop = WE().util.dialogTop(window);
	dtop.fsbody.document.getElementById("line_" + id).style.classList.remove("selected");
	let foo = dtop.document.getElementsByName("fname")[0].value.split(/,/);
	for(let text of foo) {
		if(text == e.text){
			text = "";
			break;
		}
	}
	let str = "";
	for(let text of foo) {
		if(text){
			str += text + ",";
		}
	}
	str = str.replace(/(.*),$/, "$1");
	dtop.document.getElementsByName("fname")[0].value = str;
}


function selectFilesFrom(from, to){
	unselectAllFiles();
	for(let i = from; i <= to; i++) {
		if(entries[i]){
			selectFile(entries[i].ID);
		}
	}
}

function getPositionByID(id){
	for(let i = 0; i < entries.length; i++) {
		if(entries[i].ID == id){
			return i;
		}
	}
	return -1;
}

function isFileSelected(id){
	return (WE().util.dialogTop(window).fsbody.document.getElementById("line_" + id).classList.contains("selected"));
}

function unselectAllFiles(){
	let dtop = WE().util.dialogTop(window);
	for(let entry of entries) {
		let elem = dtop.fsbody.document.getElementById("line_" + entry.ID);
		if(elem){
			elem.classList.remove("selected");
		}
	}
	dtop.document.getElementsByName("fname")[0].value = "";
	dtop.delBut(false);
	if(dtop.showPreview){
		dtop.showPreview(0);
	}
}

function queryString(what, id){
	let dtop = WE().util.dialogTop(window);
	return dtop.fileSelect.options.formtarget +
			'&we_cmd[what]=' + what +
			'&we_cmd[table]=' + dtop.fileSelect.options.table +
			'&we_cmd[id]=' + id +
			"&we_cmd[filter]=" + dtop.fileSelect.data.currentType;
}

function fillIDs(){
	let dtop = WE().util.dialogTop(window);
	dtop.fileSelect.data.allIDs = [];
	dtop.fileSelect.data.allPaths = [];
	dtop.fileSelect.data.allTexts = [];
	dtop.fileSelect.data.allIsFolder = [];
	for(var i = 0; i < entries.length; i++) {
		if(isFileSelected(entries[i].ID)){
			dtop.fileSelect.data.allIDs.push(entries[i].ID);
			dtop.fileSelect.data.allPaths.push(entries[i].path);
			dtop.fileSelect.data.allTexts.push(entries[i].text);
			dtop.fileSelect.data.allIsFolder.push(entries[i].isFolder);
		}
	}
	if(dtop.fileSelect.data.currentID !== "" && dtop.fileSelect.data.allIDs.indexOf(dtop.fileSelect.data.currentID) === -1){
		dtop.fileSelect.data.allIDs.push(dtop.fileSelect.data.currentID);
	}
	if(dtop.fileSelect.data.currentPath !== "" && dtop.fileSelect.data.allPaths.indexOf(dtop.fileSelect.data.currentPath) === -1){
		dtop.fileSelect.data.allPaths.push(dtop.fileSelect.data.currentPath);
		dtop.fileSelect.data.allTexts.push(we_makeTextFromPath(dtop.fileSelect.data.currentPath));
	}
}

function we_makeTextFromPath(path){
	let position = path.lastIndexOf("/");
	if(position > -1 && position < path.length){
		return path.substring(position + 1);
	}
	return "";
}

function weonclick(e){
	let dtop = WE().util.dialogTop(window);
	if(e.altKey || e.metaKey || e.ctrlKey){
		dtop.metaKeys.ctrl = true;
	}
	if(e.shiftKey){
		dtop.metaKeys.shift = true;
	}
	if(dtop.fileSelect.options.multiple){
		if((dtop.metaKeys.shift === false) && (dtop.metaKeys.ctrl === false)){
			dtop.unselectAllFiles();
		}
	} else {
		dtop.unselectAllFiles();
	}
}

function elementSelected(){
	return WE().util.dialogTop(window).document.getElementsByName("fname")[0].value !== "";
}

function press_ok_button(){
	if(elementSelected()){
		WE().util.dialogTop(window).exit_open();
	} else {
		WE().util.dialogTop(window).exit_close();
	}
}

function delBut(enable){
	let dtop = WE().util.dialogTop(window);
	if(enable){
		WE().layout.button.switch_button_state(document, "delete", "enabled");
		if(dtop.fileSelect.options.userCanEditCat){
			WE().layout.button.switch_button_state(document, "btn_function_trash", "enabled");
			dtop.fileSelect.data.changeCatState = true;
		}
	} else {
		WE().layout.button.switch_button_state(document, "delete", "disabled");
		WE().layout.button.switch_button_state(document, "btn_function_trash", "disabled");
		dtop.fileSelect.data.changeCatState = false;
	}
}

function startFrameset(){
	WE().util.dialogTop(window).document.getElementById('fspath').innerHTML = (WE().util.dialogTop(window).fileSelect.data.startPath === '' ? '/' : WE().util.dialogTop(window).fileSelect.data.startPath);
}

function rootDirButs(enable){
	if(enable){
		WE().layout.button.switch_button_state(document, "root_dir", "enabled");
		WE().layout.button.switch_button_state(document, "btn_fs_back", "enabled");
	} else {
		WE().layout.button.switch_button_state(document, "root_dir", "disabled");
		WE().layout.button.switch_button_state(document, "btn_fs_back", "disabled");
	}
	WE().util.dialogTop(window).fileSelect.data.rootDirButsState = enable;
}
function newFolderBut(enable){
	if(enable){
		WE().layout.button.switch_button_state(document, "btn_new_dir", "enabled");
	} else {
		WE().layout.button.switch_button_state(document, "btn_new_dir", "disabled");
	}
	WE().util.dialogTop(window).fileSelect.data.makefolderState = enable;
}

function NewBut(enable){
	if(enable){
		if(WE().util.dialogTop(window).fileSelect.options.userCanEditCat){
			WE().layout.button.switch_button_state(document, "btn_new_dir", "enabled");
			WE().layout.button.switch_button_state(document, "btn_add_cat", "enabled");
		} else {
			WE().layout.button.switch_button_state(document, "btn_new_dir", "disabled");
			WE().layout.button.switch_button_state(document, "btn_add_cat", "disabled");
		}
	}
}

function newFileBut(enable){
	if(enable){
		WE().layout.button.switch_button_state(document, "btn_add_file", "enabled");
	} else {
		WE().layout.button.switch_button_state(document, "btn_add_file", "disabled");
	}
	WE().util.dialogTop(window).fileSelect.data.newFileState = enable;
}

function clearOptions(){
	let a = WE().util.dialogTop(window).document.getElementById("lookin");
	while(a.options.length) {
		a.options.remove(0);
	}
}
function addOption(txt, id){
	let a = WE().util.dialogTop(window).document.getElementById("lookin");
	a.options[a.options.length] = new window.Option(txt, id);
	a.selectedIndex = (a.options.length > 0 ?
			a.options.length - 1 :
			0);
}
function selectIt(){
	let a = WE().util.dialogTop(window).document.getElementById("lookin");
	a.selectedIndex = a.options.length - 1;
}

function setview(view){
	let dtop = WE().util.dialogTop(window);
	dtop.fileSelect.options.view = view;
	let zoom = dtop.document.getElementsByName("zoom")[0];
	switch(view) {
		case 'list':
			zoom.value = 100;
			if(zoom.onchange){
				zoom.onchange();
			}
			zoom.disabled = true;
			zoom.style.display = "none";
			break;
		case 'icons':
			zoom.disabled = false;
			zoom.style.display = "inline";
			break;
	}
	dtop.document.getElementById('list').style.display = (view === 'list' ? "none" : "table-cell");
	dtop.document.getElementById('icons').style.display = (view === 'icons' ? "none" : "table-cell");
	dtop.writeBody(dtop.fsbody.document.body);
}

function we_cmd(){
	/*jshint validthis:true */
	let
			dtop = WE().util.dialogTop(window),
			caller = (this && this.window === this ? this : window),
			args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
			url = WE().util.getWe_cmdArgsUrl(args);
	switch(args[0]) {
		case 'clearEntries':
			dtop.clearEntries();
			break;
		case 'addEntries':
			if(!args[1]){
				break;
			}
			for(let el of args[1]) {
				dtop.addEntry.apply(window, el);
			}
			break;
		case 'writeOptions':
			dtop.clearOptions();
			if(!args[1]){
				break;
			}
			for(let el of args[1]) {
				dtop.addOption.apply(window, el);
			}
			dtop.selectIt();
			break;
		case 'writeBody':
			if(dtop.fsbody.document.body){
				dtop.writeBody(dtop.fsbody.document.body);
			}
			break;
		case 'updateTreeEntry':
			{
				let ref = (dtop.top.treeData ? dtop.top : (dtop.opener.top.opener.top.treeData ? dtop.opener.top.opener.top : null));
				if(ref){
					ref.treeData.updateEntry(args[1]);
				}
			}
			break;
		case 'makeNewTreeEntry':
			{
				let ref = (dtop.opener.top.treeData ? dtop.opener.top : (dtop.opener.top.opener.top.treeData ? dtop.opener.top.opener.top : null));
				if(ref){
					ref.treeData.makeNewEntry(args[1]);
				}
				if(dtop.fileSelect.options.canSelectDir){
					dtop.document.getElementsByName("fname")[0].value = dtop.fileSelect.data.currentText;
				}
			}
			break;
		case 'updateSelectData':
			var obj = args[1];
			for(let i in obj) {
				dtop.fileSelect.data[i] = obj[i];
				if(i === 'currentText'){
					dtop.document.getElementsByName("fname")[0].value = dtop.fileSelect.data.currentText;
				}
			}
			break;
		case 'setButtons':
			for(let enable of args[1]) {
				switch(enable[0]) {
					case 'NewFolderBut':
						newFolderBut(enable[1]);
						break;
					case 'NewFileBut':
						newFileBut(enable[1]);
						break;
					case 'RootDirButs':
						rootDirButs(enable[1]);
						break;
					case 'DelBut':
						delBut(enable[1]);
				}
			}
			break;
		case 'setLookinDir':
			dtop.setDir(dtop.document.getElementById("lookin").value);
			break;
		case 'updateCatChooserButton':
			dtop.frames.fsvalues.document.we_form.elements.FolderID.value = args[1];
			dtop.frames.fsvalues.document.we_form.elements.FolderIDPath.value = args[2];
			break;
		case 'newCatSuccess':
			dtop.hot = true; // this is hot for category edit!!

			if(dtop.fileSelect.data.currentID){
				dtop.delBut(false);
				dtop.showPref(dtop.fileSelect.data.currentID);
			}
			break;
		case 'unselectAllFiles':
			dtop.unselectAllFiles();
			break;
		case 'postRenameCat':
			dtop.hot = true; // this is hot for category edit!!
			if(dtop.fileSelect.data.currentID){
				dtop.delBut(true);
				dtop.showPref(dtop.fileSelect.data.currentID);
			}
			break;
		case 'selector_insertFromUploader':
			var importedDoc = args[1];
			dtop.reloadDir();
			dtop.unselectAllFiles();
			dtop.addEntry(importedDoc.currentID, importedDoc.currentText, false, importedDoc.currentPath, importedDoc.currentType);
			dtop.doClick(importedDoc.currentID);
			window.setTimeout(dtop.selectFile, 200, importedDoc.currentID);
			break;
		case "we_selector_file":
			(new (WE().layout.overlay)(caller, 'self', url, "we_selector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		default:
			dtop.opener.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function selectorOnClick(event, id){
	let dtop = WE().util.dialogTop(window);
	weonclick(event);
	dtop.metaKeys.doubleTout = setTimeout(function(){
		if(!dtop.metaKeys.doubleClick){
			dtop.doClick(id, 0);
		} else {
			dtop.metaKeys.doubleClick = false;
		}
	}, 300);
	return true;
}

function selectorOnDblClick(id){
	WE().util.dialogTop(window).metaKeys.doubleClick = true;
	window.clearTimeout(WE().util.dialogTop(window).metaKeys.doubleTout);
	WE().util.dialogTop(window).doClick(id, 1);
	return true;
}

function exit_open(){
	let
			elemID,
			suggestID = '',
			dtop = WE().util.dialogTop(window),
			opener = dtop.opener,
			fsData = dtop.fileSelect.data;

	if(fsData.currentType && fsData.currentType.startsWith("folder/")){
		WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=selectorSetDir&type=request", {table: dtop.fileSelect.options.table, id: fsData.currentID});
	}

	if(fsData.JSIDName){
		elemID = opener.document.we_form.elements[fsData.JSIDName];
		elemID.value = fsData.currentID;
		if(elemID.id && elemID.id.search('yuiAcResult') === 0){
			suggestID = elemID.id.substr(11);
		}
	}

	if(fsData.JSTextName){
		let elemText = opener.document.we_form.elements[fsData.JSTextName];
		elemText.value = fsData.currentID ? fsData.currentPath : "";
		if(elemID && elemID.id && elemID.id.search('yuiAcInput') === 0){
			suggestID = elemText.id.substr(10);
		}

		if((dtop.parent !== undefined) && (dtop.parent.frames.editHeader !== undefined)){
			if(fsData.currentType !== ""){
				let setTabsCurPath;
				switch(fsData.currentType) {
					case "noalias":
						setTabsCurPath = "@" + fsData.currentText;
						break;
					default:
						setTabsCurPath = fsData.currentPath;
				}
				if(getEntry(fsData.currentID).isFolder){
					opener.parent.frames.editHeader.weTabs.setTitlePath("", setTabsCurPath);
				} else {
					opener.parent.frames.editHeader.weTabs.setTitlePath(setTabsCurPath);
				}
			}
		}
		WE().layout.weSuggest.checkRequired(opener, opener.document.we_form.elements[fsData.JSTextName].id);
	}
	if(fsData.JSIDName){
		let el = opener.document.we_form.elements[fsData.JSIDName];
		if(el.length !== undefined && el.length > 1){
			dtop.doClose();
			WE().util.showMessage(WE().consts.g_l.alert.templateDuplicateEntry, WE().consts.message.WE_MESSAGE_ERROR, window);
		}
	}
// check is required for simple id-selectors on tag wizzard (with no hidden result field an no suggestor, i.e no data-xyz attributes)
// FIXME: implementing fullflegded selector/suggestors in tag wizzard too, makes this obsolete
	if(fsData.JSIDName &&
			!fsData.JSTextName &&
			!opener.document.we_form.elements[fsData.JSIDName].getAttribute("data-result") &&
			opener.document.we_form.elements[fsData.JSIDName].getAttribute("required")){
		WE().layout.weSuggest.checkRequired(opener, opener.document.we_form.elements[fsData.JSIDName].id);
	}

// if selector is used in combination with suggestor and we know sugestorID we call sugestors postporcess
	if(suggestID){
		opener.we_cmd('we_suggest_postprocessSelection', fsData, suggestID);
	}

// if not in combination with suggestor or if using custom selector callback: no onSelect is called automatically
	if(fsData.JSCommand){
		fillIDs();
		let tmp = fsData.JSCommand.split(',');
		tmp.splice(1, 0, fsData);
		opener.we_cmd.apply(opener, tmp);
	}

	WE().util.dialogTop(window).doClose();
}

WE().util.dialogTop(window).focus();
