/**
 * webEdition CMS
 *
 * $Rev: 71ea67ac3cbd $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2023-01-19 12:35 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* global top, WE */
'use strict';
var customer = WE().util.getDynamicVar(document, 'loadVarCustomer_property', 'data-customer');

var loaded = 0;

function doUnload(){
	WE().layout.jsWindow.prototype.closeAll(window);
}

function formatDate(date, format){
	var daynum = date.getDate();
	var day = daynum.toString();
	if(format.search("d") !== -1){
		if(daynum < 10){
			day = "0" + day;
		}
	}

	format = format.replace("d", day).replace("j", day);

	var monthnum = date.getMonth() + 1;
	var month = monthnum.toString();
	if(format.search("m") !== -1){
		if(monthnum < 10){
			month = "0" + month;
		}
	}

	format = format.replace("m", month).replace("n", month);

	format = format.replace("Y", date.getFullYear());
	var yearnum = date.getYear();
	var year = yearnum.toString();
	format = format.replace("y", year.substr(2, 2));

	var hournum = date.getHours();
	var hour = hournum.toString();
	if(format.search("H") !== -1){
		if(hournum < 10){
			hour = "0" + hour;
		}
	}

	format = format.replace("H", hour).replace("G", hour);

	var minnum = date.getMinutes();
	var min = minnum.toString();
	if(minnum < 10){
		min = "0" + min;
	}

	format = format.replace("i", min);


	/*var secnum=date.getSeconds();
	 var sec=secnum.toString();
	 if(secnum<10) sec="0"+sec;*/

	var sec = "00";
	format = format.replace("s", sec).replace(/\\/g, "");
	return format;
}

function we_cmd(){
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
			args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments));
	//var url = WE().util.getWe_cmdArgsUrl(args);

	switch(args[0]) {
		case "switchPage":
			document.we_form.cmd.value = args[0];
			document.we_form.branch.value = args[1];
			submitForm();
			break;
		case "submitForm":
			submitForm();
			break;
		case "setNewBranchName":
			WE().util.dialogTop(window).opener.document.we_form.branch.value = args[1];
			submitForm();
			if(WE().util.dialogTop(window).opener.document.we_form && WE().util.dialogTop(window).opener.document.we_form.branch){
				WE().util.dialogTop(window).opener.document.we_form.branch.value = args[2];
				WE().util.dialogTop(window).opener.refreshForm();
			}
			caller.close();
			break;
		case 'refreshForm':
			refreshForm();
			break;
		case 'toggleEmpty':
			toggleEmpty(args[1], args[2]);

			break;
		default:
			top.content.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function toggleEmpty(name, def){
	/**
	 * Get all regular Input or we_date_x input
	 * @returns {NodeListOf}
	 */
	function getToggleInput(){
		return document.querySelectorAll('[name="customer[' + name + ']"],[name="customer[we_date_' + name + '][date]"],[name="customer[we_date_' + name + '][time]"]');
	}

	function toggleInput(input){
		// input may be sumoselect, handle accordingly
		const isSumoSelect = input.parentElement.classList.contains('SumoSelect');
		if(isSumoSelect){
			const sumoSelect = input.parentElement.getElementsByClassName('newSelect')[0];
			if(sumoSelect.disabled){
				sumoSelect.sumo.enable();
				return;
			}
			sumoSelect.sumo.unSelectAll();
			sumoSelect.sumo.disable();
			return;
		}
		// Normal Input: do not set default for Time (may be Date value)
		if(input.disabled){
			input.disabled = false;
			input.value = input.type === 'time' ? '' : def;
			return;
		}
		input.disabled = true;
		input.value = "";
		return;
	}
	const inputs = getToggleInput();
	for(const input of inputs) {
		toggleInput(input);
	}
	let icon = document.getElementById("ban_" + name);
	if(icon.style.visibility === 'hidden'){
		icon.style.visibility = 'visible';
	} else {
		icon.style.visibility = 'hidden';
	}
	top.content.setHot();
}

function resetLogins(id){
	WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?mod=customer&cmd=customerResetFailedLogins", "custid=" + id, function(weResponse){
		if(weResponse){
			if(weResponse.DataArray.data === "true"){
				document.getElementById("FailedCustomerLogins").innerText = weResponse.DataArray.value;
			}
			WE().util.showMessage(WE().consts.g_l.customer.view.reset_failed_login_successfully, WE().consts.message.WE_MESSAGE_NOTICE, window);
		}
	});
}

function refreshForm(){
	if(document.we_form.cmd.value !== "home"){
		we_cmd("switchPage", top.content.activ_tab);
		top.content.editor.edheader.location = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=customer&pnt=edheader&text=" + encodeURI(customer.username);
	}
}

function submitForm(target, action, method, form){
	var f = form ? window.document.forms[form] : window.document.we_form;
	f.target = target ? target : "edbody";
	f.action = action ? action : WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=customer";
	f.method = method ? method : "post";

	f.submit();
}