/* global WE, weSelect, weInput, categories_edit */

/**
 * webEdition CMS
 *
 * $Rev: 62067699fe89 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-09-01 13:04 +0200 $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';
var ruleData = WE().util.getDynamicVar(document, 'loadVarNavigationRules', 'data-rules');

var resetFields = [
	"NavigationName", "NavigationID", "NavigationIDPath", "FolderID", "FolderIDPath", "DoctypeID", "ClassID", "ClassIDPath", "WorkspaceID"
];
var categories_edit;

function switchType(value) {
	// 1st hide all
	let
		allFields = document.getElementsByClassName("switch"),
		switchFields = document.getElementsByClassName("switch_" + value);

	for (let field of allFields) {
		field.style.display = "none";
	}

	// show needed

	for (let field of switchFields) {
		field.style.display = "";
	}
}

function clearNavigationForm() {
	for (let field of resetFields) {
		if (document.we_form[field]) {
			document.we_form[field].value = "";
		}
	}

	document.we_form.ID.value = "0";
	weSelect.removeOptions("WorkspaceID");
	removeAllCats();
}

function removeAllCats() {
	if (categories_edit.itemCount > 0) {
		while (categories_edit.itemCount > 0) {
			categories_edit.delItem(categories_edit.itemCount);
		}
	}
}

function addCat(paths) {
	for (let path of paths) {
		if (path !== "") {
			categories_edit.addItem();
			categories_edit.setItem(0, (categories_edit.itemCount - 1), path);
		}
	}
	categories_edit.showVariant(0);
}

function we_cmd() {
	/*jshint validthis:true */
	let caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
		url = WE().util.getWe_cmdArgsUrl(args);

	switch (args[0]) {
		case "addCat":
			addCat(args[1].allPaths);
			break;
		case "switchType":
			switchType(args[1]);
			break;
		case "new_navigation_rule":
			document.we_form.navigationRules.value = '';
			clearNavigationForm();
			break;
		case "save_navigation_rule":
			{
				let isValid = true;
				if (document.we_form.SelectionType.options[0].selected === true) {
					isValid = WE().layout.weSuggest.checkRequired(window, "yuiAcInputFolderIDPath");
				} else if (document.we_form.SelectionType.options[1] !== undefined && document.we_form.SelectionType.options[1].selected === true) {
					isValid = WE().layout.weSuggest.checkRequired(window, "yuiAcInputClassIDPath");
				}
				if (isValid && WE().layout.weSuggest.checkRequired(window, "yuiAcInputNavigationIDPath")) {
					weInput.setValue("cmd", "save_navigation_rule");
					document.we_form.submit();
					break;
				}
			}
			WE().util.showMessage(WE().consts.g_l.navigation.rule.save_error_fields_value_not_valid, WE().consts.message.WE_MESSAGE_ERROR, window);
			return false;
		case "delete_navigation_rule":
			{
				let navId = document.we_form.navigationRules.value;
				if (navId) {
					document.we_form.NavigationName.value = "";
					weInput.setValue("cmd", "delete_navigation_rule");
					weInput.setValue("ID", navId);
					document.we_form.submit();
				}
			}
			break;
		case "navigation_edit_rule":
			weInput.setValue("cmd", "navigation_edit_rule");
			weInput.setValue("ID", args[1]);
			document.we_form.submit();
			break;
		case "get_workspaces":
			weInput.setValue("cmd", "get_workspaces");
			document.we_form.submit();
			break;
		case "we_selector_directory":
			(new (WE().layout.overlay)(caller, 'top', url, "we_selector_directory", WE().consts.size.dialog.big, WE().consts.size.dialog.small)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.dirSelector).open();
			break;
		case "we_selector_file":
		case "we_selector_image":
		case "we_selector_document":
			(new (WE().layout.overlay)(caller, 'top', url, args[0], WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		default:
			WE().util.dialogTop(window).opener.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function initCat() {
	categories_edit = new (WE().util.multi_edit)("categories", window, 0, ruleData.trashButton, 400, false, '', WE().consts.contentTypes.CATEGORY);
	categories_edit.addVariant();
	document.we_form.CategoriesControl.value = categories_edit.name;
	categories_edit.showVariant(0);
}

$(function () {
	initCat();
});