/* global top, WE */

/**
 * webEdition CMS
 *
 * $Rev: dbbac2a2e121 $
 * $Author: lukasimhof $
 * $Date: 2022-10-25 17:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
WE().layout.we_tinyMCE = WE().layout.we_tinyMCE ? WE().layout.we_tinyMCE : {
	'do': {},
	functions: {},
	setup: {},
	vars: {},
	overwrite: {}
};
WE().layout.we_tinyMCE.vars.weicons = WE().util.getDynamicVar(document, 'loadWeicons', 'data-weicons');
WE().layout.we_tinyMCE.vars.timeouts = {
	maxLen: null,
	reportValidity: null,
};

/* icons preload test */
/*
 WE().layout.we_tinyMCE.vars.weicons = {};
 $(function () {
 var icons = WE().util.getDynamicVar(document, 'loadWeicons', 'data-weicons');
 var container = top.document.getElementById('tinyIconsDiv');
 var div;
 for(var icon in icons){
 if(icons.hasOwnProperty(icon)){
 div = document.createElement('DIV');
 div.innerHTML = icons[icon];
 container.appendChild(div);
 WE().layout.we_tinyMCE.vars.weicons[icon] = container.lastChild.innerHTML;//div.innerHTML;
 }
 }
 });
 */
/* END icons preload test */


WE().layout.we_tinyMCE.setup.setupEditor = function (ed) {
	if (!ed.settings.menu) {
		ed.settings.menubar = false;
	}
	WE().layout.we_tinyMCE.setup.setTinyParams(ed);
	WE().layout.we_tinyMCE.setup.addControlManager(ed);
	WE().layout.we_tinyMCE.setup.setupPasteFromTiny(ed);
	WE().layout.we_tinyMCE.setup.setContentCss(ed);
	WE().layout.we_tinyMCE.setup.addListeners(ed);
	WE().layout.we_tinyMCE.setup.addIcons(ed);
	ed.on('addStyleModifications', function (e) {
		WE().layout.we_tinyMCE.functions.processStylesFromCss(ed, e.items);
	});
	WE().layout.we_tinyMCE.setup.addImageQuickbar(ed);
};

WE().layout.we_tinyMCE.setup.setupPasteFromTiny = function (ed) {
	let conf = ed.settings;
	//div_clipboard_text_html = document.createElement('DIV'),
	//div_clipboard_text_plain = document.createElement('DIV');

	//conf.wePaste_pasteFormat_tmp = false;

	ed.on('copy', function (e) {
		WE().layout.we_tinyMCE.do.onCopyCut(ed, false);
	});

	ed.on('cut', function (e) {
		WE().layout.we_tinyMCE.do.onCopyCut(ed, true);
	});

	ed.on('paste', function (e) {});

	conf.paste_preprocess = function (plugin, args) {
		let isTinyContent = false;
		let useTinyHTML = false; // can be set as attribute to tiny HTML cleaning instead of WE

		// after paste if visualaid is on, we must re-activate it onnodechange to set respective classes on newly inserted elements
		if (conf.wevisualaidInitialized && !conf.weHasVisual && !conf.wevisualaidIsVisualTmp) {
			ed.execCommand("mceWevisualaid", true);
			conf.wevisualaidIsVisualTmp = true;
		}

		if (ed.plugins.paste.clipboard.pasteFormat.get() !== 'html') {
			// nothing to do: let tiny care for plaintext
			return;
		}
		let content = args.content;
		if (content &&
				content.search(' data-wetiny="1"') !== -1 ||
				content.search(' data-weTinyMce="1"') !== -1 || // legacy code from WE8 ?
				content.search(' we-tiny="1"') !== -1 || // legacy code from WE8 ?
				content.search(' name="we-tiny-tmpDiv"') !== -1) {
			isTinyContent = true;

			// remove tiny-marker
			content = args.content.replaceAll(/ we-tiny="1"/g, '').replaceAll(/ data-wetiny="1"/g, '');
			if (content.search(' name="we-tiny-tmpDiv"') !== -1) {// in FF we must delete surrounding tmpDiv
				let tmpDiv = document.createElement('div');
				tmpDiv.innerHTML = content;
				content = tmpDiv.firstChild.innerHTML;
				tmpDiv = null;
			}
			args.content = content;
		}

		// some cleaning we do irrespective of what type of HTML we process
		content = content.
				replaceAll(/<script.*\/script[^>]*>/gi, '').
				replaceAll(/<style.*\/style[^>]*>/gi, '');

		if (isTinyContent || useTinyHTML) {
			// paste HTML as is and let tiny do the cleaning
			args.content = content;
			return;
		}

		/* process we cleaned HTML (make function for these tasks) */
		args.content = processWeHTML(content);

		return;
	};

	conf.paste_postprocess = function (plugin, args) {};

	function processWeHTML(content) {
		let tmpDiv = document.createElement('div');
		tmpDiv.innerHTML = content.
				replaceAll(/<div[^>]*>/gi, '<p>').
				replaceAll(/<\/div[^>]*>/gi, '</p>').
				replaceAll(/<figure.*\/figure[^>]*>/gi, '').
				replaceAll(/<img[^>]*>/gi, '');
		return weCleanupR(tmpDiv).innerHTML;
	}

	function weCleanupR(elem) {
		for (let child of elem.children) {
			weCleanupR(child);
		}

		let nodeName = elem.nodeName.toUpperCase();
		switch (nodeName) {
			case 'LI':
			case 'TD':
				if (elem.firstChild && elem.firstChild.nodeName.toUpperCase() === 'P' && elem.children.length === 1) {
					elem.innerHTML = elem.firstChild.innerHTML;
				}
				break;
		}

		Array.from(elem.attributes).forEach(attr => {
			switch (nodeName) {
				case 'A':
					if (attr.name.toUpperCase() === 'HREF') {
						break;
					}
					/* falls through */
				default:
					elem.removeAttribute(attr.name);
			}
		});

		return elem;
	}
};

WE().layout.we_tinyMCE.do.onInit = function (ed) {
	//re-enable textarea, so we can show errors for this text.
	let text = ed.getElement(),
			container = ed.getContainer();
	text.style.display = "block";
	text.style.position = "absolute";
	text.style["z-index"] = -999;
	text.style.width = container.style.width;
	text.style.height = container.style.height;

	//var conf = ed.settings;
	ed.windowManager.openWePlugin = function (args, params) {
		WE().layout.we_tinyMCE.functions.openWePlugin.call(ed.windowManager, ed, args, params);
	};

	//ed.execCommand("mceWevisualaid", true);

	// custom node filters
//	ed.serializer.addNodeFilter("a", WE().layout.we_tinyMCE.functions.customNodeFilter_A);

	// add listeners for events ed has no public events for
	ed.dom.bind(ed.getWin(), 'drop', function (e) {
		return WE().layout.we_tinyMCE.do.onDrop(e, ed);
	});

	ed.getWordCount = function () {
		return ed.plugins.wordcount.getCount();
	};

	if (ed.settings.weEditorType === 'inlineFalse') {
		ed.settings.weWin.addEventListener("resize", WE().layout.we_tinyMCE.functions.resizeEditor.bind(this, ed, false));
		ed.settings.weWin.setTimeout(WE().layout.we_tinyMCE.functions.resizeEditor.bind(this, ed, true), 1);

		WE().layout.we_tinyMCE.functions.wysiwygDialog_setContent(ed);

		if (WE().util.dialogTop(ed.settings.weWin) && WE().util.dialogTop(ed.settings.weWin).opener) {
			ed.settings.weTextareaOpener = WE().util.dialogTop(ed.settings.weWin).opener.document.getElementById(ed.settings.weName);

			if (WE().util.dialogTop(ed.settings.weWin).opener.isEditorScript) {
				// register wysiwyg in header
				WE().util.dialogTop(ed.settings.weWin).opener.we_cmd('registerWysiwygInHeader', WE().util.dialogTop(ed.settings.weWin));
				// register wysiwyg in footer
				WE().util.dialogTop(ed.settings.weWin).opener.we_cmd('registerWysiwygInFooter', WE().util.dialogTop(ed.settings.weWin), ed.settings.weName);
			}
		}
	}

	ed.settings.weWin.tinymce.addI18n('de', {
		"Find whole words only": WE().consts.g_l.tinyMceTranslationObject.search_wholeWords,
		"Count": "",
		"Document": WE().consts.g_l.tinyMceTranslationObject.search_all,
		"Selection": WE().consts.g_l.tinyMceTranslationObject.search_selection,
		"Copy column": WE().consts.g_l.tinyMceTranslationObject.copyColumn,
		"Cut column": WE().consts.g_l.tinyMceTranslationObject.cutColumn,
		"Paste column before": WE().consts.g_l.tinyMceTranslationObject.pasteColumnBefore,
		"Paste column after": WE().consts.g_l.tinyMceTranslationObject.pasteColumnAfter,
	});

	WE().layout.we_tinyMCE.setup.setSpellcheckAndLang(ed, true);
	WE().layout.we_tinyMCE.functions.checkLen(ed);
};

WE().layout.we_tinyMCE.do.onSetContent = function (ed) {
	/*
	 ed.settings.weWin.tinymce.each(ed.dom.select('table'), function (node) {
	 var attrib = ed.dom.getAttrib(node, 'border');
	 if (!attrib || parseInt(attrib) === 0) {
	 ed.dom.addClass(node, 'mce-item-table');
	 } else {
	 ed.dom.addClass(node, 'mce-item-table');
	 }
	 return;
	 });
	 */
};

WE().layout.we_tinyMCE.do.onChange = function (ed) {
	WE().layout.we_tinyMCE.functions.setHot(ed);
};

WE().layout.we_tinyMCE.do.onKeyDown = function (ed, e) {
	let conf = ed.settings;

	if (e.ctrlKey || e.metaKey) {
		switch (e.keyCode) {
			case 67: // c
				// we disable synchronizeHot during copy/cut: onChange will set hot
				conf.weSynchronizeHot.doSyncHot = false;
				break;
			case 88: // x
				// do nothing
				break;
			case 68: // d
			case 79: // o
			case 82: // r
				//set keyCode = -1 to just let WE-keyListener cancel event
				if (conf.weEditorType !== 'inlineTrue') {
					e.keyCode = -1;
				}
				/* falls through */
			case 83: // s
				e.stopPropagation();
				e.preventDefault();
				WE().handler.keyListener.deal(e, window);
				return false;
			case 87:
				if (conf.weEditorType !== 'inlineTrue') {
					e.keyCode = -1;
				}
				/* falls through */
			default:
			//let tiny do its job
		}
	}
};

WE().layout.we_tinyMCE.do.onOpenDialog = function (ed) {
	let I18n = ed.settings.weWin.tinymce.util.I18n,
			elems;

	if ((elems = ed.settings.weWin.document.getElementsByClassName('tox-dialog__title')) && elems.length) {
		let dialog = elems[0].parentNode.parentNode;

		// add some plugin specific fixes to some dialogs
		switch (elems[0].innerHTML ? elems[0].innerHTML : 'none') {
			case I18n.translate('Insert/Edit code sample'):
				{
					let sel = ed.selection.getContent({format: 'text'}),
							textareas = dialog.getElementsByTagName('textarea');
					if (textareas.length && sel && !ed.dom.is(ed.selection.getStart(), 'pre[class*="language-"]')) {
						textareas[0].innerHTML = sel;
					}
				}
				break;
			case I18n.translate('Find and replace'):
				{
					dialog.parentNode.style = '';
					dialog.parentNode.classList = 'tox-dialog-wrap';
					let backdrop = document.createElement('div');
					backdrop.classList = 'tox-dialog-wrap__backdrop';
					dialog.parentNode.prepend(backdrop);
					dialog.style = 'position: relative; max-width: 540px;';
					dialog.classList = 'tox-dialog';
					dialog.ariaModal = 'true';
					let footerEnd = dialog.getElementsByClassName('tox-dialog__footer-end')[0];
					let btnShortClose = document.createElement('button');
					btnShortClose.innerHTML = 'shortClose';
					let icons = elems[0].parentNode.getElementsByTagName('button');
					if (icons.length && icons[0].title === I18n.translate('Close')) {
						btnShortClose.addEventListener('click', function () {
							icons[0].click();
						});
					}
					footerEnd.append(btnShortClose);
				}
				break;
			case I18n.translate('Special character'):
			case I18n.translate('Anchor'):
			case I18n.translate('Table properties'):
			case I18n.translate('Row properties'):
			case I18n.translate('Cell properties'):
				// do nothing
				break;
		}

		// adapt footer buttons to WE
		let btns = dialog.getElementsByClassName('tox-dialog__footer')[0].getElementsByTagName('button'),
				btnDiv,
				btnOK,
				btnCancel,
				btnSearch,
				btnReplace,
				btnReplaceAll;
		ed.settings.weWin.tinymce.each(btns, function (btn) {
			switch (btn.innerHTML) {
				case I18n.translate('Save'):
					btnOK = btn;
					btn.classList = 'weBtn weIconTextButton weBtnOk';
					btn.innerHTML = '<i class="fa fa-firsticon ' + WE().consts.g_l.button.icon.ok + '"></i><span class="text">' + WE().consts.g_l.button.ok + '</span>';
					btn.title = WE().consts.g_l.button.ok;
					btnDiv = btn.parentNode;
					break;
				case I18n.translate('Cancel'):
				case I18n.translate('Close'):
					btnCancel = btn;
					btn.classList = 'weBtn  weIconTextButton weBtnCancel';
					btn.innerHTML = '<i class="fa fa-firsticon fa-lg fa-close fa-cancel"></i><span class="text">' + WE().consts.g_l.button.cancel + '</span>';
					btn.title = WE().consts.g_l.button.cancel;
					btnDiv = btn.parentNode;
					break;
				case 'shortClose':
					btnCancel = btn;
					btn.classList = 'weBtn  weIconTextButton weBtnCancel';
					btn.innerHTML = '<i class="fa fa-firsticon fa-lg fa-close fa-cancel"></i>';
					btn.title = WE().consts.g_l.button.cancel;
					btnDiv = btn.parentNode;
					break;
				case I18n.translate('Find'):
					btnSearch = btn;
					btn.classList = 'weBtn  weIconTextButton';
					btn.innerHTML = '<i class="fa fa-firsticon fa-lg fa-binoculars"></i><span class="text">' + WE().consts.g_l.button.searchContinue + '</span>';
					btn.title = WE().consts.g_l.button.search;
					btnDiv = btn.parentNode;
					break;
				case I18n.translate('Replace'):
					btnReplace = btn;
					btn.classList = 'weBtn  weIconTextButton';
					btn.innerHTML = '<i class="fa fa-firsticon fa-lg fa-retweet"></i><span class="text">' + WE().consts.g_l.button.replace + '</span>';
					btn.title = WE().consts.g_l.button.replace;
					btnDiv = btn.parentNode;
					break;
				case I18n.translate('Replace all'):
					btnReplaceAll = btn;
					btn.classList = 'weBtn  weIconTextButton';
					btn.innerHTML = '<i class="fa fa-firsticon fa-lg fa-database"></i><i class="fa fa-firsticon fa-lg fa-retweet"></i><span class="text">' + WE().consts.g_l.button.replaceAll + '</span>';
					btn.title = WE().consts.g_l.button.replaceAll;
					btnDiv = btn.parentNode;
					break;
			}
		});

		btnDiv.innerHTML = '';
		if (btnSearch) {
			btnDiv.appendChild(btnSearch);
		}
		if (btnReplace) {
			btnDiv.appendChild(btnReplace);
		}
		if (btnReplaceAll) {
			btnDiv.appendChild(btnReplaceAll);
		}
		if (btnOK) {
			btnDiv.appendChild(btnOK);
		}
		if (btnCancel) {
			btnDiv.appendChild(btnCancel);
		}
	}
};

WE().layout.we_tinyMCE.do.onDblClick = function (ed) {
	let openDialogsOnDblClick = true;

	if (openDialogsOnDblClick) {
		switch (ed.selection.getNode().nodeName) {
			case 'IMG':
				if (ed.dom.getAttrib(ed.selection.getNode(), "src", "")) {
					ed.execCommand("mceWeimage");
				}
				return;
			case 'A':
				if (ed.dom.getAttrib(ed.selection.getNode(), "href", "")) {
					ed.execCommand("mceWelink");
					return;
				} else {
					ed.execCommand("mceAnchor");
				}
				return;
				/*
				 case 'ABBR':
				 ed.execCommand("mceWeabbr");
				 return;
				 case 'ACRONYM':
				 ed.execCommand("mceWeacronym");
				 return;
				 case 'SPAN':
				 if (ed.selection.getNode().getAttribute('lang') && ed.selection.getNode().getAttribute('lang') !== '') {
				 ed.execCommand("mceWelang");
				 }
				 return;
				 case 'CITE':
				 ed.execCommand("mceCite");
				 return;
				 case 'DEL':
				 ed.execCommand("mceDel");
				 return;
				 case 'INS':
				 ed.execCommand("mceIns");
				 return;
				 */
		}

		return;
	}

	let frameControler = WE().layout.weEditorFrameController;
	if (!frameControler) {
		return;
	}
};

WE().layout.we_tinyMCE.do.onNodeChange = function (ed, e) {
	if (ed.settings.wevisualaidIsVisualTmp) {
		ed.settings.wevisualaidIsVisualTmp = false;
		ed.execCommand("mceWevisualaid", true);
	}

	/*
	 var pc, tmp;
	 var td = ed.dom.getParent(n, 'td');

	 if (typeof td === 'object' && td && td.getElementsByTagName('p').length === 1) {
	 pc = td.getElementsByTagName('p')[0].cloneNode(true);
	 tmp = document.createElement('div');
	 tmp.appendChild(pc);

	 if (tmp.innerHTML === td.innerHTML) {
	 td.innerHTML = '';
	 ed.selection.setContent(pc.innerHTML);
	 }
	 }
	 */
};

/*WE().layout.we_tinyMCE.do.onObjectSelected = function (ed, e) {
 if(e.target && e.target.nodeName === "TABLE"){
 e.target.classList.add('mce-item-table');
 }
 };

 WE().layout.we_tinyMCE.do.onPostRender = function (ed, cm) {
 // move this to setup!
 };
 */
WE().layout.we_tinyMCE.do.onPostProcess = function (ed, o) {
	let c = document.createElement("div");
	c.innerHTML = o.content;
	let first = c.firstChild;

	ed.settings.weSynchronizeHot.isEditorHot = false;

	// 20180812: WORKS
	if (first) {
		if (first.innerHTML === '&nbsp;' && first === c.lastChild) {
			c.innerHTML = '';
		} else if (ed.settings.weRemoveFirstParagraph && first.nodeName === 'P') {
			let useDiv = false,
					div = document.createElement('div'),
					attribs = ['style', 'class', 'dir'];

			div.innerHTML = first.innerHTML;

			for (let i = 0; i < attribs.length; i++) {
				if (first.hasAttribute(attribs[i])) {
					div.setAttribute(attribs[i], first.getAttribute(attribs[i]));
					useDiv = true;
				}
			}
			if (useDiv) {
				c.replaceChild(div, first);
			} else {
				c.removeChild(first);
				c.innerHTML = first.innerHTML + c.innerHTML;
			}
		}
	}

	// remove border="0" and border="" from table tags
	let tables = c.getElementsByTagName('TABLE');
	if (tables) {
		for (let i = 0; i < tables.length; i++) {
			if (tables[i].getAttribute('border') === '0' || tables[i].getAttribute('border') === '') {
				tables[i].removeAttribute('border');
			}
			//tables[i].setAttribute('data-wetiny', 1);
		}
		let tds = c.getElementsByTagName('TD');
		if (tds) {
			for (let i = 0; i < tds.length; i++) {
				if (tds[i].innerHTML === '&nbsp;') {
					tds[i].innerHTML = '';
				}
			}
		}
	}

	// write content back
	o.content = c.innerHTML;
};

WE().layout.we_tinyMCE.do.onClickMaximizeTinyDialog = function (ed, win) {
	let rect,
			elem;

	if (!win.weMaximize.isMaximized) {
		elem = ed.settings.weWin.document.documentElement;
		rect = win._lastRepaintRect;
		win.weMaximize = {origRect: {x: rect.x, y: rect.y, w: rect.w, h: rect.h}, isMaximized: true};
		win.moveTo(0, 0).resizeTo(elem.clientWidth, elem.clientHeight - 2);
	} else {
		win.weMaximize.isMaximized = false;
		rect = win.weMaximize.origRect;
		win.moveTo(rect.x, rect.y).resizeTo(rect.w, rect.h);
	}
};

WE().layout.we_tinyMCE.do.onDrop = function (e, ed) {
	if (e.dataTransfer && e.dataTransfer.getData('text')) {
		let data = e.dataTransfer.getData('text').split(',');

		// dragging from WE (when permitted) comes with transfer text starting with "dragItem": we handle it
		if (data[0] && data[0] === 'dragItem' && data[1] === WE().consts.tables.FILE_TABLE) {
			e.preventDefault();
			e.stopPropagation();
			if (data[3] === WE().consts.contentTypes.IMAGE) {
				ed.execCommand('mceWeimage', true, data[2]);
			} else {
				ed.execCommand('mceWelink', true, data[2]);
			}
			return false;
		}

		// dragging inside tiny comes with transfer text not starting with "dragItem": let tiny handle it
		return false;
	}

	// dragging images from os comes witout transfer text: we prevent it!
	e.preventDefault();
	e.stopPropagation();
	return false;
};

WE().layout.we_tinyMCE.do.onCopyCut = function (ed, isCut) {
	let
			selection = ed.getWin().getSelection(),
			tmpDiv = top.document.createElement('div'),
			table = ed.dom.getParent(ed.selection.getNode(), 'TABLE');

	if (table) {
		let clone;
		if (ed.selection.isCollapsed()) {
			clone = isCut ? table : table.cloneNode();
			clone.innerHTML = table.innerHTML;
			if (clone.dataset) {
				for (let attrib in clone.dataset) {
					delete clone.dataset[attrib];
				}
			}
			tmpDiv.appendChild(clone);
		} else {
			if (table.querySelectorAll("td[data-mce-selected='1']").length > 1) {
				return;
			}
			clone = ed.selection.getNode().cloneNode();
			clone.innerHTML = ed.selection.getContent({format: 'html'});
		}
		tmpDiv.appendChild(clone);
	} else {
		tmpDiv.appendChild(selection.getRangeAt(0).cloneContents());
	}

	if (tmpDiv.firstElementChild !== null) {
		tmpDiv.firstElementChild.setAttribute('data-wetiny', '1');

		let secondTag;
		// removefirstparagraph will remove first <P> before the selection is written to clipboard: we must mark a second element thus
		if ((secondTag = tmpDiv.firstElementChild.firstElementChild) || (secondTag = tmpDiv.firstElementChild.nextElementSibling)) {
			secondTag.setAttribute('data-wetiny', '1');
		}

		// mark tmpDiv too: in FF tmpDiv is written to clipboard when using selection.selectAllChildren(tmpDiv)!
		tmpDiv.setAttribute('name', 'we-tiny-tmpDiv');// use conform name!

		// to get marked content into the clipboard, we must
		// - set tiny-"bookmark" to original selection (i.e. set two spans as start- and end-marker around selection)
		// - append tmpDiv to tiny content
		// - select its innerHTML,
		// - remove tmpDiv, and
		// - reselect original selection using bookmark

		WE().layout.we_tinyMCE.vars.bm = ed.selection.getBookmark();
		tmpDiv.style.position = 'absolute';
		tmpDiv.style.left = '-99999px';
		ed.getBody().appendChild(tmpDiv);
		selection.selectAllChildren(tmpDiv);

		ed.getWin().setTimeout(function () { // FIXME: make named fnc!
			ed.getBody().removeChild(tmpDiv);
			ed.selection.moveToBookmark(WE().layout.we_tinyMCE.vars.bm);
			WE().layout.we_tinyMCE.vars.bm = null;
			tmpDiv = null;
			ed.settings.weSynchronizeHot.doSyncHot = true;
			if (isCut) {
				ed.selection.setContent('');
			}
			// we mus repaint tiny-path too
		}, 100);
	}
};

WE().layout.we_tinyMCE.functions.initEditor = function (win, rawConfObj) {
	if (typeof win.tinymce === 'object' && typeof rawConfObj === 'object') {

		let elem = win.document.getElementsByClassName(rawConfObj.weSelectorClass)[0];
		rawConfObj.size = {width: elem.offsetWidth, height: elem.offsetHeight};

		let confObj = WE().layout.we_tinyMCE.getTinyConfObject(rawConfObj);

		if (!win.tinyMceTranslationObjectAdded) {
//			win.tinymce.addI18n(rawConfObj.language, WE().consts.g_l.tinyMceTranslationObject[rawConfObj.language]);
			win.tinyMceTranslationObjectAdded = true;
		}

		if (win.document.compatMode !== 'CSS1Compat') {
			let el = win.document.querySelector(confObj.selector);
			if (el) {
				let
						width = el.offsetWidth,
						height = el.offsetHeight;
				el.removeAttribute('style');
				el.readOnly = true;
				el.classList.add('we_tiny_warning');
				let warning = document.createElement('div');
				warning.classList.add('we_tiny_warning');
				warning.style.width = width + 'px';
				warning.innerHTML = WE().consts.g_l.tinyMceTranslationObject.doctypeAlert;
				el.parentNode.insertBefore(warning, el);
				el.style.width = width + 'px';
				el.style.height = Math.max((height - warning.offsetHeight), 140) + 'px';
				return;
			}
		}

		confObj.weWin = win;
		/*
		 if (win.tinymce.get(confObj.elements)) { // true when we reinitialize tiny instance
		 win.tinymce.execCommand('mceRemoveControl', false, confObj.elements);
		 }
		 */

		win.tinymce.init(confObj);
	}
};

WE().layout.we_tinyMCE.functions.initAllFromDataAttribute = function (win) {
	let
			rawConfObj,
			dialogProps = WE().util.getDynamicVar(win.document, 'loadVar_tinyConfigs', 'data-dialogProperties');

	if (dialogProps && dialogProps.isDialog) {
		// WE().util.dialogTop(win).opener.tinyMceRawConfigurations[dialogProps.weFieldname];
		rawConfObj = win.opener.tinyMceRawConfigurations[dialogProps.weFieldname];
		rawConfObj.weEditorType = dialogProps.weEditorType;
		WE().layout.we_tinyMCE.functions.initEditor(win, rawConfObj);

		return;
	}

	let
			instances = WE().util.getDynamicVar(win.document, 'loadVar_tinyConfigs', 'data-instances'),
			configurations = WE().util.getDynamicVar(win.document, 'loadVar_tinyConfigs', 'data-configurations');

	if (instances && instances.length) {
		for (let i = 0; i < instances.length; i++) {
			rawConfObj = Object.assign({}, instances[i], configurations[instances[i].weConfiguration]);
			win.tinyMceRawConfigurations[rawConfObj.weFieldName] = rawConfObj;

			if (rawConfObj.weEditorType === 'inlineTrue') {
				WE().layout.we_tinyMCE.functions.initEditor(win, rawConfObj);
			} else {
				// we register weName of editors type = inlineFalse, so tinyWrapper can find their previewDiv even before the popup is setup
				win.tinyEditors[rawConfObj.weFieldName] = rawConfObj.weName;
			}

		}
	}
};

WE().layout.we_tinyMCE.functions.setHot = function (ed) {
	let conf = ed.settings;
	if (conf.weSynchronizeHot.isEditorHot || !conf.weSynchronizeHot.doSyncHot || conf.weIsFrontend) {
		return;
	}

	switch (ed.settings.weEditorType) {
		case 'inlineFalse':
			try {
				ed.settings.weWin.opener.we_cmd('setHot');
			} catch (e) {
				return;
			}
			break;
		case 'inlineTrue':
		try {
			ed.settings.weWin.we_cmd('setHot');
		} catch (e) {
			return;
		}
	}
	conf.weSynchronizeHot.isEditorHot = true;
};

WE().layout.we_tinyMCE.functions.resizeEditor = function (ed, render) {
	switch (ed.settings.weEditorType) {
		case 'inlineFalse': // maximize editor to fit overlay
			{
				let dom = ed.dom;
				if (render) {
					let
							editorContainer = ed.getContainer(),
							conf = ed.settings;

					editorContainer.style.width = editorContainer.style.height = '';
					dom.addClass(conf.weWin.document.body, 'tox-fullscreen');
					dom.addClass(conf.weWin.document.documentElement, 'tox-fullscreen');
					dom.addClass(editorContainer, 'tox-fullscreen');
					editorContainer.style.position = 'relative';
					conf.weEditarea = editorContainer.getElementsByClassName('tox-edit-area')[0];
				}
				ed.dom.setStyle(ed.getContentAreaContainer().firstChild, 'height', ed.settings.weEditarea.clientHeight);
				ed.weControlManager.setActive('wefullscreen', WE().util.dialogTop(ed.settings.weWin).overlay.maximized);
			}
			break;
		default:
		//
	}
};

WE().layout.we_tinyMCE.functions.customNodeFilter_A = function (nodes) {
	for (let i = 0; i < nodes.length; i++) {
		if (!nodes[i].attr("href") && !nodes[i].attr("id")) {
			nodes[i].attr("id", nodes[i].attr("name"));
		}
	}
};

WE().layout.we_tinyMCE.functions.getDocumentCss = function (win, preKomma) {
	if (win.wysiwyg_documentCss) {
		return win.wysiwyg_documentCss === -1 ? '' : (preKomma ? ',' : '') + win.wysiwyg_documentCss;
	}

	let
			doc = win.document,
			stylesheets = [];
	if (doc.styleSheets) {
		for (let i = 0; i < doc.styleSheets.length; i++) {
			if (doc.styleSheets[i].href && doc.styleSheets[i].href.indexOf("&wysiwyg=0") === -1 && doc.styleSheets[i].href.indexOf("?wysiwyg=0") === -1 && !doc.styleSheets[i].href.match(/webEdition\//) && (doc.styleSheets[i].media.length === 0 || doc.styleSheets[i].media.mediaText.indexOf("all") >= 0 || doc.styleSheets[i].media.mediaText.indexOf("screen") >= 0)) {
				stylesheets.push(doc.styleSheets[i].href);
			}
		}
	}
	win.wysiwyg_documentCss = stylesheets.length ? stylesheets.join(',') : -1;

	return win.wysiwyg_documentCss !== -1 ? ((preKomma ? ',' : '') + win.wysiwyg_documentCss) : '';
};

WE().layout.we_tinyMCE.setup.setSpellcheckAndLang = function (ed, val) {

	let body = ed.getBody();
	if (!body) {
		return;
	}
	if (val) {
		body.setAttribute('spellcheck', true);
		body.setAttribute('lang', ed.settings.weSpellcheckLanguage ? ed.settings.weSpellcheckLanguage : 'de');
	} else {
		body.removeAttribute('spellcheck');
		body.removeAttribute('lang');
	}
};

WE().layout.we_tinyMCE.setup.setContentCss = function (ed) { // called on init
	let conf = ed.settings;

	conf.content_css = conf.weContentCssParts.start + // fa + contentDyn.php
			// check if our tiny should ignore all other Document CSS
					(conf.weContentCssParts.ignoreDocumentCss ? '' : WE().layout.we_tinyMCE.functions.getDocumentCss((conf.weEditorType === 'inlineTrue' ? conf.weWin : WE().util.dialogTop(conf.weWin).opener), true)) +
					(conf.weContentCssParts.end ? ',' + conf.weContentCssParts.end : '');

			let
					cssArr = conf.content_css.split(','),
					tmpArr = [];
			for (let i = 0; i < cssArr.length; i++) {
				if (cssArr[i].indexOf('/webEdition') === 0) {
					continue;
				}
				tmpArr.push(cssArr[i]);
			}

			// FIXME: doesn't work if more than on file...
			conf.importcss_file_filter = tmpArr.length ? tmpArr.join(',') : function () {
				return false;
			};
		};

		WE().layout.we_tinyMCE.functions.setToolbarRows = function (ed) {
			let conf = ed.settings;

			for (let i = 0; i < conf.weToolbarRows.length; i++) {
				conf[conf.weToolbarRows[i].name] = conf.weToolbarRows[i].value;
			}
		};

		WE().layout.we_tinyMCE.setup.setTinyParams = function (ed) {
			let conf = ed.settings,
					value = '';

			for (let i = 0; i < conf.weTinyParams.length; i++) {
				try {
					value = JSON.parse(conf.weTinyParams[i].value);
				} catch (e) {
					value = conf.weTinyParams[i].value;
				}
				conf[conf.weTinyParams[i].name] = value;
			}
		};

		WE().layout.we_tinyMCE.functions.wysiwygDialog_setContent = function (ed) {
			let conf = ed.settings;
			if (conf.weEditorType === 'inlineTrue') {
				return;
			}

			let openerDocument = conf.weWin.opener.document;

			switch (conf.weEditorType) {
				case 'inlineFalse': // TODO: use WE() for consts
					try {
						ed.setContent(openerDocument.getElementById(conf.weName).value, {customLoadContent: true});
					} catch (e) {
					}
					break;
			}
		};

		WE().layout.we_tinyMCE.functions.callCustomInit = function (ed) {
			let
					conf = ed.settings,
					weDoc_window;

			switch (conf.weEditorType) {
				case 'inlineTrue':
					weDoc_window = conf.weWin;
					weDoc_window.tinyEditors[conf.weFieldName] = ed;
					break;
				case 'inlineFalse':
					weDoc_window = conf.weWin.opener;
					weDoc_window.tinyEditorsInlineFalse[conf.weFieldName] = ed;
					break;
				default:
					return;
			}

			if (typeof weDoc_window['we_tinyMCE_' + conf.weFieldNameClean + '_init'] === 'function') {
				try {
					weDoc_window['we_tinyMCE_' + conf.weFieldNameClean + '_init'](ed);
				} catch (e) {
				}
			}
		};

		WE().layout.we_tinyMCE.functions.open_dialog_wysiwyg = function (caller, isFrontend, dialogProperties, w, h) {
			if (!isFrontend && WE().layout.weEditorFrameController.getActiveDocumentReference()) {
				try {
					WE().layout.weEditorFrameController.getActiveDocumentReference().openedWithWE = false;
				} catch (e) {
				}
			}

			if (window.screen) {
				let screen_height = ((screen.height - 50) > screen.availHeight) ? screen.height - 50 : screen.availHeight;
				screen_height = screen_height - 100;
				//var screen_width = screen.availWidth - 10;
			}

			let url = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd_frontend.php?we_cmd[0]=open_dialog_wysiwyg&we_dialog_args[dialogProperties]=' + dialogProperties;
			(new (WE().layout.overlay)(caller, 'self', url, "we_wysiwygWin", w, h)).setResizeable().open();
		};

		WE().layout.we_tinyMCE.functions.openWePlugin = function (ed, args, params) {
			let opener = ed.settings.weWin;

			ed.editorManager.setActive(ed);

			args.url = args.file; // Legacy
			args.target = args.target ? args.target : opener;
			let overlay = new (WE().layout.overlay)(opener, args.target, args.url, args.name, args.width, args.height);
			overlay.setResizeable();
			overlay.setTitle(args.title || ' ');
			overlay.open();
			overlay.setPayload('features', (args || {}));
			overlay.setPayload('params', (params || {}));

			return overlay;
		};

		WE().layout.we_tinyMCE.functions.processStylesFromCss = function (ed, items) {
			let each = ed.settings.weWin.tinymce.each,
					selector = '',
					selArr = [],
					tag = '',
					name = '',
					tableNotation = {};

			if (ed.plugins.importcss) {
				if (!items || !items.length) {
					return;
				}
				each(items, function (item) {
					ed.settings.weProcessedFormats.byFormat[item.format] = item.title;
					ed.settings.weProcessedFormats.byName[item.title] = item.format;
					ed.settings.weProcessedFormats.formats.push(item.format);

					if ((selector = item.title) && selector.indexOf('.mce-') === -1) {
						selArr = selector.split('.');
						if (selArr.length === 2 && selArr[1]) {
							tag = selArr[0] ? selArr[0] : 'noselector';
							name = selArr[1];
						} else {
							tag = 'noselector';
							name = item.title;
						}
						ed.settings.weProcessedStyles[tag] = ed.settings.weProcessedStyles[tag] ? ed.settings.weProcessedStyles[tag] : [];
						ed.settings.weProcessedStyles[tag].push({'title': selector, 'classes': name});
						tableNotation[tag] = tableNotation[tag] ? tableNotation[tag] : [];
						tableNotation[tag].push({'title': name, 'value': name});
					}
				});
			} else {
				each(ed.settings.weSelects.styleselect.formatted, function (format) {
					tag = format.selector || 'noselector';
					ed.settings.weProcessedStyles[tag] = ed.settings.weProcessedStyles[tag] ? ed.settings.weProcessedStyles[tag] : [];
					ed.settings.weProcessedStyles[tag].push({title: format.title, 'classes': format.classes/*[0]*/}); // IMI: experimental
					tableNotation[tag] = tableNotation[tag] ? tableNotation[tag] : [];
					tableNotation[tag].push({'title': name, 'value': name});
				});
			}

			ed.settings.table_class_list = tableNotation.table ? tableNotation.table : ed.settings.table_class_list;
			ed.settings.table_row_class_list = tableNotation.tr ? tableNotation.tr : ed.settings.table_row_class_list;
			ed.settings.table_cell_class_list = tableNotation.td ? tableNotation.td : ed.settings.table_cell_class_list;

			ed.settings.weProcessedStyles.processed = true;
		};

		WE().layout.we_tinyMCE.functions.getStylesByTag = function (ed, tag) {
			if (!ed.settings.weProcessedStyles.processed) {
				WE().layout.we_tinyMCE.functions.processStylesFromCss(ed);
			}
			let formats = ed.settings.weProcessedStyles[tag] ? ed.settings.weProcessedStyles[tag] : [];

			return formats.concat(ed.settings.weProcessedStyles.noselector ? ed.settings.weProcessedStyles.noselector : []);
		};

		WE().layout.we_tinyMCE.functions.maxlenSetValidity = function (ed, setInvalid) {
			// set Validity here. If Validity is not empty, form will fail to submit
			ed.getElement().setCustomValidity(setInvalid ? WE().consts.g_l.tinyMceTranslationObject.tooLongBoth : '');
			if (setInvalid) {
				ed.getBody().classList.add('we_invalid');
			} else {
				ed.getBody().classList.remove('we_invalid');
			}
		};


		WE().layout.we_tinyMCE.functions.registerLenCheck = function () {
			clearTimeout(WE().layout.we_tinyMCE.vars.timeouts.maxLen);
			//console.log(this);
			WE().layout.we_tinyMCE.vars.timeouts.maxLen = setTimeout(WE().layout.we_tinyMCE.functions.checkLen, 500, this);
		};

		WE().layout.we_tinyMCE.functions.changeCheckLen = function () {
			clearTimeout(WE().layout.we_tinyMCE.vars.timeouts.reportValidity);
			// just check for valid input
			// arrow syntax for this binding
			// we need delay, the click will cancel or input error message
			// fixme: add changeChecklen???
			WE().layout.we_tinyMCE.vars.timeouts.reportValidity = setTimeout(() => {
				this.getElement().reportValidity();
			}, 100);
		};


		WE().layout.we_tinyMCE.functions.checkLen = function (ed) {

			let maxlen = ed.settings.weMaxlen,
					tooLongHtml = false,
					tooLongText = false;

			tooLongHtml = maxlen.len.html ? ed.getContent({format: 'html'}).length > maxlen.len.html : false;
			tooLongText = maxlen.len.text ? ed.getContent({format: 'text'}).length > maxlen.len.text : false;

			let text = '';
			switch ((tooLongHtml ? (tooLongText ? 'both' : 'html') : (tooLongText ? 'text' : ''))) {
				case 'both':
					text = WE().consts.g_l.tinyMceTranslationObject.tooLongBoth;
					break;
				case 'text':
					text = WE().consts.g_l.tinyMceTranslationObject.tooLongVisible.replace('%d', maxlen.len.text);
					break;
				case 'html':
					text = WE().consts.g_l.tinyMceTranslationObject.tooLongHTML.replace('%d', maxlen.len.html);
					break;
				default:
					WE().layout.we_tinyMCE.functions.maxlenSetValidity(ed, false);
					// check succeeded
					return true;
			}
			WE().layout.we_tinyMCE.functions.maxlenSetValidity(ed, true);
			// input is invalid, check failed
			return false;
		};

		/**
		 * If successful, writes data from tiny. If checkLen fails, displays an error message and does not write tiny data.
		 * @param {type} ed
		 * @param {type} elementName
		 * @returns {undefined} */
		WE().layout.we_tinyMCE.functions.submitPopup = function (ed, elementName) {
			if (!WE().layout.we_tinyMCE.functions.checkLen(ed, true)) {
				WE().layout.we_tinyMCE.functions.invalidatePopupTA();
				// do not write tiny information
				WE().util.showMessage(WE().consts.g_l.tinyMceTranslationObject.tooLongBoth);
				return;
			}
			let form = ed.formElement,
					// if no Element set, use dummy object
					element = elementName ? form.elements["we_dialog_args[" + elementName + "]"] : {};
			element.value = 1;
			form.submit();
			element.value = 0;
		};

		WE().layout.we_tinyMCE.functions.resetPopupTA = function () {
			// restore hidden popup validity
			let areas = WE().layout.weEditorFrameController.getActiveEditorFrame().getContentEditor().document.getElementsByClassName("wetextarea popup");
			for (let area of areas) {
				area.setCustomValidity('');
			}
		};

		WE().layout.we_tinyMCE.functions.invalidatePopupTA = function () {
				// invalidate hidden textareas in opener
				let areas = WE().layout.weEditorFrameController.getActiveEditorFrame().getContentEditor().document.getElementsByClassName("wetextarea popup");
				for (let area of areas) {
					area.setCustomValidity('err');
				}
		};

		WE().layout.we_tinyMCE.setup.addListeners = function (ed) {
			ed.on('Init', WE().layout.we_tinyMCE.do.onInit.bind(this, ed));
			ed.on('DblClick', WE().layout.we_tinyMCE.do.onDblClick.bind(this, ed));
			ed.on('OpenWindow', WE().layout.we_tinyMCE.do.onOpenDialog.bind(this, ed));
			ed.on('KeyDown', function (e) {
				WE().layout.we_tinyMCE.do.onKeyDown(ed, e);
			});
			ed.on('PostProcess', function (e) {
				WE().layout.we_tinyMCE.do.onPostProcess(ed, e);
			});
			ed.on('NodeChange', function (e) {
				WE().layout.we_tinyMCE.do.onNodeChange(ed, e);
			});
			ed.on('Change', function (e) {
				WE().layout.we_tinyMCE.do.onChange(ed);
			});

			if (ed.settings.weMaxlen.len.html || ed.settings.weMaxlen.len.text) {
				// during events fired by tiny editor, THIS will be the the calling editor
				// tiny does not handle paste as input...
				ed.on('paste', WE().layout.we_tinyMCE.functions.registerLenCheck);
				ed.on('input', WE().layout.we_tinyMCE.functions.registerLenCheck);
				// check on change
				ed.on('change', WE().layout.we_tinyMCE.functions.changeCheckLen);
			}
		};

		WE().layout.we_tinyMCE.setup.addIcons = function (ed) {
			ed.settings.weWin.tinymce.IconManager.add('weicons', {
				icons: WE().layout.we_tinyMCE.vars.weicons
			});
		};

		WE().layout.we_tinyMCE.setup.addImageQuickbar = function (ed) {//tablemergecells
			ed.ui.registry.addContextToolbar('imagequickbar', {
				predicate: function (node) {
					return node.nodeName.toLowerCase() === 'img';
				},
				items: 'weimage_quickbar | welink weunlink',
				position: 'node',
				scope: 'node'
			});

			ed.ui.registry.addContextToolbar('tableselectedcells', {
				predicate: function (node) {
					let table = ed.dom.getParent(ed.selection.getNode(), 'TABLE');
					return table && table.querySelectorAll("td[data-mce-selected='1']").length > 1;
				},
				items: 'tablemergecells tablecellprops',
				position: 'selection',
				scope: 'node'
			});
		};

		WE().layout.we_tinyMCE.setup.addControlManager = function (ed) {
			ed.weControlManager = {
				registry: {button: {}, menuitem: {}, context: {}},
				register: function (type, name, elem) {
					if (type && name && elem) {
						this.registry[type][name] = elem;
					}
				},
				get: function (cmd, only) {
					if (only) {
						return this.registry[only] && this.registry[only][cmd] ? this.registry[only][cmd] : null;
					}
					return {
						button: (this.registry.button[cmd] ? this.registry.button[cmd] : null),
						menuitem: (this.registry.menuitem[cmd] ? this.registry.menuitem[cmd] : null),
						context: (this.registry.context[cmd] ? this.registry.context[cmd] : null)
					};
				},

				setActive: function (cmd, active, only) {
					active = active !== false;
					if (only) {
						if (this.registry[only] && this.registry[only][cmd]) {
							this.registry[only][cmd].setActive(active);
						}
						return;
					}
					if (this.registry.button[cmd]) {
						this.registry.button[cmd].setActive(active);
					}
					if (this.registry.menuitem[cmd]) {
						this.registry.menuitem[cmd].setActive(active);
					}
					if (this.registry.context[cmd]) {
						this.registry.context[cmd].setActive(active);
					}
				},
				setDisabled: function (cmd, disabled, only) {
					disabled = disabled !== false;
					if (only) {
						if (this.registry[only] && this.registry[only][cmd]) {
							this.registry[only][cmd].setDisabled(disabled);
						}
						return;
					}
					if (this.registry.button[cmd]) {
						this.registry.button[cmd].setDisabled(disabled);
					}
					if (this.registry.menuitem[cmd]) {
						this.registry.menuitem[cmd].setDisabled(disabled);
					}
					if (this.registry.context[cmd]) {
						this.registry.context[cmd].setDisabled(disabled);
					}
				}
			};
		};
